@@ -21,41 +21,15 @@ using namespace lime;
 
 #define dirName ((direction == SOAPY_SDR_RX)?"Rx":"Tx")
 
-//lazy fix for the const call issue -- FIXME
-#define _accessMutex const_cast<std::recursive_mutex &>(_accessMutex)
-
 // arbitrary upper limit for CGEN automatic tune
+#define MIN_CGEN_RATE 6.4e6
 #define MAX_CGEN_RATE 640e6
+#define CGEN_DEADZONE_LO 450e6
+#define CGEN_DEADZONE_HI 491.5e6
 
 //reasonable limits when advertising the rate
 #define MIN_SAMP_RATE 1e5
-#define MAX_SAMP_RATE 60e6
-
-/*******************************************************************
- * Special LMS7002M with log forwarding
- ******************************************************************/
-class LMS7002M_withLogging : public LMS7002M
-{
-public:
-    LMS7002M_withLogging(void):
-        LMS7002M()
-    {
-        //SoapySDR::setLogLevel(SOAPY_SDR_DEBUG);
-        return;
-    }
-
-protected:
-    void Log(const char* text, LogType type)
-    {
-        switch(type)
-        {
-        case LOG_INFO: SoapySDR::log(SOAPY_SDR_INFO, text); break;
-        case LOG_WARNING: SoapySDR::log(SOAPY_SDR_WARNING, text); break;
-        case LOG_ERROR: SoapySDR::log(SOAPY_SDR_ERROR, text); break;
-        case LOG_DATA: SoapySDR::log(SOAPY_SDR_DEBUG, text); break;
-        }
-    }
-};
+#define MAX_SAMP_RATE 65e6
 
 /*******************************************************************
  * Constructor/destructor
@@ -78,14 +52,11 @@ SoapyLMS7::SoapyLMS7(const ConnectionHandle &handle, const SoapySDR::Kwargs &arg
     SoapySDR::logf(SOAPY_SDR_INFO, "Device name: %s", devInfo.deviceName.c_str());
     SoapySDR::logf(SOAPY_SDR_INFO, "Reference: %g MHz", _conn->GetReferenceClockRate()/1e6);
 
-    //disable cal hooks during setup
-    std::map<size_t, std::shared_ptr<LMS7002M_SelfCalState>> calStates;
-
     //LMS7002M driver for each RFIC
     for (size_t i = 0; i < numRFICs; i++)
     {
         SoapySDR::logf(SOAPY_SDR_INFO, "Init LMS7002M(%d)", int(i));
-        _rfics.push_back(new LMS7002M_withLogging());
+        _rfics.push_back(new LMS7002M());
         _rfics.back()->SetConnection(_conn, i);
         SoapySDR::logf(SOAPY_SDR_INFO, "Ver=%d, Rev=%d, Mask=%d",
             _rfics.back()->Get_SPI_Reg_bits(LMS7param(VER), true),
@@ -102,8 +73,6 @@ SoapyLMS7::SoapyLMS7(const ConnectionHandle &handle, const SoapySDR::Kwargs &arg
 
         st = _rfics.back()->UploadAll();
         if (st != 0) throw std::runtime_error("UploadAll() failed");
-
-        calStates[i].reset(new LMS7002M_SelfCalState(_rfics.back()));
     }
 
     //enable all channels
@@ -145,9 +114,10 @@ SoapyLMS7::SoapyLMS7(const ConnectionHandle &handle, const SoapySDR::Kwargs &arg
     }
 
     //reset flags for user calls
-    _fixedClockRate = false;
+    _fixedClockRate = args.count("clock") != 0;
     _fixedRxSampRate.clear();
     _fixedTxSampRate.clear();
+    _channelsToCal.clear();
 }
 
 SoapyLMS7::~SoapyLMS7(void)
@@ -247,7 +217,7 @@ std::vector<std::string> SoapyLMS7::listAntennas(const int direction, const size
 void SoapyLMS7::setAntenna(const int direction, const size_t channel, const std::string &name)
 {
     std::unique_lock<std::recursive_mutex> lock(_accessMutex);
-    SoapySDR::logf(SOAPY_SDR_INFO, "SoapyLMS7::setAntenna(%s, %d, %s)", dirName, int(channel), name.c_str());
+    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapyLMS7::setAntenna(%s, %d, %s)", dirName, int(channel), name.c_str());
     auto rfic = getRFIC(channel);
 
     if (direction == SOAPY_SDR_RX)
@@ -274,6 +244,8 @@ void SoapyLMS7::setAntenna(const int direction, const size_t channel, const std:
 
         rfic->SetBandTRF(band);
     }
+
+    _channelsToCal.emplace(direction, channel);
 }
 
 std::string SoapyLMS7::getAntenna(const int direction, const size_t channel) const
@@ -405,10 +377,31 @@ std::vector<std::string> SoapyLMS7::listGains(const int direction, const size_t
     return gains;
 }
 
+void SoapyLMS7::setGain(const int direction, const size_t channel, const double value)
+{
+    std::unique_lock<std::recursive_mutex> lock(_accessMutex);
+
+    //Distribute Rx gain from elements in the direction of RFE to RBB
+    //This differs from the default gain distribution in that it
+    //does not scale the gain to the negative range of the PGA.
+    //This keep the PGA in mid-range unless extreme values are used.
+    if (direction == SOAPY_SDR_RX)
+    {
+        double remaining(value);
+        for (const auto &name : this->listGains(direction, channel))
+        {
+            this->setGain(direction, channel, name, remaining);
+            remaining -= this->getGain(direction, channel, name);
+        }
+    }
+
+    else SoapySDR::Device::setGain(direction, channel, value);
+}
+
 void SoapyLMS7::setGain(const int direction, const size_t channel, const std::string &name, const double value)
 {
     std::unique_lock<std::recursive_mutex> lock(_accessMutex);
-    SoapySDR::logf(SOAPY_SDR_INFO, "SoapyLMS7::setGain(%s, %d, %s, %g dB)", dirName, int(channel), name.c_str(), value);
+    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapyLMS7::setGain(%s, %d, %s, %g dB)", dirName, int(channel), name.c_str(), value);
     auto rfic = getRFIC(channel);
 
     if (direction == SOAPY_SDR_RX and name == "LNA")
@@ -494,6 +487,16 @@ double SoapyLMS7::getGain(const int direction, const size_t channel, const std::
     else throw std::runtime_error("SoapyLMS7::getGain("+name+") - unknown gain name");
 }
 
+SoapySDR::Range SoapyLMS7::getGainRange(const int direction, const size_t channel) const
+{
+    if (direction == SOAPY_SDR_RX)
+    {
+        //make it so gain of 0.0 sets PGA at its mid-range
+        return SoapySDR::Range(-12.0, 19.0+12.0+30.0);
+    }
+    return SoapySDR::Device::getGainRange(direction, channel);
+}
+
 SoapySDR::Range SoapyLMS7::getGainRange(const int direction, const size_t channel, const std::string &name) const
 {
     if (direction == SOAPY_SDR_RX and name == "LNA") return SoapySDR::Range(0.0, 30.0);
@@ -510,17 +513,7 @@ SoapySDR::Range SoapyLMS7::getGainRange(const int direction, const size_t channe
  ******************************************************************/
 SoapySDR::ArgInfoList SoapyLMS7::getFrequencyArgsInfo(const int direction, const size_t channel) const
 {
-    auto infos = SoapySDR::Device::getFrequencyArgsInfo(direction, channel);
-    {
-        SoapySDR::ArgInfo info;
-        info.key = "CORRECTIONS";
-        info.name = "Corrections";
-        info.value = "true";
-        info.description = "Automatically apply DC/IQ corrections";
-        info.type = SoapySDR::ArgInfo::BOOL;
-        infos.push_back(info);
-    }
-    return infos;
+    return SoapySDR::Device::getFrequencyArgsInfo(direction, channel);
 }
 
 void SoapyLMS7::setFrequency(const int direction, const size_t channel, const std::string &name, const double frequency, const SoapySDR::Kwargs &args)
@@ -528,7 +521,7 @@ void SoapyLMS7::setFrequency(const int direction, const size_t channel, const st
     std::unique_lock<std::recursive_mutex> lock(_accessMutex);
     auto rfic = getRFIC(channel);
     const auto lmsDir = (direction == SOAPY_SDR_TX)?LMS7002M::Tx:LMS7002M::Rx;
-    SoapySDR::logf(SOAPY_SDR_INFO, "SoapyLMS7::setFrequency(%s, %d, %s, %g MHz)", dirName, int(channel), name.c_str(), frequency/1e6);
+    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapyLMS7::setFrequency(%s, %d, %s, %g MHz)", dirName, int(channel), name.c_str(), frequency/1e6);
 
     if (name == "RF")
     {
@@ -537,37 +530,23 @@ void SoapyLMS7::setFrequency(const int direction, const size_t channel, const st
         if (targetRfFreq < 30e6) targetRfFreq = 30e6;
         if (targetRfFreq > 3.8e9) targetRfFreq = 3.8e9;
         rfic->SetFrequencySX(lmsDir, targetRfFreq);
-
-        //optional way to skip corrections (used by cal utility)
-        if (args.count("CORRECTIONS") != 0 and args.at("CORRECTIONS") == "false") return;
-
-        //apply corrections to channel A
-        rfic->SetActiveChannel(LMS7002M::ChA);
-        if (rfic->ApplyDigitalCorrections(lmsDir) != 0)
-        {
-            SoapySDR::log(SOAPY_SDR_WARNING, lime::GetLastErrorMessage());
-        }
-        //success, now channel B (ignore errors)
-        else
-        {
-            rfic->SetActiveChannel(LMS7002M::ChB);
-            rfic->ApplyDigitalCorrections(lmsDir);
-        }
-
+        _channelsToCal.emplace(direction, channel);
         return;
     }
 
     if (name == "BB")
     {
+        int pos = 0, neg = 1;
         switch (direction)
         {
         case SOAPY_SDR_RX:
+            if (rfic->Get_SPI_Reg_bits(LMS7_MASK, true) != 0) std::swap(pos, neg);
             rfic->Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_RXTSP), (frequency == 0)?1:0);
-            rfic->Modify_SPI_Reg_bits(LMS7param(CMIX_SC_RXTSP), (frequency < 0)?1:0);
+            rfic->Modify_SPI_Reg_bits(LMS7param(CMIX_SC_RXTSP), (frequency < 0)?neg:pos);
             break;
         case SOAPY_SDR_TX:
             rfic->Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_TXTSP), (frequency == 0)?1:0);
-            rfic->Modify_SPI_Reg_bits(LMS7param(CMIX_SC_TXTSP), (frequency < 0)?1:0);
+            rfic->Modify_SPI_Reg_bits(LMS7param(CMIX_SC_TXTSP), (frequency < 0)?neg:pos);
             break;
         }
         if (rfic->SetNCOFrequency(lmsDir, 0, abs(frequency)) != 0)
@@ -595,7 +574,18 @@ double SoapyLMS7::getFrequency(const int direction, const size_t channel, const
 
     if (name == "BB")
     {
-        int sign = rfic->Get_SPI_Reg_bits(lmsDir==LMS7002M::Tx?LMS7param(CMIX_SC_TXTSP):LMS7param(CMIX_SC_RXTSP)) == 0 ? 1 : -1;
+        int sign = 0;
+        int pos = 0, neg = 1;
+        switch (direction)
+        {
+        case SOAPY_SDR_RX:
+            if (rfic->Get_SPI_Reg_bits(LMS7_MASK, true) != 0) std::swap(pos, neg);
+            sign = (rfic->Get_SPI_Reg_bits(LMS7param(CMIX_SC_RXTSP)) == pos) ? 1 : -1;
+            break;
+        case SOAPY_SDR_TX:
+            sign = (rfic->Get_SPI_Reg_bits(LMS7param(CMIX_SC_TXTSP)) == pos) ? 1 : -1;
+            break;
+        }
         return rfic->GetNCOFrequency(lmsDir, 0) * sign;
     }
 
@@ -640,6 +630,13 @@ SoapySDR::RangeList SoapyLMS7::getFrequencyRange(const int direction, const size
  * Sample Rate API
  ******************************************************************/
 
+static bool cgenRatePossible(const double rate)
+{
+    if (rate > MAX_CGEN_RATE) return false;
+    if (rate > CGEN_DEADZONE_LO and rate < CGEN_DEADZONE_HI) return false; //avoid range where CGEN does not lock
+    return true;
+}
+
 static double calculateClockRate(
     const int adcFactorRx,
     const int dacFactorTx,
@@ -653,8 +650,7 @@ static double calculateClockRate(
     for (int decim = 2; decim <= 32; decim *= 2)
     {
         const double rateClock = rateRx*decim*adcFactorRx;
-        if (rateClock > MAX_CGEN_RATE) continue;
-        if (rateClock > 450e6 && rateClock < 491.5e6) continue; //avoid range where CGEN does not lock
+        if (not cgenRatePossible(rateClock)) continue;
         if (rateClock < bestClockRate) continue;
         for (int interp = 2; interp <= 32; interp *= 2)
         {
@@ -681,7 +677,7 @@ void SoapyLMS7::setSampleRate(const int direction, const size_t channel, const d
 {
     std::unique_lock<std::recursive_mutex> lock(_accessMutex);
     auto rfic = getRFIC(channel);
-    LMS7002M_SelfCalState state(rfic);
+
     const auto lmsDir = (direction == SOAPY_SDR_TX)?LMS7002M::Tx:LMS7002M::Rx;
 
     double clockRate = this->getMasterClockRate();
@@ -703,7 +699,7 @@ void SoapyLMS7::setSampleRate(const int direction, const size_t channel, const d
     const double dspRate = clockRate/dspFactor;
     const double factor = dspRate/rate;
     int intFactor = 1 << int((std::log(factor)/std::log(2.0)) + 0.5);
-    SoapySDR::logf(SOAPY_SDR_INFO, "SoapyLMS7::setSampleRate(%s, %d, %g MHz), CGEN=%g MHz, %s=%g MHz, %s=%g",
+    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapyLMS7::setSampleRate(%s, %d, %g MHz), CGEN=%g MHz, %s=%g MHz, %s=%g",
         dirName, int(channel), rate/1e6, clockRate/1e6,
         (direction == SOAPY_SDR_RX)?"ADC":"DAC", dspRate/1e6,
         (direction == SOAPY_SDR_RX)?"decim":"interp", factor);
@@ -749,7 +745,7 @@ double SoapyLMS7::getSampleRate(const int direction, const size_t channel) const
     return rfic->GetSampleRate(lmsDir, rfic->GetActiveChannel());
 }
 
-std::vector<double> SoapyLMS7::listSampleRates(const int direction, const size_t channel) const
+std::vector<double> SoapyLMS7::_getEnumeratedRates(const int direction, const size_t channel) const
 {
     std::unique_lock<std::recursive_mutex> lock(_accessMutex);
     auto rfic = getRFIC(channel);
@@ -777,6 +773,7 @@ std::vector<double> SoapyLMS7::listSampleRates(const int direction, const size_t
         for (int iTx = 32; iTx >= 2; iTx /= 2)
         {
             const double clockRate = txRate*dacFactor*iTx;
+            if (not cgenRatePossible(clockRate)) continue;
             for (int iRx = 32; iRx >= 2; iRx /= 2)
             {
                 const double rxRate = clockRate/(adcFactor*iRx);
@@ -805,18 +802,46 @@ std::vector<double> SoapyLMS7::listSampleRates(const int direction, const size_t
         }
     }
 
+    std::sort(rates.begin(), rates.end());
+    return rates;
+}
+
+std::vector<double> SoapyLMS7::listSampleRates(const int direction, const size_t channel) const
+{
+    auto rates = this->_getEnumeratedRates(direction, channel);
+
     //otherwise, the clock is the only limiting factor
     //just give a reasonable high and low
-    else
+    if (rates.empty())
     {
         rates.push_back(MIN_SAMP_RATE);
         rates.push_back(MAX_SAMP_RATE);
     }
 
-    std::sort(rates.begin(), rates.end());
     return rates;
 }
 
+SoapySDR::RangeList SoapyLMS7::getSampleRateRange(const int direction, const size_t channel) const
+{
+    SoapySDR::RangeList ranges;
+    auto rates = this->_getEnumeratedRates(direction, channel);
+
+    //no enumerated rates? full continuous range is available
+    if (rates.empty())
+    {
+        ranges.push_back(SoapySDR::Range(MIN_SAMP_RATE, CGEN_DEADZONE_LO/8));
+        ranges.push_back(SoapySDR::Range(CGEN_DEADZONE_HI/8, MAX_SAMP_RATE));
+    }
+
+    //normal list of enumerated rates becomes ranges with start == stop
+    for (const auto &rate : rates)
+    {
+        ranges.push_back(SoapySDR::Range(rate, rate));
+    }
+
+    return ranges;
+}
+
 /*******************************************************************
  * Bandwidth API
  ******************************************************************/
@@ -826,13 +851,12 @@ void SoapyLMS7::setBandwidth(const int direction, const size_t channel, const do
     if (bw == 0.0) return; //special ignore value
 
     std::unique_lock<std::recursive_mutex> lock(_accessMutex);
-    SoapySDR::logf(SOAPY_SDR_INFO, "SoapyLMS7::setBandwidth(%s, %d, %g MHz)", dirName, int(channel), bw/1e6);
+    SoapySDR::logf(SOAPY_SDR_DEBUG, "SoapyLMS7::setBandwidth(%s, %d, %g MHz)", dirName, int(channel), bw/1e6);
 
     //save dc offset mode
     auto saveDcMode = this->getDCOffsetMode(direction, channel);
 
     auto rfic = getRFIC(channel);
-    LMS7002M_SelfCalState state(rfic);
 
     _actualBw[direction][channel] = bw;
 
@@ -856,6 +880,8 @@ void SoapyLMS7::setBandwidth(const int direction, const size_t channel, const do
 
     //restore dc offset mode
     this->setDCOffsetMode(direction, channel, saveDcMode);
+
+    _channelsToCal.emplace(direction, channel);
 }
 
 double SoapyLMS7::getBandwidth(const int direction, const size_t channel) const
@@ -912,6 +938,14 @@ double SoapyLMS7::getMasterClockRate(void) const
     return _rfics.front()->GetFrequencyCGEN();
 }
 
+SoapySDR::RangeList SoapyLMS7::getMasterClockRates(void) const
+{
+    SoapySDR::RangeList r;
+    r.push_back(SoapySDR::Range(MIN_CGEN_RATE, CGEN_DEADZONE_LO));
+    r.push_back(SoapySDR::Range(CGEN_DEADZONE_HI, MAX_CGEN_RATE));
+    return r;
+}
+
 /*******************************************************************
  * Time API
  ******************************************************************/
@@ -959,6 +993,7 @@ std::vector<std::string> SoapyLMS7::listSensors(void) const
 {
     std::vector<std::string> sensors;
     sensors.push_back("clock_locked");
+    sensors.push_back("lms7_temp");
     return sensors;
 }
 
@@ -973,6 +1008,15 @@ SoapySDR::ArgInfo SoapyLMS7::getSensorInfo(const std::string &name) const
         info.value = "false";
         info.description = "CGEN clock is locked, good VCO selection.";
     }
+    else if (name == "lms7_temp")
+    {
+        info.key = "lms7_temp";
+        info.name = "LMS7 Temperature";
+        info.type = SoapySDR::ArgInfo::FLOAT;
+        info.value = "0.0";
+        info.units = "C";
+        info.description = "The temperature of the LMS7002M in degrees C.";
+    }
     return info;
 }
 
@@ -984,6 +1028,10 @@ std::string SoapyLMS7::readSensor(const std::string &name) const
     {
         return _rfics.front()->GetCGENLocked()?"true":"false";
     }
+    if (name == "lms7_temp")
+    {
+        return std::to_string(_rfics.front()->GetTemperature());
+    }
 
     throw std::runtime_error("SoapyLMS7::readSensor("+name+") - unknown sensor name");
 }
@@ -1071,38 +1119,6 @@ void SoapyLMS7::writeSetting(const std::string &key, const std::string &value)
         }
     }
 
-    else if (key == "STORE_RX_CORRECTIONS")
-    {
-        for (size_t channel = 0; channel < _rfics.size()*2; channel++)
-        {
-            this->writeSetting(SOAPY_SDR_RX, channel, "STORE_CORRECTIONS", value);
-        }
-    }
-
-    else if (key == "STORE_TX_CORRECTIONS")
-    {
-        for (size_t channel = 0; channel < _rfics.size()*2; channel++)
-        {
-            this->writeSetting(SOAPY_SDR_TX, channel, "STORE_CORRECTIONS", value);
-        }
-    }
-
-    else if (key == "APPLY_RX_CORRECTIONS")
-    {
-        for (size_t channel = 0; channel < _rfics.size()*2; channel++)
-        {
-            this->writeSetting(SOAPY_SDR_RX, channel, "APPLY_CORRECTIONS", value);
-        }
-    }
-
-    else if (key == "APPLY_TX_CORRECTIONS")
-    {
-        for (size_t channel = 0; channel < _rfics.size()*2; channel++)
-        {
-            this->writeSetting(SOAPY_SDR_TX, channel, "APPLY_CORRECTIONS", value);
-        }
-    }
-
     else if (key == "CALIBRATE_TX")
     {
         for (size_t channel = 0; channel < _rfics.size()*2; channel++)
@@ -1211,24 +1227,6 @@ SoapySDR::ArgInfoList SoapyLMS7::getSettingInfo(const int direction, const size_
         infos.push_back(info);
     }
 
-    {
-        SoapySDR::ArgInfo info;
-        info.key = "STORE_CORRECTIONS";
-        info.name = "Store corrections";
-        info.type = SoapySDR::ArgInfo::INT;
-        info.description = "Store digital corrections with the current LO frequency.";
-        infos.push_back(info);
-    }
-
-    {
-        SoapySDR::ArgInfo info;
-        info.key = "APPLY_CORRECTIONS";
-        info.name = "Apply corrections";
-        info.type = SoapySDR::ArgInfo::INT;
-        info.description = "Apply digital corrections for the current LO frequency.";
-        infos.push_back(info);
-    }
-
     return infos;
 }
 
@@ -1247,19 +1245,6 @@ void SoapyLMS7::writeSetting(const int direction, const size_t channel, const st
         rfic->LoadDC_REG_IQ(isTx, ampl, 0);
     }
 
-    else if (key == "STORE_CORRECTIONS")
-    {
-        rfic->StoreDigitalCorrections(isTx);
-    }
-
-    else if (key == "APPLY_CORRECTIONS")
-    {
-        if (rfic->ApplyDigitalCorrections(isTx) != 0)
-        {
-            throw std::runtime_error(lime::GetLastErrorMessage());
-        }
-    }
-
     else if (key == "CALIBRATE_TX")
     {
         float_type bw = std::stof(value);
@@ -1465,6 +1450,46 @@ void SoapyLMS7::writeSetting(const int direction, const size_t channel, const st
 
     else throw std::runtime_error("unknown setting key: "+key);
 }
+/*******************************************************************
+ * GPIO API
+ ******************************************************************/
+
+std::vector<std::string> SoapyLMS7::listGPIOBanks(void) const
+{
+    std::vector<std::string> banks;
+    banks.push_back("MAIN"); //just one associated with the connection
+    return banks;
+}
+
+void SoapyLMS7::writeGPIO(const std::string &, const unsigned value)
+{
+    uint8_t buffer = uint8_t(value);
+    int r = _conn->GPIOWrite(&buffer, 1);
+    if (r != 0) throw std::runtime_error("SoapyLMS7::writeGPIO() " + std::string(lime::GetLastErrorMessage()));
+}
+
+unsigned SoapyLMS7::readGPIO(const std::string &) const
+{
+    uint8_t buffer(0);
+    int r = _conn->GPIORead(&buffer, 1);
+    if (r != 0) throw std::runtime_error("SoapyLMS7::readGPIO() " + std::string(lime::GetLastErrorMessage()));
+    return unsigned(buffer);
+}
+
+void SoapyLMS7::writeGPIODir(const std::string &, const unsigned dir)
+{
+    uint8_t buffer = uint8_t(dir);
+    int r = _conn->GPIODirWrite(&buffer, 1);
+    if (r != 0) throw std::runtime_error("SoapyLMS7::writeGPIODir() " + std::string(lime::GetLastErrorMessage()));
+}
+
+unsigned SoapyLMS7::readGPIODir(const std::string &) const
+{
+    uint8_t buffer(0);
+    int r = _conn->GPIODirRead(&buffer, 1);
+    if (r != 0) throw std::runtime_error("SoapyLMS7::readGPIODir() " + std::string(lime::GetLastErrorMessage()));
+    return unsigned(buffer);
+}
 
 /*******************************************************************
  * I2C API
