@@ -170,14 +170,6 @@ struct LIME_API StreamConfig
 class LIME_API IConnection
 {
 public:
-    /*!
-     * Callback from programming processes
-     * @param bsent number of bytes transferred
-     * @param btotal total number of bytes to send
-     * @param progressMsg string describing current progress state
-     * @return 0-continue programming, 1-abort operation
-     */
-    typedef std::function<bool(int bsent, int btotal, const char* progressMsg)> ProgrammingCallback;
 
     //! IConnection constructor
     IConnection(void);
@@ -224,6 +216,9 @@ public:
      */
     virtual int TransactSPI(const int addr, const uint32_t *writeData, uint32_t *readData, const size_t size);
 
+    virtual int WriteLMS7002MSPI(const uint32_t *writeData, size_t size,unsigned periphID = 0)=0;
+    virtual int ReadLMS7002MSPI(const uint32_t *writeData, uint32_t *readData, size_t size, unsigned periphID = 0)=0;
+
     /*!
      * Write to an available I2C slave.
      * @param addr the address of the slave
@@ -274,6 +269,7 @@ public:
      * @param rxRate the baseband receive data rate in Hz (RFIC to BBIC)
      */
     virtual int UpdateExternalDataRate(const size_t channel, const double txRate, const double rxRate);
+    virtual int UpdateExternalDataRate(const size_t channel, const double txRate, const double rxRate, const double txPhase, const double rxPhase);
 
     /*!
      * Called by the LMS7002M driver before the board begins self-calibration.
@@ -402,10 +398,10 @@ public:
      * Read blocking data from the stream into the specified buffer.
      *
      * @param streamID the RX stream index number
-     * @param buffs an array of buffers pointers
+     * @param buffer an array of buffers pointers
      * @param length the number of samples per buffer
      * @param timeout_ms the timeout in milliseconds
-     * @param [out] metadata optional stream metadata
+     * @param metadata [out] optional stream metadata
      * @return the number of samples read or error code
      */
     virtual int ReadStream(const size_t streamID, void* buffer, const size_t length, const long timeout_ms, StreamMetadata &metadata);
@@ -441,13 +437,31 @@ public:
     @param chCount number of waveform channels
     @param sample_count number of samples in each channel
     @param format waveform data format
+    @param epIndex endpoint identifier
     */
-    virtual int UploadWFM(const void* const* samples, uint8_t chCount, size_t sample_count, StreamConfig::StreamDataFormat format);
+    virtual int UploadWFM(const void* const* samples, uint8_t chCount, size_t sample_count, StreamConfig::StreamDataFormat format, int epIndex);
+
+    /**	@brief Read raw stream data from device streaming port
+    @param buffer       read buffer pointer
+    @param length       number of bytes to read
+    @param epIndex      endpoint identifier?
+    @param timeout_ms   timeout in milliseconds
+    */
+    virtual int ReadRawStreamData(char* buffer, unsigned length, int epIndex, int timeout_ms = 100);
 
     /***********************************************************************
      * Programming API
      **********************************************************************/
 
+    /*!
+     * Callback from programming processes
+     * @param bsent number of bytes transferred
+     * @param btotal total number of bytes to send
+     * @param progressMsg string describing current progress state
+     * @return 0-continue programming, 1-abort operation
+     */
+    typedef std::function<bool(int bsent, int btotal, const char* progressMsg)> ProgrammingCallback;
+
     /** @brief Uploads program to selected device
         @param buffer binary program data
         @param length buffer length
@@ -462,7 +476,7 @@ public:
     virtual int ProgramWrite(const char *buffer, const size_t length, const int programmingMode, const int index, ProgrammingCallback callback = 0);
 
     /**	@brief Reads current program from selected device
-        @param destination buffer for binary program data
+        @param buffer destination for binary program data
         @param length buffer length to read
         @param index target device number
         @param callback callback for progress reporting or early termination
@@ -486,26 +500,58 @@ public:
 
         This could be a quite long operation, use callback to get progress info or to terminate early
     */
-    virtual int ProgramMCU(const uint8_t *buffer, const size_t length, const MCU_PROG_MODE mode, ProgrammingCallback callback = 0);
+    virtual int ProgramMCU(const uint8_t *buffer, const size_t length, const MCU_PROG_MODE mode, ProgrammingCallback callback = 0)=0;
+
+    /*!
+     * Download up-to-date images files and flash the board when applicable.
+     *
+     * Program update is an implementation specific function that may handle
+     * several different use cases depending upon what options are applicable:
+     *
+     * - If a board relies on certain firmware and gateware images,
+     *   these images will be downloaded if not present on the system.
+     *
+     * - If the board has a programmable flash for firmware and gateware,
+     *   then up-to-date images will be written to the flash on the board.
+     *
+     * @param download true to enable downloading missing images
+     * @param callback callback for progress reporting or early termination
+     * @return 0-success
+     */
+    virtual int ProgramUpdate(const bool download = true, ProgrammingCallback callback = 0);
 
     /***********************************************************************
      * GPIO API
      **********************************************************************/
 
     /**	@brief Writes GPIO values to device
-    @param source buffer for GPIO values LSB first, each bit sets GPIO state
+    @param buffer for source of GPIO values LSB first, each bit sets GPIO state
     @param bufLength buffer length
     @return the operation success state
     */
     virtual int GPIOWrite(const uint8_t *buffer, const size_t bufLength);
 
     /**	@brief Reads GPIO values from device
-    @param destination buffer for GPIO values LSB first, each bit represent GPIO state
+    @param buffer destination for GPIO values LSB first, each bit represent GPIO state
     @param bufLength buffer length to read
     @return the operation success state
     */
     virtual int GPIORead(uint8_t *buffer, const size_t bufLength);
 
+    /**	@brief Write GPIO direction control values to device.
+    @param buffer with GPIO direction configuration (0 input, 1 output)
+    @param bufLength buffer length
+    @return the operation success state
+    */
+    virtual int GPIODirWrite(const uint8_t *buffer, const size_t bufLength);
+
+    /**	@brief Read GPIO direction configuration from device
+    @param buffer to put GPIO direction configuration (0 input, 1 output)
+    @param bufLength buffer length to read
+    @return the operation success state
+    */
+    virtual int GPIODirRead(uint8_t *buffer, const size_t bufLength);
+
     /***********************************************************************
      * Register API
      **********************************************************************/
@@ -562,7 +608,6 @@ public:
     /** @brief Sets callback function which gets called each time data is sent or received
     */
     void SetDataLogCallback(std::function<void(bool, const unsigned char*, const unsigned int)> callback);
-
 protected:
     std::function<void(bool, const unsigned char*, const unsigned int)> callback_logData;
     bool mSystemBigEndian;
@@ -597,7 +642,7 @@ public:
         uint64_t timestamp;
     };
     IStreamChannel(){};
-    IStreamChannel(IConnection* port){};
+    IStreamChannel(IConnection* port, StreamConfig conf){};
     virtual int Start() = 0;
     virtual int Stop() = 0;
     virtual ~IStreamChannel(){};
@@ -615,7 +660,8 @@ public:
     /** @brief Returns samples from receiver FIFO
         @param samples destination array of data type used in SetupStream()
         @param count number of samples to read
-        @param
+        @param metadata [in,out?] information about the transfer
+	@param timeout_ms return error if operation does not complete in timeout_ms (milliseconds)
         @return number of samples received
     */
     virtual int Read(void* samples, const uint32_t count, Metadata* metadata, const int32_t timeout_ms = 100) = 0;
@@ -623,6 +669,8 @@ public:
     /** @brief Writes samples to transmitter FIFO
         @param samples source array of data type used in SetupStream()
         @param count number of samples to write
+	@param metadata [in,out?] information about the transfer
+	@param timeout_ms return error if operation does not complete in timeout_ms (milliseconds)
         @return number of samples transmitted
     */
     virtual int Write(const void* samples, const uint32_t count, const Metadata* metadata, const int32_t timeout_ms = 100) = 0;
