@@ -7,10 +7,10 @@
 #include "ConnectionSTREAM.h"
 #include "ErrorReporting.h"
 #include <cstring>
-#include <iostream>
 #include "Si5351C.h"
 #include "FPGA_common.h"
 #include "LMS7002M.h"
+#include "Logger.h"
 #include <ciso646>
 #include <fstream>
 #include <thread>
@@ -28,8 +28,6 @@ using namespace std;
 
 using namespace lime;
 
-const uint8_t ConnectionSTREAM::streamBulkOutAddr = 0x01;
-const uint8_t ConnectionSTREAM::streamBulkInAddr = 0x81;
 const uint8_t ConnectionSTREAM::ctrlBulkOutAddr = 0x0F;
 const uint8_t ConnectionSTREAM::ctrlBulkInAddr = 0x8F;
 
@@ -48,11 +46,13 @@ const std::set<uint8_t> ConnectionSTREAM::commandsToBulkCtrlHw2 =
     CMD_ANALOG_VAL_WR, CMD_ANALOG_VAL_RD,
     CMD_ADF4002_WR,
     CMD_LMS7002_RST,
+    CMD_GPIO_DIR_WR, CMD_GPIO_DIR_RD,
+    CMD_GPIO_WR, CMD_GPIO_RD,
 };
 
 /**	@brief Initializes port type and object necessary to communicate to usb device.
 */
-ConnectionSTREAM::ConnectionSTREAM(void *arg, const unsigned index, const int vid, const int pid)
+ConnectionSTREAM::ConnectionSTREAM(void *arg, const std::string &vidpid, const std::string &serial, const unsigned index)
 {
     bulkCtrlAvailable = false;
     bulkCtrlInProgress = false;
@@ -68,18 +68,15 @@ ConnectionSTREAM::ConnectionSTREAM(void *arg, const unsigned index, const int vi
 	OutCtrlEndPt3 = nullptr;
 	InCtrlBulkEndPt = nullptr;
 	OutCtrlBulkEndPt = nullptr;
+    for (int i = 0; i < MAX_EP_CNT; i++)
+        InEndPt[i] = OutEndPt[i] = nullptr;
+
 #else
-    dev_handle = 0;
-    devs = 0;
+    dev_handle = nullptr;
     ctx = (libusb_context *)arg;
 #endif
-    if (this->Open(index, vid, pid) != 0)
-        std::cerr << GetLastErrorMessage() << std::endl;
-
-    struct ExpectedVersion
-    {
-        int hw, fw, gw, gw_rev;
-    };
+    if (this->Open(vidpid, serial, index) != 0)
+        lime::error(GetLastErrorMessage());
 
     commandsToBulkCtrl = commandsToBulkCtrlHw2;
 
@@ -90,75 +87,12 @@ ConnectionSTREAM::ConnectionSTREAM(void *arg, const unsigned index, const int vi
         commandsToBulkCtrl = commandsToBulkCtrlHw1;
     }
 
-    FPGAinfo fpgaInfo = this->GetFPGAInfo();
-    //expected version numbers based on HW number
-    vector<ExpectedVersion> versionList; //expected HW,FW,GW combinations
-    bool correctFWHW = false;
-    if (info.device == LMS_DEV_LIMESDR)
-    {
-        versionList = {
-            {4, 2, 2, 2},
-            {3, 2, 1, 20},
-            {2, 2, 1, 20},
-            {1, 6, 1, 20},
-        };
-    }
-    else if(info.device == LMS_DEV_STREAM)
-    {
-        versionList = {
-            {3, 8, 1, 2},
-        };
-    }
+    this->VersionCheck();
 
-    for(auto iter : versionList)
-        if(info.hardware == iter.hw
-           && info.firmware == iter.fw
-           && fpgaInfo.gatewareVersion == iter.gw
-           && fpgaInfo.gatewareRevision == iter.gw_rev)
-        {
-            correctFWHW = true;
-            break;
-        }
-
-    if(not correctFWHW)
-    {
-        ExpectedVersion expected = {-1, -1, -1, -1};
-        bool knownHardware = false;
-        for(auto iter : versionList)
-            if(info.hardware == iter.hw)
-        {
-            knownHardware = true;
-            expected = iter;
-            break;
-        }
-        if(not knownHardware)
-            std::cerr << "Unsupported hardware connected" << std::endl;
-
-        //check and warn about firmware mismatch problems
-        if (knownHardware && info.firmware != expected.fw)
-            std::cerr << std::endl
-            << "########################################################" << std::endl
-            << "##   !!!  Warning: firmware version mismatch  !!!" << std::endl
-            << "## Expected firmware version " << expected.fw << ", but found version " << info.firmware << std::endl
-            << "## Follow the FW and FPGA upgrade instructions:" << std::endl
-            << "## http://wiki.myriadrf.org/Lime_Suite#Flashing_images" << std::endl
-            << "########################################################" << std::endl
-            << std::endl;
-
-        //check and warn about gateware mismatch problems
-        if (knownHardware && (fpgaInfo.gatewareVersion != expected.gw
-            || fpgaInfo.gatewareRevision != expected.gw_rev))
-            std::cerr << std::endl
-            << "########################################################" << std::endl
-            << "##   !!!  Warning: gateware version mismatch  !!!" << std::endl
-            << "## Expected gateware version " << expected.gw << ", revision " << expected.gw_rev << std::endl
-            << "## But found version " << fpgaInfo.gatewareVersion << ", revision " << fpgaInfo.gatewareRevision<< std::endl
-            << "## Follow the FW and FPGA upgrade instructions:" << std::endl
-            << "## http://wiki.myriadrf.org/Lime_Suite#Flashing_images" << std::endl
-            << "########################################################" << std::endl
-            << std::endl;
-    }
+    if (info.device == LMS_DEV_LIMESDR || info.device == LMS_DEV_LIMESDR_USB_SP || info.device == LMS_DEV_LMS7002M_ULTIMATE_EVB)
+        DetectRefClk();
 
+    GetChipVersion();
     //must configure synthesizer before using LimeSDR
     if (info.device == LMS_DEV_LIMESDR && info.hardware < 4)
     {
@@ -173,29 +107,78 @@ ConnectionSTREAM::ConnectionSTREAM(void *arg, const unsigned index, const int vi
         Si5351C::Status status = si5351module->ConfigureClocks();
         if (status != Si5351C::SUCCESS)
         {
-            std::cerr << "Warning: Failed to configure Si5351C" << std::endl;
+            lime::warning("Failed to configure Si5351C");
             return;
         }
         status = si5351module->UploadConfiguration();
         if (status != Si5351C::SUCCESS)
-            std::cerr << "Warning: Failed to upload Si5351C configuration" << std::endl;
+            lime::warning("Failed to upload Si5351C configuration");
         std::this_thread::sleep_for(std::chrono::milliseconds(10)); //some settle time
     }
 }
 
+double ConnectionSTREAM::DetectRefClk(void)
+{
+    const double fx3Clk = 100e6 * 1.008;    //fx3 clock 100MHz (adjusted to 100.8 MHz based on measurement on multiple boards)
+    const double fx3Cnt = 16777210;         //fixed fx3 counter in FPGA
+    const double clkTbl[] = { 30.72e6, 38.4e6, 40e6, 52e6 };
+    const uint32_t addr[] = { 0x61, 0x63 };
+    const uint32_t vals[] = { 0x0, 0x0 };
+    if (this->WriteRegisters(addr, vals, 2) != 0)
+    {
+        return -1;
+    }
+    auto start = std::chrono::steady_clock::now();
+    if (this->WriteRegister(0x61, 0x4) != 0)
+    {
+        return -1;
+    }
+
+    while (1) //wait for test to finish
+    {
+        unsigned completed;
+        if (this->ReadRegister(0x65, completed) != 0)
+        {
+            return -1;
+        }
+        if (completed & 0x4)
+            break;
+
+        auto end = std::chrono::steady_clock::now();
+        std::chrono::duration<double> elapsed_seconds = end - start;
+        if (elapsed_seconds.count() > 0.5) //timeout
+        {
+            return -1;
+        }
+    }
+
+    const uint32_t addr2[] = { 0x72, 0x73 };
+    uint32_t vals2[2];
+    if (this->ReadRegisters(addr2, vals2, 2) != 0)
+    {
+        return -1;
+    }
+    double count = (vals2[0] | (vals2[1] << 16)); //cock counter
+    count *= fx3Clk / fx3Cnt;   //estimate ref clock based on FX3 Clock
+    lime::info("Estimated reference clock %1.4f MHz", count/1e6);
+    unsigned i = 0;
+    double delta = 100e6;
+
+    while (i < sizeof(clkTbl) / sizeof(*clkTbl))
+        if (delta < fabs(count - clkTbl[i]))
+            break;
+        else
+            delta = fabs(count - clkTbl[i++]);
+
+    this->SetReferenceClockRate(clkTbl[i-1]);
+    lime::info("Selected reference clock %1.3f MHz", clkTbl[i - 1] / 1e6);
+    return clkTbl[i - 1];
+}
+
 /**	@brief Closes connection to chip and deallocates used memory.
 */
 ConnectionSTREAM::~ConnectionSTREAM()
 {
-    for(auto i : mTxStreams)
-        ControlStream((size_t)i, false);
-    for(auto i : mRxStreams)
-        ControlStream((size_t)i, false);
-    for(auto i : mTxStreams)
-        CloseStream((size_t)i);
-    for(auto i : mRxStreams)
-        CloseStream((size_t)i);
-    UpdateThreads();
     Close();
 #ifndef __unix__
     delete USBDevicePrimary;
@@ -205,7 +188,7 @@ ConnectionSTREAM::~ConnectionSTREAM()
 /**	@brief Tries to open connected USB device and find communication endpoints.
 	@return Returns 0-Success, other-EndPoints not found or device didn't connect.
 */
-int ConnectionSTREAM::Open(const unsigned index, const int vid, const int pid)
+int ConnectionSTREAM::Open(const std::string &vidpid, const std::string &serial, const unsigned index)
 {
     bulkCtrlAvailable = false;
 #ifndef __unix__
@@ -239,22 +222,23 @@ int ConnectionSTREAM::Open(const unsigned index, const int vid, const int pid)
     OutCtrlEndPt3->Index = CTR_W_INDEX;
     OutCtrlEndPt3->TimeOut = 3000;
 
-    for (int i=0; i<USBDevicePrimary->EndPointCount(); i++)
-        if(USBDevicePrimary->EndPoints[i]->Address == streamBulkOutAddr)
+    for (int i = 0; i < USBDevicePrimary->EndPointCount(); i++)
+    {
+        auto adr = USBDevicePrimary->EndPoints[i]->Address;
+        if (adr < ctrlBulkOutAddr)
         {
-            OutEndPt = USBDevicePrimary->EndPoints[i];
-            long len = OutEndPt->MaxPktSize * 64;
-            OutEndPt->SetXferSize(len);
-            break;
+            OutEndPt[adr] = USBDevicePrimary->EndPoints[i];
+            long len = OutEndPt[adr]->MaxPktSize * 64;
+            OutEndPt[adr]->SetXferSize(len);
         }
-    for (int i=0; i<USBDevicePrimary->EndPointCount(); i++)
-        if(USBDevicePrimary->EndPoints[i]->Address == streamBulkInAddr)
+        else if (adr < ctrlBulkInAddr)
         {
-            InEndPt = USBDevicePrimary->EndPoints[i];
-            long len = InEndPt->MaxPktSize * 64;
-            InEndPt->SetXferSize(len);
-            break;
+            adr &= 0xF;
+            InEndPt[adr] = USBDevicePrimary->EndPoints[i];
+            long len = InEndPt[adr]->MaxPktSize * 64;
+            InEndPt[adr]->SetXferSize(len);
         }
+    }
 
     InCtrlBulkEndPt = nullptr;
     for (int i=0; i<USBDevicePrimary->EndPointCount(); i++)
@@ -277,14 +261,53 @@ int ConnectionSTREAM::Open(const unsigned index, const int vid, const int pid)
     isConnected = true;
     return 0;
 #else
-    dev_handle = libusb_open_device_with_vid_pid(ctx, vid, pid);
+    const auto splitPos = vidpid.find(":");
+    const auto vid = std::stoi(vidpid.substr(0, splitPos), nullptr, 16);
+    const auto pid = std::stoi(vidpid.substr(splitPos+1), nullptr, 16);
+
+    libusb_device **devs; //pointer to pointer of device, used to retrieve a list of devices
+    int usbDeviceCount = libusb_get_device_list(ctx, &devs);
+
+    if (usbDeviceCount < 0) {
+        return ReportError(-1, "ConnectionSTREAM: libusb_get_device_list failed: %s", libusb_strerror(libusb_error(usbDeviceCount)));
+    }
+
+    for(int i=0; i<usbDeviceCount; ++i)
+    {
+        libusb_device_descriptor desc;
+        int r = libusb_get_device_descriptor(devs[i], &desc);
+        if(r<0) {
+            lime::error("failed to get device description");
+            continue;
+        }
+        if (desc.idProduct != pid) continue;
+        if (desc.idVendor != vid) continue;
+        if(libusb_open(devs[i], &dev_handle) != 0) continue;
+
+        std::string foundSerial;
+        if (desc.iSerialNumber > 0)
+        {
+            char data[255];
+            r = libusb_get_string_descriptor_ascii(dev_handle,desc.iSerialNumber,(unsigned char*)data, sizeof(data));
+            if(r<0)
+                lime::error("failed to get serial number");
+            else
+                foundSerial = std::string(data, size_t(r));
+        }
+
+        if (serial == foundSerial) break; //found it
+        libusb_close(dev_handle);
+        dev_handle = nullptr;
+    }
+    libusb_free_device_list(devs, 1);
+
     if(dev_handle == nullptr)
         return ReportError(-1, "ConnectionSTREAM: libusb_open failed");
     if(libusb_kernel_driver_active(dev_handle, 0) == 1)   //find out if kernel driver is attached
     {
-        printf("Kernel Driver Active\n");
+        lime::info("Kernel Driver Active");
         if(libusb_detach_kernel_driver(dev_handle, 0) == 0) //detach it
-            printf("Kernel Driver Detached!\n");
+            lime::info("Kernel Driver Detached!");
     }
     int r = libusb_claim_interface(dev_handle, 0); //claim interface 0 (the first) of device
     if(r < 0)
@@ -294,7 +317,7 @@ int ConnectionSTREAM::Open(const unsigned index, const int vid, const int pid)
     libusb_config_descriptor* descriptor = nullptr;
     if(libusb_get_active_config_descriptor(device, &descriptor) < 0)
     {
-        printf("failed to get config descriptor\n");
+        lime::error("failed to get config descriptor");
     }
     //check for 0x0F and 0x8F endpoints
     if(descriptor->bNumInterfaces > 0)
@@ -316,32 +339,31 @@ int ConnectionSTREAM::Open(const unsigned index, const int vid, const int pid)
         ctrPkt.cmd = CMD_USB_FIFO_RST;
         ctrPkt.outBuffer.push_back(0x01); //reset bulk endpoints
         if(TransferPacket(ctrPkt) != 0)
-            printf("Failed to reset USB bulk endpoints\n");
+            lime::error("Failed to reset USB bulk endpoints");
     }
     return 0;
 #endif
 }
-
 /**	@brief Closes communication to device.
 */
 void ConnectionSTREAM::Close()
 {
     #ifndef __unix__
-	USBDevicePrimary->Close();
-	InEndPt = nullptr;
-	OutEndPt = nullptr;
-	InCtrlBulkEndPt = nullptr;
-	OutCtrlBulkEndPt = nullptr;
-	if (InCtrlEndPt3)
-	{
-		delete InCtrlEndPt3;
-		InCtrlEndPt3 = nullptr;
-	}
-	if (OutCtrlEndPt3)
-	{
-		delete OutCtrlEndPt3;
-		OutCtrlEndPt3 = nullptr;
-	}
+    USBDevicePrimary->Close();
+    for (int i = 0; i < MAX_EP_CNT; i++)
+        InEndPt[i] = OutEndPt[i] = nullptr;
+    InCtrlBulkEndPt = nullptr;
+    OutCtrlBulkEndPt = nullptr;
+    if (InCtrlEndPt3)
+    {
+        delete InCtrlEndPt3;
+        InCtrlEndPt3 = nullptr;
+    }
+    if (OutCtrlEndPt3)
+    {
+        delete OutCtrlEndPt3;
+        OutCtrlEndPt3 = nullptr;
+    }
     #else
     if(dev_handle != 0)
     {
@@ -404,7 +426,7 @@ int ConnectionSTREAM::Write(const unsigned char *buffer, const int length, int t
     else
         len = libusb_control_transfer(dev_handle, LIBUSB_REQUEST_TYPE_VENDOR,CTR_W_REQCODE ,CTR_W_VALUE, CTR_W_INDEX, wbuffer, length, timeout_ms);
     #endif
-    delete wbuffer;
+    delete[] wbuffer;
     return len;
 }
 
@@ -456,7 +478,7 @@ void callback_libusbtransfer(libusb_transfer *trans)
 	switch(trans->status)
 	{
     case LIBUSB_TRANSFER_CANCELLED:
-        //printf("Transfer %i canceled\n", context->id);
+        //lime::error("Transfer %i canceled", context->id);
         context->bytesXfered = trans->actual_length;
         context->done.store(true);
         //context->used = false;
@@ -470,27 +492,27 @@ void callback_libusbtransfer(libusb_transfer *trans)
 		}
         break;
     case LIBUSB_TRANSFER_ERROR:
-        printf("TRANSFER ERRRO\n");
+        lime::error("TRANSFER ERRROR");
         context->bytesXfered = trans->actual_length;
         context->done.store(true);
         //context->used = false;
         break;
     case LIBUSB_TRANSFER_TIMED_OUT:
-        //printf("transfer timed out %i\n", context->id);
+        //lime::error("transfer timed out %i", context->id);
         context->bytesXfered = trans->actual_length;
         context->done.store(true);
         //context->used = false;
 
         break;
     case LIBUSB_TRANSFER_OVERFLOW:
-        printf("transfer overflow\n");
+        lime::error("transfer overflow");
 
         break;
     case LIBUSB_TRANSFER_STALL:
-        printf("transfer stalled\n");
+        lime::error("transfer stalled");
         break;
     case LIBUSB_TRANSFER_NO_DEVICE:
-        printf("transfer no device\n");
+        lime::error("transfer no device");
 
         break;
 	}
@@ -503,9 +525,10 @@ void callback_libusbtransfer(libusb_transfer *trans)
 	@brief Starts asynchronous data reading from board
 	@param *buffer buffer where to store received data
 	@param length number of bytes to read
+	@param streamBulkInAddr endpoint index?
 	@return handle of transfer context
 */
-int ConnectionSTREAM::BeginDataReading(char *buffer, uint32_t length)
+int ConnectionSTREAM::BeginDataReading(char *buffer, uint32_t length, const uint8_t streamBulkInAddr)
 {
     int i = 0;
 	bool contextFound = false;
@@ -520,13 +543,16 @@ int ConnectionSTREAM::BeginDataReading(char *buffer, uint32_t length)
     }
     if(!contextFound)
     {
-        printf("No contexts left for reading data\n");
+        lime::error("No contexts left for reading data");
         return -1;
     }
     contexts[i].used = true;
     #ifndef __unix__
-    if(InEndPt)
-        contexts[i].context = InEndPt->BeginDataXfer((unsigned char*)buffer, length, contexts[i].inOvLap);
+    if (InEndPt[streamBulkInAddr & 0xF])
+    {
+        contexts[i].EndPt = InEndPt[streamBulkInAddr & 0xF];
+        contexts[i].context = contexts[i].EndPt->BeginDataXfer((unsigned char*)buffer, length, contexts[i].inOvLap);
+    }
 	return i;
     #else
     unsigned int Timeout = 500;
@@ -538,7 +564,7 @@ int ConnectionSTREAM::BeginDataReading(char *buffer, uint32_t length)
 	int status = libusb_submit_transfer(tr);
     if(status != 0)
     {
-        printf("ERROR BEGIN DATA READING %s\n", libusb_error_name(status));
+        lime::error("BEGIN DATA READING %s", libusb_error_name(status));
         contexts[i].used = false;
         return -1;
     }
@@ -558,8 +584,7 @@ int ConnectionSTREAM::WaitForReading(int contextHandle, unsigned int timeout_ms)
     {
     #ifndef __unix__
     int status = 0;
-	if(InEndPt)
-        status = InEndPt->WaitForXfer(contexts[contextHandle].inOvLap, timeout_ms);
+    status = contexts[contextHandle].EndPt->WaitForXfer(contexts[contextHandle].inOvLap, timeout_ms);
 	return status;
     #else
     auto t1 = chrono::high_resolution_clock::now();
@@ -593,8 +618,7 @@ int ConnectionSTREAM::FinishDataReading(char *buffer, uint32_t length, int conte
     #ifndef __unix__
     int status = 0;
     long len = length;
-    if(InEndPt)
-        status = InEndPt->FinishDataXfer((unsigned char*)buffer, len, contexts[contextHandle].inOvLap, contexts[contextHandle].context);
+    status = contexts[contextHandle].EndPt->FinishDataXfer((unsigned char*)buffer, len, contexts[contextHandle].inOvLap, contexts[contextHandle].context);
     contexts[contextHandle].used = false;
     contexts[contextHandle].reset();
     return len;
@@ -612,14 +636,16 @@ int ConnectionSTREAM::FinishDataReading(char *buffer, uint32_t length, int conte
 /**
 	@brief Aborts reading operations
 */
-void ConnectionSTREAM::AbortReading()
+void ConnectionSTREAM::AbortReading(int ep)
 {
 #ifndef __unix__
-	InEndPt->Abort();
+    for (int i = 0; i < MAX_EP_CNT; i++)
+        if (InEndPt[i] && InEndPt[i]->Address == ep)
+	        InEndPt[i]->Abort();
 #else
     for(int i=0; i<USB_MAX_CONTEXTS; ++i)
     {
-        if(contexts[i].used)
+        if(contexts[i].used && contexts[i].transfer->endpoint == ep)
             libusb_cancel_transfer( contexts[i].transfer );
     }
 #endif
@@ -629,9 +655,10 @@ void ConnectionSTREAM::AbortReading()
 	@brief Starts asynchronous data Sending to board
 	@param *buffer buffer to send
 	@param length number of bytes to send
+	@param streamBulkOutAddr endpoint index?
 	@return handle of transfer context
 */
-int ConnectionSTREAM::BeginDataSending(const char *buffer, uint32_t length)
+int ConnectionSTREAM::BeginDataSending(const char *buffer, uint32_t length, const uint8_t streamBulkOutAddr)
 {
     int i = 0;
 	//find not used context
@@ -648,8 +675,11 @@ int ConnectionSTREAM::BeginDataSending(const char *buffer, uint32_t length)
         return -1;
     contextsToSend[i].used = true;
     #ifndef __unix__
-    if(OutEndPt)
-        contextsToSend[i].context = OutEndPt->BeginDataXfer((unsigned char*)buffer, length, contextsToSend[i].inOvLap);
+    if (OutEndPt[streamBulkOutAddr])
+    {
+        contextsToSend[i].EndPt = OutEndPt[streamBulkOutAddr];
+        contextsToSend[i].context = contextsToSend[i].EndPt->BeginDataXfer((unsigned char*)buffer, length, contextsToSend[i].inOvLap);
+    }
 	return i;
     #else
     unsigned int Timeout = 500;
@@ -661,7 +691,7 @@ int ConnectionSTREAM::BeginDataSending(const char *buffer, uint32_t length)
     int status = libusb_submit_transfer(tr);
     if(status != 0)
     {
-        printf("ERROR BEGIN DATA SENDING %s\n", libusb_error_name(status));
+        lime::error("BEGIN DATA SENDING %s", libusb_error_name(status));
         contextsToSend[i].used = false;
         return -1;
     }
@@ -679,12 +709,11 @@ int ConnectionSTREAM::WaitForSending(int contextHandle, unsigned int timeout_ms)
 {
     if( contextsToSend[contextHandle].used == true )
     {
-    #ifndef __unix__
+#   ifndef __unix__
 	int status = 0;
-	if(OutEndPt)
-        status = OutEndPt->WaitForXfer(contextsToSend[contextHandle].inOvLap, timeout_ms);
+	status = contextsToSend[contextHandle].EndPt->WaitForXfer(contextsToSend[contextHandle].inOvLap, timeout_ms);
 	return status;
-    #else
+#   else
     auto t1 = chrono::high_resolution_clock::now();
     auto t2 = chrono::high_resolution_clock::now();
 
@@ -696,7 +725,7 @@ int ConnectionSTREAM::WaitForSending(int contextHandle, unsigned int timeout_ms)
         t2 = chrono::high_resolution_clock::now();
     }
 	return contextsToSend[contextHandle].done == true;
-    #endif
+#   endif
     }
     else
         return 0;
@@ -715,8 +744,7 @@ int ConnectionSTREAM::FinishDataSending(const char *buffer, uint32_t length, int
     {
 #ifndef __unix__
     long len = length;
-	if(OutEndPt)
-        OutEndPt->FinishDataXfer((unsigned char*)buffer, len, contextsToSend[contextHandle].inOvLap, contextsToSend[contextHandle].context);
+    contextsToSend[contextHandle].EndPt->FinishDataXfer((unsigned char*)buffer, len, contextsToSend[contextHandle].inOvLap, contextsToSend[contextHandle].context);
     contextsToSend[contextHandle].used = false;
     contextsToSend[contextHandle].reset();
     return len;
@@ -734,19 +762,39 @@ int ConnectionSTREAM::FinishDataSending(const char *buffer, uint32_t length, int
 /**
 	@brief Aborts sending operations
 */
-void ConnectionSTREAM::AbortSending()
+void ConnectionSTREAM::AbortSending(int ep)
 {
 #ifndef __unix__
-	OutEndPt->Abort();
+    for (int i = 0; i < MAX_EP_CNT; i++)
+        if (OutEndPt[i] && OutEndPt[i]->Address == ep)
+            OutEndPt[i]->Abort();
 #else
     for (int i = 0; i<USB_MAX_CONTEXTS; ++i)
     {
-        if(contextsToSend[i].used)
+        if(contextsToSend[i].used && contextsToSend[i].transfer->endpoint == ep)
             libusb_cancel_transfer(contextsToSend[i].transfer);
     }
 #endif
 }
 
+int ConnectionSTREAM::SendData(const char* buffer, int length, int epIndex, int timeout)
+{
+    const unsigned char ep = 0x01;
+    int context = BeginDataSending((char*)buffer, length, ep);
+    if (WaitForSending(context, timeout)==false)
+        AbortSending(ep);
+    return FinishDataSending((char*)buffer, length , context);
+}
+
+int ConnectionSTREAM::ReceiveData(char* buffer, int length, int epIndex, int timeout)
+{
+    const unsigned char ep = 0x81;
+    int context = BeginDataReading(buffer, length, ep);
+    if (WaitForReading(context, timeout) == false)
+        AbortReading(ep);
+    return FinishDataReading(buffer, length, context);
+}
+
 int ConnectionSTREAM::ProgramWrite(const char *buffer, const size_t length, const int programmingMode, const int device, ProgrammingCallback callback)
 {
     if (device == LMS64CProtocol::FX3 && programmingMode == 1)
@@ -755,7 +803,7 @@ int ConnectionSTREAM::ProgramWrite(const char *buffer, const size_t length, cons
         libusb_device_descriptor desc;
         int ret = libusb_get_device_descriptor(libusb_get_device(dev_handle), &desc);
         if(ret<0)
-            printf("failed to get device description\n");
+            lime::error("failed to get device description");
         else if (desc.idProduct == 243)
 #else
 		if (USBDevicePrimary->ProductID == 243)
@@ -821,7 +869,7 @@ int ConnectionSTREAM::ram_write(unsigned char *buf, unsigned int ramAddress, int
 		r = libusb_control_transfer(dev_handle, 0x40, 0xA0, GET_LSW(ramAddress), GET_MSW(ramAddress),&buf[index], size, VENDORCMD_TIMEOUT);
 		if ( r != size )
 		{
-			printf("Vendor write to FX3 RAM failed\n");
+			lime::error("Vendor write to FX3 RAM failed");
 			return -1;
 		}
 		ramAddress += size;
@@ -886,7 +934,7 @@ int ConnectionSTREAM::fx3_usbboot_download(unsigned char *fwBuf, int filesize)
 
 			r = libusb_control_transfer(dev_handle, 0x40, 0xA0, GET_LSW(address), GET_MSW(address), NULL,0, VENDORCMD_TIMEOUT);
 			if ( r != 0 )
-				printf("Ignored error in control transfer: %d\n", r);
+				lime::error("Ignored error in control transfer: %d", r);
 			break;
 		}
 		index += (8 + length * 4);
