@@ -44,17 +44,10 @@ Connection_uLimeSDR::Connection_uLimeSDR(void *arg)
 */
 Connection_uLimeSDR::Connection_uLimeSDR(void *arg, const unsigned index, const int vid, const int pid)
 {
-    mTimestampOffset = 0;
-    rxLastTimestamp.store(0);
+    RxLoopFunction = bind(&Connection_uLimeSDR::ReceivePacketsLoop, this, std::placeholders::_1);
+    TxLoopFunction = bind(&Connection_uLimeSDR::TransmitPacketsLoop, this, std::placeholders::_1);
     mExpectedSampleRate = 0;
-    generateData.store(false);
-    rxRunning.store(false);
-    txRunning.store(false);
     isConnected = false;
-    terminateRx.store(false);
-    terminateTx.store(false);
-    rxDataRate_Bps.store(0);
-    txDataRate_Bps.store(0);
 
     mStreamWrEndPtAddr = 0x03;
     mStreamRdEndPtAddr = 0x83;
@@ -69,23 +62,16 @@ Connection_uLimeSDR::Connection_uLimeSDR(void *arg, const unsigned index, const
 	mUsbCounter = 0;
     ctx = (libusb_context *)arg;
 #endif
-    if(this->Open(index, vid, pid) != 0)
+    if (this->Open(index, vid, pid) != 0)
         std::cerr << GetLastErrorMessage() << std::endl;
+    this->SetReferenceClockRate(52e6);
+    GetChipVersion();
 }
 
 /**	@brief Closes connection to chip and deallocates used memory.
 */
 Connection_uLimeSDR::~Connection_uLimeSDR()
 {
-    for(auto i : mTxStreams)
-        ControlStream((size_t)i, false);
-    for(auto i : mRxStreams)
-        ControlStream((size_t)i, false);
-    for(auto i : mTxStreams)
-        CloseStream((size_t)i);
-    for(auto i : mRxStreams)
-        CloseStream((size_t)i);
-    UpdateThreads();
     Close();
 }
 #ifdef __unix__
@@ -159,9 +145,7 @@ int Connection_uLimeSDR::Open(const unsigned index, const int vid, const int pid
 	DWORD devCount;
 	FT_STATUS ftStatus = FT_OK;
 	DWORD dwNumDevices = 0;
-	//
 	// Open a device
-	//
 	ftStatus = FT_Create(0, FT_OPEN_BY_INDEX, &mFTHandle);
 	if (FT_FAILED(ftStatus))
 	{
@@ -174,6 +158,8 @@ int Connection_uLimeSDR::Open(const unsigned index, const int vid, const int pid
 	FT_AbortPipe(mFTHandle, mStreamWrEndPtAddr);
 	FT_SetStreamPipe(mFTHandle, FALSE, FALSE, 0x82, 64);
 	FT_SetStreamPipe(mFTHandle, FALSE, FALSE, 0x02, 64);
+    FT_SetPipeTimeout(mFTHandle, 0x02, 500);
+    FT_SetPipeTimeout(mFTHandle, 0x82, 500);
 	isConnected = true;
 	return 0;
 #else
@@ -236,6 +222,16 @@ bool Connection_uLimeSDR::IsOpen()
     return isConnected;
 }
 
+#ifndef __unix__
+int Connection_uLimeSDR::ReinitPipe(unsigned char ep)
+{
+    FT_AbortPipe(mFTHandle, ep);
+    FT_FlushPipe(mFTHandle, ep);
+    FT_SetStreamPipe(mFTHandle, FALSE, FALSE, ep, 64);
+    return 0;
+}
+#endif
+
 /**	@brief Sends given data buffer to chip through USB port.
 @param buffer data buffer, must not be longer than 64 bytes.
 @param length given buffer size.
@@ -246,49 +242,48 @@ int Connection_uLimeSDR::Write(const unsigned char *buffer, const int length, in
 {
     std::lock_guard<std::mutex> lock(mExtraUsbMutex);
     long len = 0;
-    if(IsOpen() == false)
+    if (IsOpen() == false)
         return 0;
 
+#ifndef __unix__
+    // Write to channel 1 ep 0x02
+    ULONG ulBytesWrite = 0;
+    FT_STATUS ftStatus = FT_OK;
+    OVERLAPPED	vOverlapped = { 0 };
+    FT_InitializeOverlapped(mFTHandle, &vOverlapped);
+    ftStatus = FT_WritePipe(mFTHandle, 0x02, (unsigned char*)buffer, length, &ulBytesWrite, &vOverlapped);
+    if (ftStatus != FT_IO_PENDING)
+    {
+        FT_ReleaseOverlapped(mFTHandle, &vOverlapped);
+        ReinitPipe(0x02);
+        return -1;
+    }
+
+    DWORD dwRet = WaitForSingleObject(vOverlapped.hEvent, timeout_ms);
+    if (dwRet == WAIT_OBJECT_0 || dwRet == WAIT_TIMEOUT)
+    {
+        if (GetOverlappedResult(mFTHandle, &vOverlapped, &ulBytesWrite, FALSE) == FALSE)
+        {
+            ReinitPipe(0x02);
+            ulBytesWrite = -1;
+        }
+    }
+    else
+    {
+        ReinitPipe(0x02);
+        ulBytesWrite = -1;
+    }
+    FT_ReleaseOverlapped(mFTHandle, &vOverlapped);
+    return ulBytesWrite;
+#else
     unsigned char* wbuffer = new unsigned char[length];
     memcpy(wbuffer, buffer, length);
-#ifndef __unix__
-
-	//
-	// Write to channel 1 ep 0x02
-	//
-	ULONG ulBytesWrite = 0;
-	FT_STATUS ftStatus = FT_OK;
-	OVERLAPPED	vOverlapped = { 0 };
-	memset(&vOverlapped, 0, sizeof(OVERLAPPED));
-	vOverlapped.hEvent = CreateEvent(NULL, false, false, NULL);
-	ftStatus = FT_WritePipe(mFTHandle, 0x02, (unsigned char*)buffer, length, &ulBytesWrite, &vOverlapped);
-	if (ftStatus != FT_IO_PENDING)
-	{
-		CloseHandle(vOverlapped.hEvent);
-		return -1;
-	}
-
-	DWORD dwRet = WaitForSingleObject(vOverlapped.hEvent, timeout_ms);
-	if (dwRet == WAIT_OBJECT_0 || dwRet == WAIT_TIMEOUT)
-	{
-		if (GetOverlappedResult(mFTHandle, &vOverlapped, &ulBytesWrite, FALSE)==FALSE)
-		{
-			ulBytesWrite = -1;
-		}
-	}
-	else
-	{
-		ulBytesWrite = -1;
-	}
-	CloseHandle(vOverlapped.hEvent);
-	return ulBytesWrite;
-#else
     int actual = 0;
     libusb_bulk_transfer(dev_handle, 0x02, wbuffer, length, &actual, timeout_ms);
     len = actual;
-#endif
     delete[] wbuffer;
     return len;
+#endif
 }
 
 /**	@brief Reads data coming from the chip through USB port.
@@ -298,6 +293,7 @@ big enough to fit received data.
 @param timeout_ms timeout limit for operation in milliseconds
 @return number of bytes received.
 */
+
 int Connection_uLimeSDR::Read(unsigned char *buffer, const int length, int timeout_ms)
 {
     std::lock_guard<std::mutex> lock(mExtraUsbMutex);
@@ -305,35 +301,37 @@ int Connection_uLimeSDR::Read(unsigned char *buffer, const int length, int timeo
     if(IsOpen() == false)
         return 0;
 #ifndef __unix__
-	//
-	// Read from channel 1 ep 0x82
-	//
-	ULONG ulBytesRead = 0;
-	FT_STATUS ftStatus = FT_OK;
-	OVERLAPPED	vOverlapped = { 0 };
-	memset(&vOverlapped, 0, sizeof(OVERLAPPED));
-	vOverlapped.hEvent = CreateEvent(NULL, false, false, NULL);
-	ftStatus = FT_ReadPipe(mFTHandle, 0x82, buffer, length, &ulBytesRead, &vOverlapped);
-	if (ftStatus != FT_IO_PENDING)
-	{
-		CloseHandle(vOverlapped.hEvent);
-		return -1;;
-	}
+    //
+    // Read from channel 1 ep 0x82
+    //
+    ULONG ulBytesRead = 0;
+    FT_STATUS ftStatus = FT_OK;
+    OVERLAPPED	vOverlapped = { 0 };
+    FT_InitializeOverlapped(mFTHandle, &vOverlapped);
+    ftStatus = FT_ReadPipe(mFTHandle, 0x82, buffer, length, &ulBytesRead, &vOverlapped);
+    if (ftStatus != FT_IO_PENDING)
+    {
+        FT_ReleaseOverlapped(mFTHandle, &vOverlapped);
+        ReinitPipe(0x82);
+        return -1;;
+    }
 
-	DWORD dwRet = WaitForSingleObject(vOverlapped.hEvent, timeout_ms);
-	if (dwRet == WAIT_OBJECT_0 || dwRet == WAIT_TIMEOUT)
-	{
-		if (GetOverlappedResult(mFTHandle, &vOverlapped, &ulBytesRead, FALSE)==FALSE)
-		{
-			ulBytesRead = -1;
-		}
-	}
-	else
-	{
-		ulBytesRead = -1;
-	}
-	CloseHandle(vOverlapped.hEvent);
-	return ulBytesRead;
+    DWORD dwRet = WaitForSingleObject(vOverlapped.hEvent, timeout_ms);
+    if (dwRet == WAIT_OBJECT_0 || dwRet == WAIT_TIMEOUT)
+    {
+        if (GetOverlappedResult(mFTHandle, &vOverlapped, &ulBytesRead, FALSE)==FALSE)
+        {
+            ReinitPipe(0x82);
+            ulBytesRead = -1;
+        }
+    }
+    else
+    {
+        ReinitPipe(0x82);
+        ulBytesRead = -1;
+    }
+    FT_ReleaseOverlapped(mFTHandle, &vOverlapped);
+    return ulBytesRead;
 #else
     int actual = 0;
     libusb_bulk_transfer(dev_handle, 0x82, buffer, len, &actual, timeout_ms);
@@ -348,48 +346,49 @@ int Connection_uLimeSDR::Read(unsigned char *buffer, const int length, int timeo
 static void callback_libusbtransfer(libusb_transfer *trans)
 {
     Connection_uLimeSDR::USBTransferContext *context = reinterpret_cast<Connection_uLimeSDR::USBTransferContext*>(trans->user_data);
+    std::unique_lock<std::mutex> lck(context->transferLock);
     switch(trans->status)
     {
-    case LIBUSB_TRANSFER_CANCELLED:
-        //printf("Transfer %i canceled\n", context->id);
-        context->bytesXfered = trans->actual_length;
-        context->done.store(true);
-        //context->used = false;
-        //context->reset();
+        case LIBUSB_TRANSFER_CANCELLED:
+            //printf("Transfer %i canceled\n", context->id);
+            context->bytesXfered = trans->actual_length;
+            context->done.store(true);
+            //context->used = false;
+            //context->reset();
+            break;
+        case LIBUSB_TRANSFER_COMPLETED:
+            //if(trans->actual_length == context->bytesExpected)
+            {
+                context->bytesXfered = trans->actual_length;
+                context->done.store(true);
+            }
         break;
-    case LIBUSB_TRANSFER_COMPLETED:
-        //if(trans->actual_length == context->bytesExpected)
-    {
-        context->bytesXfered = trans->actual_length;
-        context->done.store(true);
+        case LIBUSB_TRANSFER_ERROR:
+            printf("TRANSFER ERRRO\n");
+            context->bytesXfered = trans->actual_length;
+            context->done.store(true);
+            //context->used = false;
+            break;
+        case LIBUSB_TRANSFER_TIMED_OUT:
+            //printf("transfer timed out %i\n", context->id);
+            context->bytesXfered = trans->actual_length;
+            context->done.store(true);
+            //context->used = false;
+
+            break;
+        case LIBUSB_TRANSFER_OVERFLOW:
+            printf("transfer overflow\n");
+
+            break;
+        case LIBUSB_TRANSFER_STALL:
+            printf("transfer stalled\n");
+            break;
+        case LIBUSB_TRANSFER_NO_DEVICE:
+            printf("transfer no device\n");
+
+            break;
     }
-    break;
-    case LIBUSB_TRANSFER_ERROR:
-        printf("TRANSFER ERRRO\n");
-        context->bytesXfered = trans->actual_length;
-        context->done.store(true);
-        //context->used = false;
-        break;
-    case LIBUSB_TRANSFER_TIMED_OUT:
-        //printf("transfer timed out %i\n", context->id);
-        context->bytesXfered = trans->actual_length;
-        context->done.store(true);
-        //context->used = false;
-
-        break;
-    case LIBUSB_TRANSFER_OVERFLOW:
-        printf("transfer overflow\n");
-
-        break;
-    case LIBUSB_TRANSFER_STALL:
-        printf("transfer stalled\n");
-        break;
-    case LIBUSB_TRANSFER_NO_DEVICE:
-        printf("transfer no device\n");
-
-        break;
-    }
-    context->transferLock.unlock();
+    lck.unlock();
     context->cv.notify_one();
 }
 #endif
@@ -419,19 +418,19 @@ int Connection_uLimeSDR::BeginDataReading(char *buffer, uint32_t length)
         return -1;
     }
     contexts[i].used = true;
+
 #ifndef __unix__
 	if (length != rxSize)
 	{
 		rxSize = length;
 		FT_SetStreamPipe(mFTHandle, FALSE, FALSE, mStreamRdEndPtAddr, rxSize);
 	}
-	memset(&contexts[i].inOvLap, 0, sizeof(OVERLAPPED));
-	contexts[i].inOvLap.hEvent = CreateEvent(NULL, false, false, NULL);
+    FT_InitializeOverlapped(mFTHandle, &contexts[i].inOvLap);
 	ULONG ulActual;
-	if (FT_ReadPipe(mFTHandle, mStreamRdEndPtAddr, (unsigned char*)buffer, length, &ulActual, &contexts[i].inOvLap)!= FT_IO_PENDING)
-	{
-		return -1;
-	}
+    FT_STATUS ftStatus = FT_OK;
+    ftStatus = FT_ReadPipe(mFTHandle, mStreamRdEndPtAddr, (unsigned char*)buffer, length, &ulActual, &contexts[i].inOvLap);
+    if (ftStatus != FT_IO_PENDING)
+        return -1;
 #else
     if (length != rxSize)
     {
@@ -451,8 +450,6 @@ int Connection_uLimeSDR::BeginDataReading(char *buffer, uint32_t length)
         contexts[i].used = false;
         return -1;
     }
-    else
-        contexts[i].transferLock.lock();
 #endif
     return i;
 }
@@ -468,9 +465,8 @@ int Connection_uLimeSDR::WaitForReading(int contextHandle, unsigned int timeout_
     if(contextHandle >= 0 && contexts[contextHandle].used == true)
     {
 #ifndef __unix__
-		contexts[contextHandle].inOvLap.InternalHigh = 0;
         DWORD dwRet = WaitForSingleObject(contexts[contextHandle].inOvLap.hEvent, timeout_ms);
-		if (dwRet == WAIT_OBJECT_0 || dwRet == WAIT_TIMEOUT)
+		if (dwRet == WAIT_OBJECT_0)
 			return 1;
 #else
         auto t1 = chrono::high_resolution_clock::now();
@@ -480,7 +476,7 @@ int Connection_uLimeSDR::WaitForReading(int contextHandle, unsigned int timeout_
         while(contexts[contextHandle].done.load() == false && std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count() < timeout_ms)
         {
             //blocking not to waste CPU
-            contexts[contextHandle].cv.wait(lck);
+            contexts[contextHandle].cv.wait_for(lck, chrono::milliseconds(timeout_ms));
             t2 = chrono::high_resolution_clock::now();
         }
         return contexts[contextHandle].done.load() == true;
@@ -501,15 +497,17 @@ int Connection_uLimeSDR::FinishDataReading(char *buffer, uint32_t length, int co
     if(contextHandle >= 0 && contexts[contextHandle].used == true)
     {
 #ifndef __unix__
-		ULONG ulActualBytesTransferred;
+	ULONG ulActualBytesTransferred;
+        FT_STATUS ftStatus = FT_OK;
 
-		if (GetOverlappedResult(mFTHandle, &contexts[contextHandle].inOvLap, &ulActualBytesTransferred, FALSE) == FALSE)
-		{
-			return -1;
-		}
-		length = ulActualBytesTransferred;
-		contexts[contextHandle].used = false;
-		return length;
+        ftStatus = FT_GetOverlappedResult(mFTHandle, &contexts[contextHandle].inOvLap, &ulActualBytesTransferred, FALSE);
+        if (ftStatus != FT_OK)
+            length = 0;
+        else
+            length = ulActualBytesTransferred;
+        FT_ReleaseOverlapped(mFTHandle, &contexts[contextHandle].inOvLap);
+        contexts[contextHandle].used = false;
+        return length;
 #else
         length = contexts[contextHandle].bytesXfered;
         contexts[contextHandle].used = false;
@@ -532,10 +530,11 @@ void Connection_uLimeSDR::AbortReading()
 	{
 		if (contexts[i].used == true)
 		{
-			CloseHandle(contexts[i].inOvLap.hEvent);
+            FT_ReleaseOverlapped(mFTHandle, &contexts[i].inOvLap);
 			contexts[i].used = false;
 		}
 	}
+    FT_FlushPipe(mFTHandle, mStreamRdEndPtAddr);
 	rxSize = 0;
 #else
 
@@ -570,7 +569,8 @@ int Connection_uLimeSDR::BeginDataSending(const char *buffer, uint32_t length)
     }
     if(!contextFound)
         return -1;
-        contextsToSend[i].used = true;
+    contextsToSend[i].used = true;
+
 #ifndef __unix__
 	FT_STATUS ftStatus = FT_OK;
 	ULONG ulActualBytesSend;
@@ -579,13 +579,10 @@ int Connection_uLimeSDR::BeginDataSending(const char *buffer, uint32_t length)
 		txSize = length;
 		FT_SetStreamPipe(mFTHandle, FALSE, FALSE, mStreamWrEndPtAddr, txSize);
 	}
-	memset(&contextsToSend[i].inOvLap, 0, sizeof(OVERLAPPED));
-	contextsToSend[i].inOvLap.hEvent = CreateEvent(NULL, false, false, NULL);
+    FT_InitializeOverlapped(mFTHandle, &contextsToSend[i].inOvLap);
 	ftStatus = FT_WritePipe(mFTHandle, mStreamWrEndPtAddr, (unsigned char*)buffer, length, &ulActualBytesSend, &contextsToSend[i].inOvLap);
 	if (ftStatus != FT_IO_PENDING)
-	{
 		return -1;
-	}
 #else
     if (length != txSize)
     {
@@ -605,8 +602,6 @@ int Connection_uLimeSDR::BeginDataSending(const char *buffer, uint32_t length)
         contextsToSend[i].used = false;
         return -1;
     }
-    else
-        contextsToSend[i].transferLock.lock();
 #endif
     return i;
 }
@@ -622,9 +617,8 @@ int Connection_uLimeSDR::WaitForSending(int contextHandle, unsigned int timeout_
     if(contextsToSend[contextHandle].used == true)
     {
 #ifndef __unix__
-		contextsToSend[contextHandle].inOvLap.InternalHigh = 0;
         DWORD dwRet = WaitForSingleObject(contextsToSend[contextHandle].inOvLap.hEvent, timeout_ms);
-		if (dwRet == WAIT_OBJECT_0 || dwRet == WAIT_TIMEOUT)
+		if (dwRet == WAIT_OBJECT_0)
 			return 1;
 #else
         auto t1 = chrono::high_resolution_clock::now();
@@ -633,7 +627,7 @@ int Connection_uLimeSDR::WaitForSending(int contextHandle, unsigned int timeout_
         while(contextsToSend[contextHandle].done.load() == false && std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count() < timeout_ms)
         {
             //blocking not to waste CPU
-            contextsToSend[contextHandle].cv.wait(lck);
+            contextsToSend[contextHandle].cv.wait_for(lck, chrono::milliseconds(timeout_ms));
             t2 = chrono::high_resolution_clock::now();
         }
         return contextsToSend[contextHandle].done == true;
@@ -654,16 +648,16 @@ int Connection_uLimeSDR::FinishDataSending(const char *buffer, uint32_t length,
     if(contextsToSend[contextHandle].used == true)
     {
 #ifndef __unix__
-		ULONG ulActualBytesTransferred ;
-
-		if (GetOverlappedResult(mFTHandle, &contextsToSend[contextHandle].inOvLap, &ulActualBytesTransferred, FALSE)==FALSE)
-		{
-			return -1;
-		}
-		length = ulActualBytesTransferred;
-		CloseHandle(contextsToSend[contextHandle].inOvLap.hEvent);
-		contextsToSend[contextHandle].used = false;
-		return length;
+        ULONG ulActualBytesTransferred ;
+        FT_STATUS ftStatus = FT_OK;
+        ftStatus = FT_GetOverlappedResult(mFTHandle, &contextsToSend[contextHandle].inOvLap, &ulActualBytesTransferred, FALSE);
+        if (ftStatus != FT_OK)
+            length = 0;
+        else
+        length = ulActualBytesTransferred;
+        FT_ReleaseOverlapped(mFTHandle, &contextsToSend[contextHandle].inOvLap);
+	    contextsToSend[contextHandle].used = false;
+	    return length;
 #else
         length = contextsToSend[contextHandle].bytesXfered;
         contextsToSend[contextHandle].used = false;
@@ -686,7 +680,7 @@ void Connection_uLimeSDR::AbortSending()
 	{
 		if (contextsToSend[i].used == true)
 		{
-			CloseHandle(contextsToSend[i].inOvLap.hEvent);
+            FT_ReleaseOverlapped(mFTHandle, &contextsToSend[i].inOvLap);
 			contextsToSend[i].used = false;
 		}
 	}
