@@ -7,16 +7,18 @@
 #include <wx/checkbox.h>
 #include <wx/msgdlg.h>
 #include <wx/radiobox.h>
+#include <wx/radiobut.h>
 
 #include "pnlQSpark.h"
 #include <ciso646>
 #include <map>
 
 #include <FPGA_common.h>
+#include <lms7_device.h>
 using namespace lime;
 using namespace std;
 
-BEGIN_EVENT_TABLE(pnlQSpark,wxFrame)
+BEGIN_EVENT_TABLE(pnlQSpark,wxPanel)
 END_EVENT_TABLE()
 
 pnlQSpark::Register::Register()
@@ -36,7 +38,7 @@ pnlQSpark::pnlQSpark(wxWindow* parent,wxWindowID id, const wxString &title, cons
 
     wxFlexGridSizer* FlexGridSizer1;
 
-    Create(parent, id, title, wxDefaultPosition, wxDefaultSize, style, _T("id"));
+    Create(parent, id, wxDefaultPosition, wxDefaultSize, style, _T("id"));
 #ifdef WIN32
     SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 #endif
@@ -48,16 +50,16 @@ pnlQSpark::pnlQSpark(wxWindow* parent,wxWindowID id, const wxString &title, cons
     wxSize freqTextfieldSize(64, -1);
     mPanelStreamPLL = new wxPanel(this, wxNewId());
     wxFlexGridSizer* sizerPllControls = new wxFlexGridSizer(0, 3, 5, 5);
-    sizerPllControls->Add(new wxStaticText(mPanelStreamPLL, wxID_ANY, _("DAC (MHz):")), 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
+    sizerPllControls->Add(new wxStaticText(mPanelStreamPLL, wxID_ANY, _("DAC :")), 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
     txtPllFreqTxMHz = new wxTextCtrl(mPanelStreamPLL, wxNewId(), _("160"), wxDefaultPosition, freqTextfieldSize);
     sizerPllControls->Add(txtPllFreqTxMHz, 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
-    lblRealFreqTx = new wxStaticText(mPanelStreamPLL, wxID_ANY, _("Real: ? MHz"));
+    lblRealFreqTx = new wxStaticText(mPanelStreamPLL, wxID_ANY, _("MHz"));
     sizerPllControls->Add(lblRealFreqTx, 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
 
-    sizerPllControls->Add(new wxStaticText(mPanelStreamPLL, wxID_ANY, _("ADC (MHz):")), 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
+    sizerPllControls->Add(new wxStaticText(mPanelStreamPLL, wxID_ANY, _("ADC :")), 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
     txtPllFreqRxMHz = new wxTextCtrl(mPanelStreamPLL, wxNewId(), _("160"), wxDefaultPosition, freqTextfieldSize);
     sizerPllControls->Add(txtPllFreqRxMHz, 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
-    lblRealFreqRx = new wxStaticText(mPanelStreamPLL, wxID_ANY, _("Real: ? MHz"));
+    lblRealFreqRx = new wxStaticText(mPanelStreamPLL, wxID_ANY, _("MHz"));
     sizerPllControls->Add(lblRealFreqRx, 1, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL, 5);
 
     btnConfigurePLL = new wxButton(mPanelStreamPLL, wxNewId(), _("Configure"));
@@ -70,14 +72,29 @@ pnlQSpark::pnlQSpark(wxWindow* parent,wxWindowID id, const wxString &title, cons
     streamPllGroup->Fit(mPanelStreamPLL);
     FlexGridSizer1->Add(mPanelStreamPLL, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
 
-    wxFlexGridSizer* moduleEnables = new wxFlexGridSizer(0, 2, 0, 0);
+    wxStaticBoxSizer* chBox = new wxStaticBoxSizer(wxVERTICAL, this, _T(""));
+    wxFlexGridSizer* chSizer = new wxFlexGridSizer(6, 0, 0, 0);
+    wxFlexGridSizer* chSelect = new wxFlexGridSizer(0, 2, 0, 0);
+    rbChannelA = new wxRadioButton(this, wxNewId(), wxT("A CHANNEL"), wxDefaultPosition, wxDefaultSize, 0);
+    rbChannelA->SetValue(true);
+    rbChannelA->Connect(wxEVT_COMMAND_RADIOBUTTON_SELECTED, wxCommandEventHandler(pnlQSpark::OnSwitchToChannelA), NULL, this);
+    chSelect->Add(rbChannelA, 0, wxEXPAND, 5);
+
+    rbChannelB = new wxRadioButton(this, wxNewId(), wxT("B CHANNEL"), wxDefaultPosition, wxDefaultSize, 0);
+    chSelect->Add(rbChannelB, 0, wxEXPAND, 5);
+    rbChannelB->Connect(wxEVT_COMMAND_RADIOBUTTON_SELECTED, wxCommandEventHandler(pnlQSpark::OnSwitchToChannelB), NULL, this);
+    chSizer->Add(chSelect, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP | wxALL, 5);
+    chBox->Add(chSizer, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP | wxALL, 5);
+    FlexGridSizer1->Add(chBox, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
+
+    wxStaticBoxSizer* moduleEnables = new wxStaticBoxSizer(wxHORIZONTAL, this, _T(""));
     chkEN_RXTSP = new wxCheckBox(this, wxNewId(), _("Enable RxTSP"));
     moduleEnables->Add(chkEN_RXTSP, 1, wxALIGN_LEFT | wxALIGN_TOP, 5);
     Connect(chkEN_RXTSP->GetId(), wxEVT_CHECKBOX, wxCommandEventHandler(pnlQSpark::RegisterParameterChangeHandler), NULL, this);
     chkEN_TXTSP = new wxCheckBox(this, wxNewId(), _("Enable TxTSP"));
     moduleEnables->Add(chkEN_TXTSP, 1, wxALIGN_LEFT | wxALIGN_TOP, 5);
     Connect(chkEN_TXTSP->GetId(), wxEVT_CHECKBOX, wxCommandEventHandler(pnlQSpark::RegisterParameterChangeHandler), NULL, this);
-    FlexGridSizer1->Add(moduleEnables, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP | wxALL, 5);
+    chSizer->Add(moduleEnables, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP | wxALL, 5);
 
     wxStaticBoxSizer* bypassesGroup = new wxStaticBoxSizer(wxVERTICAL, this, _T("Bypass"));
     wxFlexGridSizer* bypasses = new wxFlexGridSizer(0, 2, 0, 0);
@@ -106,7 +123,7 @@ pnlQSpark::pnlQSpark(wxWindow* parent,wxWindowID id, const wxString &title, cons
     Connect(chkTX_DCCORR_BYP->GetId(), wxEVT_CHECKBOX, wxCommandEventHandler(pnlQSpark::RegisterParameterChangeHandler), NULL, this);
 
     bypassesGroup->Add(bypasses , 1, wxALIGN_LEFT | wxALIGN_TOP, 5);
-    FlexGridSizer1->Add(bypassesGroup, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
+    chSizer->Add(bypassesGroup, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
 
     wxFlexGridSizer* tx_correctorsSizer = new wxFlexGridSizer(0, 2, 0, 5);
     wxSize spinBoxSize(64, -1);
@@ -175,7 +192,7 @@ pnlQSpark::pnlQSpark(wxWindow* parent,wxWindowID id, const wxString &title, cons
 
     wxStaticBoxSizer* correctorsGroup = new wxStaticBoxSizer(wxVERTICAL, this, _T("Correctors"));
     correctorsGroup->Add(correctorsSizer, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
-    FlexGridSizer1->Add(correctorsGroup, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
+    chSizer->Add(correctorsGroup, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
 
     txtNcoFreq = new wxTextCtrl(this, wxNewId(), _("1.0"), wxDefaultPosition, freqTextfieldSize);
     wxArrayString insel_choices;
@@ -195,12 +212,7 @@ pnlQSpark::pnlQSpark(wxWindow* parent,wxWindowID id, const wxString &title, cons
     Connect(txtNcoFreq->GetId(), wxEVT_TEXT_ENTER, wxCommandEventHandler(pnlQSpark::OnNcoFrequencyChanged), NULL, this);
 
     loopbackGroup->Add(loopbackSizer, 1, wxALIGN_LEFT | wxALIGN_TOP | wxEXPAND, 5);
-    FlexGridSizer1->Add(loopbackGroup, 1, wxALIGN_LEFT | wxALIGN_TOP | wxEXPAND, 5);
-
-
-    btnUpdateAll = new wxButton(this, wxNewId(), _T("Refresh All"), wxDefaultPosition, wxDefaultSize, 0, wxDefaultValidator, _T("ID_BUTTON1"));
-    Connect(btnUpdateAll->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&pnlQSpark::OnbtnUpdateAll);
-    FlexGridSizer1->Add(btnUpdateAll, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
+    chSizer->Add(loopbackGroup, 1, wxALIGN_LEFT | wxALIGN_TOP | wxEXPAND, 5);
 
     mainSizer->Add(FlexGridSizer1, 1, wxEXPAND | wxALIGN_LEFT | wxALIGN_TOP, 5);
 
@@ -209,38 +221,33 @@ pnlQSpark::pnlQSpark(wxWindow* parent,wxWindowID id, const wxString &title, cons
     Layout();
 
     controlsPtr2Registers.clear();
-    controlsPtr2Registers[chkEN_RXTSP] = Register(0x0080, 0, 0, 0);
-    controlsPtr2Registers[chkEN_TXTSP] = Register(0x0060, 0, 0, 0);
+    controlsPtr2Registers[chkEN_RXTSP] = Register(0x00A0, 0, 0, 0);
+    controlsPtr2Registers[chkEN_TXTSP] = Register(0x0080, 0, 0, 0);
 
-    controlsPtr2Registers[chkRX_DCCORR_BYP] = Register(0x008C, 2, 2, 0);
-    controlsPtr2Registers[chkRX_PHCORR_BYP] = Register(0x008C, 0, 0, 0);
-    controlsPtr2Registers[chkRX_GCORR_BYP] = Register(0x008C, 1, 1, 0);
-    controlsPtr2Registers[chkTX_PHCORR_BYP] = Register(0x0068, 0, 0, 0);
-    controlsPtr2Registers[chkTX_GCORR_BYP] = Register(0x0068, 1, 1, 0);
-    controlsPtr2Registers[chkTX_DCCORR_BYP] = Register(0x0068, 3, 3, 0);
+    controlsPtr2Registers[chkRX_DCCORR_BYP] = Register(0x00AC, 2, 2, 0);
+    controlsPtr2Registers[chkRX_PHCORR_BYP] = Register(0x00AC, 0, 0, 0);
+    controlsPtr2Registers[chkRX_GCORR_BYP] = Register(0x00AC, 1, 1, 0);
+    controlsPtr2Registers[chkTX_PHCORR_BYP] = Register(0x0088, 0, 0, 0);
+    controlsPtr2Registers[chkTX_GCORR_BYP] = Register(0x0088, 1, 1, 0);
+    controlsPtr2Registers[chkTX_DCCORR_BYP] = Register(0x0088, 3, 3, 0);
 
-    controlsPtr2Registers[spinTX_GCORRQ] = Register(0x0061, 10, 0, 2047);
-    controlsPtr2Registers[spinTX_GCORRI] = Register(0x0062, 10, 0, 2047);
-    controlsPtr2Registers[spinTX_PHCORR] = Register(0x0063, 11, 0, 0);
-    controlsPtr2Registers[spinTX_DCCORRI] = Register(0x0064, 15, 8, 0);
-    controlsPtr2Registers[spinTX_DCCORRQ] = Register(0x0064, 7, 0, 0);
+    controlsPtr2Registers[spinTX_GCORRQ] = Register(0x0081, 10, 0, 2047);
+    controlsPtr2Registers[spinTX_GCORRI] = Register(0x0082, 10, 0, 2047);
+    controlsPtr2Registers[spinTX_PHCORR] = Register(0x0083, 11, 0, 0);
+    controlsPtr2Registers[spinTX_DCCORRI] = Register(0x0084, 15, 8, 0);
+    controlsPtr2Registers[spinTX_DCCORRQ] = Register(0x0084, 7, 0, 0);
 
-    controlsPtr2Registers[spinRX_GCORRQ] = Register(0x0081, 10, 0, 2047);
-    controlsPtr2Registers[spinRX_GCORRI] = Register(0x0082, 10, 0, 2047);
-    controlsPtr2Registers[spinRX_PHCORR] = Register(0x0083, 11, 0, 0);
+    controlsPtr2Registers[spinRX_GCORRQ] = Register(0x00A1, 10, 0, 2047);
+    controlsPtr2Registers[spinRX_GCORRI] = Register(0x00A2, 10, 0, 2047);
+    controlsPtr2Registers[spinRX_PHCORR] = Register(0x00A3, 11, 0, 0);
 
-    controlsPtr2Registers[cmbInsel] = Register(0x0060, 2, 2, 0);
+    controlsPtr2Registers[cmbInsel] = Register(0x0080, 2, 2, 0);
 }
 
 void pnlQSpark::Initialize(lms_device_t* pControl)
 {
     lmsControl = pControl;
-}
-
-void pnlQSpark::UpdatePanel()
-{
-    wxCommandEvent evt;
-    OnbtnUpdateAll(evt);
+    LMS_WriteFPGAReg(lmsControl, 0x001F, rbChannelB->GetValue() ? 0x2 : 0x1);
 }
 
 void pnlQSpark::RegisterParameterChangeHandler(wxCommandEvent& event)
@@ -301,38 +308,44 @@ void pnlQSpark::OnbtnUpdateAll(wxCommandEvent& event)
         else if(iter->first->IsKindOf(choicectr))
             reinterpret_cast<wxComboBox*>(iter->first)->SetSelection(value);
     }
+
+    double refClk_MHz, ncoFreq_MHz;
+    txtPllFreqTxMHz->GetValue().ToDouble(&refClk_MHz);
+    uint32_t fcw = 0;
+    vector<uint32_t> addrs = { 0x008E, 0x008F };
+
+    for (size_t i = 0; i <addrs.size(); i++)
+    {
+        uint16_t value;
+        if (LMS_ReadFPGAReg(lmsControl, addrs[i], &value) != 0)
+        {
+            wxMessageBox(LMS_GetLastErrorMessage(), _("Error"), wxICON_ERROR | wxOK);
+            return;
+        }
+        fcw <<= 16;
+        fcw += value;
+    }
+    ncoFreq_MHz = fcw * refClk_MHz / 4294967296;
+    txtNcoFreq->SetValue(wxString::Format("%1.3f", ncoFreq_MHz));
 }
 
 void pnlQSpark::OnConfigurePLL(wxCommandEvent &event)
 {
+    if (!LMS_IsOpen(lmsControl, 1))
+    {
+        wxMessageBox(_("device not connected"), _("Error"), wxICON_ERROR | wxOK);
+        return;
+    }
+
     double FreqTxMHz, FreqRxMHz;
     txtPllFreqTxMHz->GetValue().ToDouble(&FreqTxMHz);
     txtPllFreqRxMHz->GetValue().ToDouble(&FreqRxMHz);
 
-    lime::fpga::FPGA_PLL_clock clocks[2];
-    //ADC
-    clocks[0].bypass = false;
-    clocks[0].index = 0;
-    clocks[0].outFrequency = FreqRxMHz*1e6;
-    clocks[0].phaseShift_deg = 0;
-
-    //DAC
-    clocks[1].bypass = false;
-    clocks[1].index = 1;
-    clocks[1].outFrequency = FreqTxMHz*1e6;
-    clocks[1].phaseShift_deg = 0;
-
-//  TODO
-    wxMessageBox(_("Not implemented"), _("Error"), wxICON_ERROR | wxOK);
-    return;
-//  if(lime::fpga::SetPllFrequency(m_serPort, 2, 30.72e6, clocks, 2) != 0)
-//      wxMessageBox(GetLastErrorMessage(), _("Error"), wxICON_ERROR | wxOK);
-//  else
-    {
+    auto conn = ((LMS7_Device*)lmsControl)->GetConnection();
+    if(conn->UpdateExternalDataRate(2,FreqTxMHz*1e6,FreqRxMHz*1e6,0,0))
+        wxMessageBox(LMS_GetLastErrorMessage(), _("Error"), wxICON_ERROR | wxOK);
+    else
         OnNcoFrequencyChanged(event);
-        lblRealFreqTx->SetLabel(wxString::Format("Real: %g MHz", clocks[1].rd_actualFrequency / 1e6));
-        lblRealFreqRx->SetLabel(wxString::Format("Real: %g MHz", clocks[0].rd_actualFrequency / 1e6));
-    }
 }
 
 void pnlQSpark::OnNcoFrequencyChanged(wxCommandEvent& event)
@@ -341,7 +354,7 @@ void pnlQSpark::OnNcoFrequencyChanged(wxCommandEvent& event)
     txtPllFreqTxMHz->GetValue().ToDouble(&refClk_MHz);
     txtNcoFreq->GetValue().ToDouble(&ncoFreq_MHz);
     uint32_t fcw = (uint32_t)((ncoFreq_MHz / refClk_MHz) * 4294967296);
-    vector<uint32_t> addrs = { 0x006E, 0x006F };
+    vector<uint32_t> addrs = { 0x008E, 0x008F };
     vector<uint32_t> values = { (fcw >> 16) & 0xFFFF, fcw & 0xFFFF };
     if (!LMS_IsOpen(lmsControl,1))
     {
@@ -358,3 +371,36 @@ void pnlQSpark::OnNcoFrequencyChanged(wxCommandEvent& event)
         }
     }
 }
+
+void pnlQSpark::OnReadAll(wxCommandEvent &event)
+{
+     OnbtnUpdateAll(event);
+}
+
+void pnlQSpark::OnWriteAll(wxCommandEvent &event)
+{
+    OnNcoFrequencyChanged(event);
+    OnConfigurePLL(event);
+}
+
+void pnlQSpark::OnSwitchToChannelA(wxCommandEvent& event)
+{
+    if (LMS_WriteFPGAReg(lmsControl, 0x001F, 0x1) != 0)
+    {
+        wxMessageBox(LMS_GetLastErrorMessage(), _("Error"), wxICON_ERROR | wxOK);
+        return;
+    }
+    wxCommandEvent evt;
+    OnbtnUpdateAll(evt);
+}
+
+void pnlQSpark::OnSwitchToChannelB(wxCommandEvent& event)
+{
+    if (LMS_WriteFPGAReg(lmsControl, 0x001F, 0x2) != 0)
+    {
+        wxMessageBox(LMS_GetLastErrorMessage(), _("Error"), wxICON_ERROR | wxOK);
+        return;
+    }
+    wxCommandEvent evt;
+    OnbtnUpdateAll(evt);
+}
