@@ -9,13 +9,20 @@
 #include "LMS7002M.h"
 #include "windowFunction.h"
 #include <fstream>
+#include "lms7suiteEvents.h"
 
 using namespace std;
 using namespace lime;
 
-void fftviewer_frFFTviewer::Initialize(lms_device_t* pDataPort)
+void fftviewer_frFFTviewer::Initialize(lms_device_t* pDataPort, unsigned index)
 {
     lmsControl = pDataPort;
+    lmsIndex = index;
+    for (unsigned i =0; i < this->cMaxChCount ; i++)
+    {
+        this->rxStreams[i].handle = 0;
+        this->txStreams[i].handle = 0;
+    }
 }
 
 fftviewer_frFFTviewer::fftviewer_frFFTviewer( wxWindow* parent )
@@ -25,7 +32,7 @@ mStreamRunning(false),
 lmsControl(nullptr)
 {
     captureSamples.store(false);
-    averageCount.store(10);
+    averageCount.store(50);
     spinAvgCount->SetValue(averageCount);
     updateGUI.store(true);
 #ifndef __unix__
@@ -152,10 +159,14 @@ void fftviewer_frFFTviewer::StartStreaming()
         captureSamples.store(false);
     chkCaptureToFile->Disable();
     spinCaptureCount->Disable();
+    chkEnTx->Disable();
 
     switch (cmbStreamType->GetSelection())
     {
     case 0: //SISO
+        if (cmbChannelVisibility->GetSelection() > 1)
+            cmbChannelVisibility->SetSelection(0);
+        cmbChannelVisibility->Disable();
         threadProcessing = std::thread(StreamingLoop, this, fftSize, 1, 0);
         break;
     case 1: //MIMO
@@ -175,11 +186,13 @@ void fftviewer_frFFTviewer::StopStreaming()
         return;
     stopProcessing.store(true);
     threadProcessing.join();
-	btnStartStop->SetLabel(_("START"));
-	cmbStreamType->Enable();
-	spinFFTsize->Enable();
-	chkCaptureToFile->Enable();
-	spinCaptureCount->Enable();
+    btnStartStop->SetLabel(_("START"));
+    cmbStreamType->Enable();
+    spinFFTsize->Enable();
+    chkCaptureToFile->Enable();
+    spinCaptureCount->Enable();
+    cmbChannelVisibility->Enable();
+    chkEnTx->Enable();
 }
 
 void fftviewer_frFFTviewer::OnUpdateStats(wxTimerEvent& event)
@@ -194,6 +207,11 @@ void fftviewer_frFFTviewer::OnUpdateStats(wxTimerEvent& event)
         gaugeRxBuffer->SetValue((int)RxFilled);
         lblRxDataRate->SetLabel(printDataRate(rxStats.linkRate));
     }
+    else
+    {
+        gaugeRxBuffer->SetValue(0);
+        lblRxDataRate->SetLabel(printDataRate(0));
+    }
     if(txStreams[0].handle != 0)
     {
         lms_stream_status_t txStats;
@@ -202,6 +220,11 @@ void fftviewer_frFFTviewer::OnUpdateStats(wxTimerEvent& event)
         gaugeTxBuffer->SetValue((int)TxFilled);
         lblTxDataRate->SetLabel(printDataRate(txStats.linkRate));
     }
+    else
+    {
+        gaugeTxBuffer->SetValue(0);
+        lblTxDataRate->SetLabel(printDataRate(0));
+    }
 }
 
 void fftviewer_frFFTviewer::OnUpdatePlots(wxThreadEvent& event)
@@ -209,7 +232,38 @@ void fftviewer_frFFTviewer::OnUpdatePlots(wxThreadEvent& event)
     if (mStreamRunning.load() == false)
         return;
 
-    if (streamData.fftBins_dbFS[0].size() > 0)
+    const int fftSize = streamData.fftBins[0].size();
+
+    float chPwr[2] = { 0, 0 };
+    double cFreq[2] = { 0, 0 };
+    txtCenterOffset1->GetValue().ToDouble(&cFreq[0]);
+    txtCenterOffset2->GetValue().ToDouble(&cFreq[1]);
+    double bw[2] = {1, 1};
+    txtBW1->GetValue().ToDouble(&bw[0]);
+    txtBW2->GetValue().ToDouble(&bw[1]);
+
+    for (int c = 0; c<2; ++c)
+    {
+        float f0 = (cFreq[c] - bw[c]/2) * 1e6;
+        float fn = (cFreq[c] + bw[c]/2) * 1e6;
+        float sum = 0;
+        int bins = 0;
+        for (int i = 0; i<fftSize; ++i)
+            if (f0 <= fftFreqAxis[i] && fftFreqAxis[i] <= fn)
+            {
+                sum += streamData.fftBins[0][i];
+                ++bins;
+            }
+        chPwr[c] = sum;
+    }
+
+    float pwr1 = (chPwr[0] != 0 ? (10 * log10(chPwr[0])) - 69.2369 : -300);
+    lblPower1->SetLabel(wxString::Format("%.3f", pwr1));
+    float pwr2 = (chPwr[1] != 0 ? (10 * log10(chPwr[1])) - 69.2369 : -300);
+    lblPower2->SetLabel(wxString::Format("%.3f", pwr2));
+    lbldBc->SetLabel(wxString::Format("%.3f", pwr2-pwr1));
+
+    if (fftSize > 0)
     {
         if (chkFreezeTimeDomain->IsChecked() == false)
         {
@@ -225,8 +279,15 @@ void fftviewer_frFFTviewer::OnUpdatePlots(wxThreadEvent& event)
         }
         if (chkFreezeFFT->IsChecked() == false)
         {
-            mFFTpanel->series[0]->AssignValues(&fftFreqAxis[0], &streamData.fftBins_dbFS[0][0], streamData.fftBins_dbFS[0].size());
-            mFFTpanel->series[1]->AssignValues(&fftFreqAxis[0], &streamData.fftBins_dbFS[1][0], streamData.fftBins_dbFS[1].size());
+            for (int ch = 0; ch<2; ++ch)
+            {
+                for (int s = 0; s < fftSize; ++s)
+                {
+                    streamData.fftBins[ch][s] = (streamData.fftBins[ch][s] != 0 ? (10 * log10(streamData.fftBins[ch][s])) - 69.2369 : -300);
+                }
+            }
+            mFFTpanel->series[0]->AssignValues(&fftFreqAxis[0], &streamData.fftBins[0][0], fftSize);
+            mFFTpanel->series[1]->AssignValues(&fftFreqAxis[0], &streamData.fftBins[1][0], fftSize);
         }
     }
 
@@ -246,73 +307,42 @@ void fftviewer_frFFTviewer::OnUpdatePlots(wxThreadEvent& event)
     {
         mFFTpanel->Refresh();
         mFFTpanel->Draw();
+        enableFFT.store(true);
     }
-
-    float chPwr[2] = {0, 0};
-    double cFreq[2] = {0, 0};
-    txtCenterOffset1->GetValue().ToDouble(&cFreq[0]);
-    cFreq[0] *= 1000000;
-    txtCenterOffset2->GetValue().ToDouble(&cFreq[1]);
-    cFreq[1] *= 1000000;
-    double bw[2] = {1e6, 1e6};
-    txtBW1->GetValue().ToDouble(&bw[0]);
-    bw[0] *= 1000000;
-    txtBW2->GetValue().ToDouble(&bw[1]);
-    bw[1] *= 1000000;
-    /*
-    // TODO
-    for(int c=0; c<2; ++c)
-    {
-        float f0 = cFreq[c]-bw[c]/2;
-        float fn = cFreq[c]+bw[c]/2;
-        float sum = 0;
-        int bins = 0;
-        const int fftSize = streamData.fftBins_dbFS[0].size();
-        for(int i=0; i<fftSize; ++i)
-            if(f0 <= fftFreqAxis[i] && fftFreqAxis[i] <= fn)
-            {
-                double val = streamData.fftBins_dbFS[0][i];
-                sum += val;
-                ++bins;
-            }
-        chPwr[c] = sum;
-    }
-    */
-
-    float pwr1  = (chPwr[0] != 0 ? (20 * log10(chPwr[0])) - 69.2369 : -300);
-    lblPower1->SetLabel(wxString::Format("%.3f", pwr1));
-    float pwr2 = (chPwr[1] != 0 ? (20 * log10(chPwr[1])) - 69.2369 : -300);
-    lblPower2->SetLabel(wxString::Format("%.3f", pwr2));
-    lbldBc->SetLabel(wxString::Format("%.3f", 20*log10(chPwr[1]/chPwr[0])));
+    else enableFFT.store(false);
 
     updateGUI.store(true);
 }
 
 void fftviewer_frFFTviewer::StreamingLoop(fftviewer_frFFTviewer* pthis, const unsigned int fftSize, const int channelsCount, const uint32_t format)
 {
-    bool runTx = true;
-    const int cMaxChCount = 2;
-    const int test_count = fftSize;// / 680*26;//4096*16;
+    const bool runTx = pthis->chkEnTx->GetValue();
+    const int fifoSize = fftSize*512;
     int avgCount = pthis->spinAvgCount->GetValue();
     int wndFunction = pthis->windowFunctionID.load();
-
+    bool fftEnabled = true;
+    int ch_offset = 0;
+    if (channelsCount == 1)
+    {
+        if (pthis->cmbChannelVisibility->GetSelection() == 1)
+            ch_offset = 1;
+    }
     vector<float> wndCoef;
-    float amplitudeCorrection = 1;
-    GenerateWindowCoefficients(wndFunction, fftSize, wndCoef, amplitudeCorrection);
+    GenerateWindowCoefficients(wndFunction, fftSize, wndCoef, 1);
 
     lime::complex16_t** buffers;
 
     DataToGUI localDataResults;
     localDataResults.nyquist_Hz = 7.68e6;
-    localDataResults.samplesI[0].resize(fftSize, 0);
-    localDataResults.samplesI[1].resize(fftSize, 0);
-    localDataResults.samplesQ[0].resize(fftSize, 0);
-    localDataResults.samplesQ[1].resize(fftSize, 0);
-    localDataResults.fftBins_dbFS[0].resize(fftSize, 0);
-    localDataResults.fftBins_dbFS[1].resize(fftSize, 0);
+    for (unsigned i = 0; i < cMaxChCount; i++)
+    {
+        localDataResults.samplesI[i].resize(fftSize, 0);
+        localDataResults.samplesQ[i].resize(fftSize, 0);
+        localDataResults.fftBins[i].resize(fftSize, 0);
+    }
     buffers = new lime::complex16_t*[channelsCount];
     for (int i = 0; i < channelsCount; ++i)
-        buffers[i] = new complex16_t[test_count];
+        buffers[i] = new complex16_t[fftSize];
 
     vector<complex16_t> captureBuffer[cMaxChCount];
     uint32_t samplesToCapture[cMaxChCount];
@@ -323,19 +353,22 @@ void fftviewer_frFFTviewer::StreamingLoop(fftviewer_frFFTviewer* pthis, const un
             captureBuffer[ch].resize(samplesToCapture[ch]);
         }
 
+    if ((pthis->lmsIndex == 1) && (LMS_GetNumChannels(pthis->lmsControl, false)>2))
+        ch_offset = 2;
+
     auto fmt = lms_stream_t::LMS_FMT_I12;
     for(int i=0; i<channelsCount; ++i)
     {
-        pthis->rxStreams[i].channel = i;
-        pthis->rxStreams[i].fifoSize = test_count*40;
+        pthis->rxStreams[i].channel = i + ch_offset;
+        pthis->rxStreams[i].fifoSize = fifoSize;
         pthis->rxStreams[i].isTx = false;
         pthis->rxStreams[i].dataFmt = fmt;
         pthis->rxStreams[i].throughputVsLatency = 1;
         LMS_SetupStream(pthis->lmsControl, &pthis->rxStreams[i]);
 
         pthis->txStreams[i].handle = 0;
-        pthis->txStreams[i].channel = i;
-        pthis->txStreams[i].fifoSize = test_count*40;
+        pthis->txStreams[i].channel = i + ch_offset;
+        pthis->txStreams[i].fifoSize = fifoSize;
         pthis->txStreams[i].isTx = true;
         pthis->txStreams[i].dataFmt = fmt;
         pthis->txStreams[i].throughputVsLatency = 1;
@@ -353,32 +386,32 @@ void fftviewer_frFFTviewer::StreamingLoop(fftviewer_frFFTviewer* pthis, const un
         if(runTx)
             LMS_StartStream(&pthis->txStreams[i]);
     }
+    wxCommandEvent evt;
+    evt.SetEventType(SAMPLE_POS_CHANGED);
+    wxPostEvent(pthis->GetParent(), evt);
 
     pthis->mStreamRunning.store(true);
-    auto t1 = chrono::high_resolution_clock::now();
-    auto t2 = chrono::high_resolution_clock::now();
     lms_stream_meta_t meta;
     meta.waitForTimestamp = true;
     int fftCounter = 0;
 
     while (pthis->stopProcessing.load() == false)
     {
-        for(int a = fftCounter; a<avgCount && pthis->stopProcessing.load() == false; ++a)
+        do
         {
-            uint32_t samplesPopped[2] = {0, 0};
-            uint64_t ts[2];
+            uint32_t samplesPopped[cMaxChCount];
+            uint64_t ts[cMaxChCount];
             for(int i=0; i<channelsCount; ++i)
             {
-                samplesPopped[i] += LMS_RecvStream(&pthis->rxStreams[i], &buffers[i][samplesPopped[i]], test_count, &meta, 1000);
-                ts[i] = meta.timestamp + 1024 * 1024*2;
+                samplesPopped[i] = LMS_RecvStream(&pthis->rxStreams[i], &buffers[i][0], fftSize, &meta, 1000);
+                ts[i] = meta.timestamp + fifoSize/4;
             }
 
-            uint32_t samplesPushed[2] = {0, 0};
             for(int i=0; runTx && i<channelsCount; ++i)
             {
                 meta.timestamp = ts[i];
                 meta.waitForTimestamp = true;
-                samplesPushed[i] += LMS_SendStream(&pthis->txStreams[i], &buffers[i][samplesPushed[i]], samplesPopped[i], &meta, 1000);
+                LMS_SendStream(&pthis->txStreams[i], &buffers[i][0], samplesPopped[i], &meta, 1000);
             }
 
             if(pthis->captureSamples.load())
@@ -397,45 +430,55 @@ void fftviewer_frFFTviewer::StreamingLoop(fftviewer_frFFTviewer* pthis, const un
             {
                 //take only first buffer for time domain display
                 //reset fftBins for accumulation
-                for (unsigned i = 0; a==0 && i < fftSize; ++i)
+                for (unsigned i = 0; fftCounter==0 && i < fftSize; ++i)
                 {
-                    localDataResults.fftBins_dbFS[ch][i] = 0;
+                    if (fftEnabled)
+                        localDataResults.fftBins[ch][i] = 0;
                     localDataResults.samplesI[ch][i] = buffers[ch][i].i;
                     localDataResults.samplesQ[ch][i] = buffers[ch][i].q;
                 }
-                for (unsigned i = 0; i < fftSize; ++i)
+                if (fftEnabled)
                 {
-                    m_fftCalcIn[i].r = buffers[ch][i].i * amplitudeCorrection * wndCoef[i];
-                    m_fftCalcIn[i].i = buffers[ch][i].q * amplitudeCorrection * wndCoef[i];
-                }
+                    if (wndFunction == 0)
+                    {
+                        for (unsigned i = 0; i < fftSize; ++i)
+                        {
+                            m_fftCalcIn[i].r = buffers[ch][i].i;
+                            m_fftCalcIn[i].i = buffers[ch][i].q;
+                        }
+                    }
+                    else
+                    {
+                        for (unsigned i = 0; i < fftSize; ++i)
+                        {
+                            m_fftCalcIn[i].r = buffers[ch][i].i * wndCoef[i];
+                            m_fftCalcIn[i].i = buffers[ch][i].q * wndCoef[i];
+                        }
+                    }
 
-                kiss_fft(m_fftCalcPlan, m_fftCalcIn, m_fftCalcOut);
-                ++fftCounter;
-                for(unsigned int i=0; i<fftSize; ++i)
-                {
-                    m_fftCalcOut[i].r /= fftSize;
-                    m_fftCalcOut[i].i /= fftSize;
+                    kiss_fft(m_fftCalcPlan, m_fftCalcIn, m_fftCalcOut);
+
+                    int output_index = 0;
+                    for (unsigned i = fftSize / 2 + 1; i < fftSize; ++i)
+                        localDataResults.fftBins[ch][output_index++] += m_fftCalcOut[i].r * m_fftCalcOut[i].r + m_fftCalcOut[i].i * m_fftCalcOut[i].i;
+                    for (unsigned i = 0; i < fftSize / 2 + 1; ++i)
+                        localDataResults.fftBins[ch][output_index++] += m_fftCalcOut[i].r * m_fftCalcOut[i].r + m_fftCalcOut[i].i * m_fftCalcOut[i].i;
                 }
-                int output_index = 0;
-                for (unsigned i = fftSize / 2 + 1; i < fftSize; ++i)
-                    localDataResults.fftBins_dbFS[ch][output_index++] += sqrt(m_fftCalcOut[i].r * m_fftCalcOut[i].r + m_fftCalcOut[i].i * m_fftCalcOut[i].i);
-                for (unsigned i = 0; i < fftSize / 2 + 1; ++i)
-                    localDataResults.fftBins_dbFS[ch][output_index++] += sqrt(m_fftCalcOut[i].r * m_fftCalcOut[i].r + m_fftCalcOut[i].i * m_fftCalcOut[i].i);
             }
-        }
+        } while (++fftCounter < avgCount && pthis->stopProcessing.load() == false);
 
-        t2 = chrono::high_resolution_clock::now();
-        auto timePeriod = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
-        if (timePeriod >= 33 && fftCounter >= avgCount && pthis->updateGUI.load() == true)
+        if (fftCounter >= avgCount && pthis->updateGUI.load() == true)
         {
-            t1 = t2;
-            //shift fft, convert to dBfs
-            for(int ch=0; ch<channelsCount; ++ch)
+            //shift fft
+            if (fftEnabled)
             {
-                for (unsigned s = 0; s < fftSize; ++s)
+                for(int ch=0; ch<channelsCount; ++ch)
                 {
-                    localDataResults.fftBins_dbFS[ch][s] /= fftCounter;
-                    localDataResults.fftBins_dbFS[ch][s] = (localDataResults.fftBins_dbFS[ch][s] != 0 ? (20 * log10(localDataResults.fftBins_dbFS[ch][s])) - 69.2369 : -300);
+                    for (unsigned s = 0; s < fftSize; ++s)
+                    {
+                        const float div = (float)fftCounter*fftSize*fftSize;
+                        localDataResults.fftBins[ch][s] /= div;
+                    }
                 }
             }
             if(pthis->stopProcessing.load() == false)
@@ -447,13 +490,14 @@ void fftviewer_frFFTviewer::StreamingLoop(fftviewer_frFFTviewer* pthis, const un
                 pthis->QueueEvent(evt);
             }
             fftCounter = 0;
-        }
-        avgCount = pthis->averageCount.load();
-        int wndFunctionSelection = pthis->windowFunctionID.load();
-        if(wndFunctionSelection != wndFunction)
-        {
-            wndFunction = wndFunctionSelection;
-            GenerateWindowCoefficients(wndFunction, fftSize, wndCoef, amplitudeCorrection);
+            fftEnabled = pthis->enableFFT.load();
+            avgCount = pthis->averageCount.load();
+            int wndFunctionSelection = pthis->windowFunctionID.load();
+            if(wndFunctionSelection != wndFunction)
+            {
+                wndFunction = wndFunctionSelection;
+                GenerateWindowCoefficients(wndFunction, fftSize, wndCoef, 1);
+            }
         }
     }
 
@@ -475,6 +519,25 @@ void fftviewer_frFFTviewer::StreamingLoop(fftviewer_frFFTviewer* pthis, const un
             }
             fout << endl;
         }
+        fout.close();
+
+        string filename = pthis->captureFilename+".absdbfs";
+        fout.open(filename.c_str());
+        fout << "AI\tAQ";
+        if(channelsCount > 1)
+            fout << "\tBI\tBQ";
+        fout << endl;
+
+        for(int i=0; i<samplesCnt; ++i)
+        {
+            for(int ch=0; ch<channelsCount; ++ch)
+            {
+                fout
+                << (captureBuffer[ch][i].i == 0 ? -67 : 20*log10(abs(captureBuffer[ch][i].i)/2048))<< "\t"
+                << (captureBuffer[ch][i].q == 0 ? -67 : 20*log10(abs(captureBuffer[ch][i].q)/2048))<< "\t";
+            }
+            fout << endl;
+        }
     }
 
     kiss_fft_free(m_fftCalcPlan);
@@ -517,23 +580,34 @@ void fftviewer_frFFTviewer::SetNyquistFrequency(float freqHz)
 
 void fftviewer_frFFTviewer::OnChannelVisibilityChange(wxCommandEvent& event)
 {
-    const int channelCount = 2;
-    bool visibilities[channelCount];
+    bool visibilities[cMaxChCount];
 
-    switch(event.GetInt())
+    if (cmbStreamType->GetSelection() == 1)
+    {
+        switch (event.GetInt())
+        {
+        case 0:
+            visibilities[0] = true;
+            visibilities[1] = false;
+            break;
+        case 1:
+            visibilities[0] = false;
+            visibilities[1] = true;
+            break;
+        case 2:
+            visibilities[0] = true;
+            visibilities[1] = true;
+            break;
+        default:
+            visibilities[0] = false;
+            visibilities[1] = false;
+            break;
+        }
+    }
+    else
     {
-    case 0:
         visibilities[0] = true;
         visibilities[1] = false;
-        break;
-    case 1:
-        visibilities[0] = false;
-        visibilities[1] = true;
-        break;
-    case 2:
-        visibilities[0] = true;
-        visibilities[1] = true;
-        break;
     }
     mTimeDomainPanel->series[0]->visible = visibilities[0];
     mTimeDomainPanel->series[1]->visible = visibilities[0];
