@@ -102,8 +102,7 @@ API_EXPORT int CALL_CONV LMS_GetDeviceList(lms_info_str_t *dev_list);
  *
  * @return      0 on success, (-1) on failure
  */
-API_EXPORT int CALL_CONV LMS_Open(lms_device_t **device, lms_info_str_t info,
-                                   void* args);
+API_EXPORT int CALL_CONV LMS_Open(lms_device_t **device, const lms_info_str_t info, void* args);
 
 /**
  * Close device
@@ -116,7 +115,6 @@ API_EXPORT int CALL_CONV LMS_Open(lms_device_t **device, lms_info_str_t info,
  */
 API_EXPORT int CALL_CONV LMS_Close(lms_device_t *device);
 
-
 /**
  * Disconnect device but keep configuration cache (device is not deallocated).
  *
@@ -136,11 +134,8 @@ API_EXPORT int CALL_CONV LMS_Disconnect(lms_device_t *device);
  */
 API_EXPORT bool CALL_CONV LMS_IsOpen(lms_device_t *device, int port);
 
-
 /** @} (End FN_INIT) */
 
-
-
 /**
  * @defgroup FN_HIGH_LVL    High-level control functions
  *
@@ -151,8 +146,8 @@ API_EXPORT bool CALL_CONV LMS_IsOpen(lms_device_t *device, int port);
  */
 
 /**Convenience constants for TX/RX selection*/
-const bool LMS_CH_TX = true;
-const bool LMS_CH_RX = false;
+static const bool LMS_CH_TX = true;
+static const bool LMS_CH_RX = false;
 
 /**Structure used to represent the allowed value range of various parameters*/
 typedef struct
@@ -162,12 +157,6 @@ typedef struct
     float_type step;    /**<Minimum value step*/
 }lms_range_t;
 
-/**Enumeration of loopback modes*/
-typedef enum {
-    LMS_LOOPBACK_NONE   /**<Return to normal operation (disable loopback)*/
-} lms_loopback_t;
-
-
 /**Enumeration of LMS7 TEST signal types*/
 typedef enum
 {
@@ -261,9 +250,7 @@ API_EXPORT int CALL_CONV LMS_GetSampleRateRange(lms_device_t *device, bool dir_t
                                                 lms_range_t *range);
 
 /**
- * Set RF center frequency in Hz. This automatically selects the appropriate
- * antenna (band path) for the desired frequency. In oder to override antenna
- * selection use LMS_SetAntenna().
+ * Set RF center frequency in Hz.
  *
  * @note setting different frequencies for A and B channels is not supported by
  * LMS7 chip. Changing frequency for one (A or B) channel will result in
@@ -319,7 +306,24 @@ API_EXPORT int CALL_CONV LMS_GetLOFrequencyRange(lms_device_t *device,bool dir_t
  * @return  0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_SetNormalizedGain(lms_device_t *device, bool dir_tx,
-                                                size_t chan,float_type gain);
+                                               size_t chan,float_type gain);
+
+/**
+ * Set the combined gain value in dB
+ *
+ * This function computes and sets the optimal gain values of various amplifiers
+ * that are present in the device based on desired  gain value in dB.
+ *
+ * @note currently works only with RX
+ *
+ * @param   device      Device handle previously obtained by LMS_Open().
+ * @param   dir_tx      Select RX or TX
+ * @param   chan        Channel index
+ * @param   gain        Desired gain, range [0, 70] for RX
+ * @return  0 on success, (-1) on failure
+ */
+API_EXPORT int CALL_CONV LMS_SetGaindB(lms_device_t *device, bool dir_tx,
+                                        size_t chan, unsigned gain);
 
 /**
  * Obtain the current combined gain value
@@ -332,30 +336,20 @@ API_EXPORT int CALL_CONV LMS_SetNormalizedGain(lms_device_t *device, bool dir_tx
  * @return  0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_GetNormalizedGain(lms_device_t *device, bool dir_tx,
-                                                size_t chan,float_type *gain);
-
-typedef enum
-{
-	LPF_BW_5_MHz,
-	LPF_BW_10_MHz,
-	LPF_BW_15_MHz,
-	LPF_BW_20_MHz,
-
-	LPF_BW_COUNT
-} LPF_FixedBW;
+                                                size_t chan, float_type *gain);
 /**
- * Configure analog LPF and GFIR of the LMS chip for selected fixed bandwidth.
- * This function automatically enables LPF.
+ * Obtain the current combined gain value in dB
  *
- * @param   device      Device handle previously obtained by LMS_Open().
- * @param   dir_tx      Select RX or TX
- * @param   chan        Channel index
- * @param   bandwidth   Fixed bandwidth selection
+ * @note currently works only with RX
  *
- * @return  0 on success
+ * @param       device      Device handle previously obtained by LMS_Open().
+ * @param       dir_tx      Select RX or TX
+ * @param       chan        Channel index
+ * @param[out]  gain        Current gain, range [0, 700]
+ * @return  0 on success, (-1) on failure
  */
-API_EXPORT int CALL_CONV LMS_SetLPFBWFixed(lms_device_t *device, bool dir_tx,
-                                             size_t chan, LPF_FixedBW bandwidth);
+API_EXPORT int CALL_CONV LMS_GetGaindB(lms_device_t *device, bool dir_tx,
+                                                size_t chan, unsigned *gain);
 
 /**
  * Configure analog LPF of the LMS chip for the desired RF bandwidth.
@@ -374,6 +368,9 @@ API_EXPORT int CALL_CONV LMS_SetLPFBW(lms_device_t *device, bool dir_tx,
 /**
  * Get the currently configured analog LPF RF bandwidth.
  *
+ * @note readback from board is currently not supported, only returns last set
+ * value cached by software.
+ *
  * @param       device      Device handle previously obtained by LMS_Open().
  * @param       dir_tx      Select RX or TX
  * @param       chan        Channel index
@@ -385,7 +382,7 @@ API_EXPORT int CALL_CONV LMS_GetLPFBW(lms_device_t *device, bool dir_tx,
                                             size_t chan, float_type *bandwidth);
 
 /**
- * Get the RF bandwidth setting range supported by the analog LPF of LMS chip
+ * Get the RF bandwidth setting range supported by the analog LPF of LMS chip.
  *
  * @param       device      Device handle previously obtained by LMS_Open().
  * @param       dir_tx      Select RX or TX
@@ -394,8 +391,8 @@ API_EXPORT int CALL_CONV LMS_GetLPFBW(lms_device_t *device, bool dir_tx,
  *
  * @return  0 on success, (-1) on failure
  */
-API_EXPORT int CALL_CONV LMS_GetLPFBWRange(lms_device_t *device,
-                                  bool dir_tx, lms_range_t *range);
+API_EXPORT int CALL_CONV LMS_GetLPFBWRange(lms_device_t *device, bool dir_tx,
+                                            lms_range_t *range);
 
 /**
  * Disables or enables the analog LPF of LMS chip without reconfiguring it.
@@ -432,9 +429,6 @@ API_EXPORT int CALL_CONV LMS_SetGFIRLPF(lms_device_t *device, bool dir_tx,
  * calibration must be run after device configuration is finished because
  * calibration values are dependant on various configuration settings.
  *
- * @note automatic RX calibration is not available when RX_LNA_H path is
- * selected
- *
  * @pre Device should be configured
  *
  * @param   device      Device handle previously obtained by LMS_Open().
@@ -451,6 +445,10 @@ API_EXPORT int CALL_CONV LMS_Calibrate(lms_device_t *device, bool dir_tx,
 /**
  * Load LMS chip configuration from a file
  *
+ * @note this only loads LMS chip configuration, in oder for streaming to work
+ * properly FPGA has also to be configured. Use LMS_SetSampleRate() to configure
+ * LMS and FPGA for streaming.
+ *
  * @param   device      Device handle
  * @param   filename    path to file
  *
@@ -468,29 +466,6 @@ API_EXPORT int CALL_CONV LMS_LoadConfig(lms_device_t *device, const char *filena
  */
 API_EXPORT int CALL_CONV LMS_SaveConfig(lms_device_t *device, const char *filename);
 
-/**
- * Apply the specified loopback mode
- *
- * @param   device      Device handle previously obtained by LMS_Open().
- * @param   mode        Loopback mode. LMS_LOOPBACK_NONE returns to normal
- *                      operation
- *
- * @return  0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_SetLoopback(lms_device_t *device,
-                                         lms_loopback_t mode);
-/**
- * Get the current loopback mode
- *
- * @param       device      Device handle previously obtained by LMS_Open().
- * @param[out]  mode        Current loopback mode
- *
- * @return  0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_GetLoopback(lms_device_t *device,
-                                         lms_loopback_t* mode);
-
-
 /**
  * Apply the specified test signal
  *
@@ -520,7 +495,6 @@ API_EXPORT int CALL_CONV LMS_GetTestSignal(lms_device_t *device, bool dir_tx,
 
 /** @} (End FN_HIGH_LVL) */
 
-
 /**
  * @defgroup FN_ADVANCED    Advanced control functions
  *
@@ -539,10 +513,8 @@ typedef enum
     LMS_GFIR3
 }lms_gfir_t;
 
-
 /**Number of NCO frequency/phase offset values*/
-const size_t LMS_NCO_VAL_COUNT = 16;
-
+static const int LMS_NCO_VAL_COUNT = 16;
 
 /** Convenience type for fixed length name string*/
 typedef char lms_name_t[16];
@@ -559,15 +531,11 @@ typedef char lms_name_t[16];
  * @return      number of available antennae, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_GetAntennaList(lms_device_t *device, bool dir_tx,
-                                        size_t chan, lms_name_t *list);
+                                            size_t chan, lms_name_t *list);
 
 /**
  * Select the antenna for the specified RX or TX channel.
  *
- * LMS_SetFrequency() automatically selects antenna based on frequency. This
- * function is meant to override path selected by LMS_SetFrequency() and should
- * be called after LMS_SetFrequency().
- *
  * @param       device      Device handle previously obtained by LMS_Open().
  * @param       dir_tx      Select RX or TX
  * @param       chan        channel index
@@ -698,13 +666,13 @@ API_EXPORT int CALL_CONV LMS_GetNCOPhase(lms_device_t *device, bool dir_tx,
  * @param dev       Device handle previously obtained by LMS_Open().
  * @param dir_tx    Select RX or TX
  * @param chan      channel index
- * @param index     NCO frequency/phase index to activate
+ * @param index     NCO frequency/phase index to activate or (-1) to disable NCO
  * @param downconv  true(1) CMIX downconvert, false(0) CMIX upconvert
  *
  * @return 0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_SetNCOIndex(lms_device_t *device, bool dir_tx,
-                                    size_t chan, size_t index, bool downconv);
+                                    size_t chan, int index, bool downconv);
 
 /**
  * Get the currently active NCO frequency/phase offset index
@@ -742,21 +710,6 @@ API_EXPORT int CALL_CONV LMS_ReadParam(lms_device_t *device,
 API_EXPORT int CALL_CONV LMS_WriteParam(lms_device_t *device,
                                       struct LMS7Parameter param, uint16_t val);
 
-/**
- * Generates LPF coefficients for LMS GFIR.
- * Pass and stop band frequencies are relative to sampling rate.
- *
- * @param      n        number of coefficients to generate.
- * @param      w1       Relative pass band frequency. Range [0, 0.5].
- * @param      w2       Relative Stop band frequency. Range [w1, 0.5].
- * @param      g_stop   Stop_band gain. Range [0, 0.5].
- * @param[out] coef     generated filter coefficients
- *
- * @return      0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_GenerateLPFCoef(size_t n, float_type w1,
-                        float_type w2, float_type g_stop, float_type *coef);
-
 /**
  * Configure LMS GFIR using specified filter coefficients. Maximum number of
  * coefficients is 40 for GFIR1 and GFIR2, and 120 for GFIR3.
@@ -804,7 +757,6 @@ API_EXPORT int CALL_CONV LMS_SetGFIR(lms_device_t * device, bool dir_tx,
 
 /**
  * @defgroup FN_LOW_LVL    Low-Level control functions
- *
  * @{
  */
 
@@ -828,7 +780,7 @@ API_EXPORT int CALL_CONV LMS_Reset(lms_device_t *device);
  * @return  0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_ReadLMSReg(lms_device_t *device, uint32_t address,
-                                     uint16_t *val);
+                                        uint16_t *val);
 
 /**
  * Write device LMS chip register
@@ -840,16 +792,7 @@ API_EXPORT int CALL_CONV LMS_ReadLMSReg(lms_device_t *device, uint32_t address,
  * @return  0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_WriteLMSReg(lms_device_t *device, uint32_t address,
-                                      uint16_t val);
-
-/**
- * Perform register test
- *
- * @param device    Device handle previously obtained by LMS_Open().
- *
- * @return  0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_RegisterTest(lms_device_t *device);
+                                        uint16_t val);
 
 /**
  * Read device FPGA register
@@ -861,7 +804,7 @@ API_EXPORT int CALL_CONV LMS_RegisterTest(lms_device_t *device);
  * @return  0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_ReadFPGAReg(lms_device_t *device, uint32_t address,
-                                     uint16_t *val);
+                                        uint16_t *val);
 
 /**
  * Write device FPGA register
@@ -873,7 +816,7 @@ API_EXPORT int CALL_CONV LMS_ReadFPGAReg(lms_device_t *device, uint32_t address,
  * @return  0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_WriteFPGAReg(lms_device_t *device, uint32_t address,
-                                      uint16_t val);
+                                        uint16_t val);
 
 /**
  * Read custom parameter from board
@@ -902,32 +845,11 @@ API_EXPORT int CALL_CONV LMS_WriteCustomBoardParam(lms_device_t *device,
                             uint8_t id, float_type val, const lms_name_t units);
 
 /**
- * Changes device reference clock used by API for various calculations.
- * Normally reference clock should be detected automatically based on device.
- * Use this function in case you have replaced the reference crystal.
- *
- * @param device      Device handle previously obtained by LMS_Open().
- * @param clock_Hz    reference clock in Hz.
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_SetReferenceClock(lms_device_t * device,
-                                               float_type clock_Hz);
-
-/**
- * Get the currently set reference clock
- *
- * @param device      Device handle previously obtained by LMS_Open().
- * @param clock_Hz    reference clock in Hz.
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_GetReferenceClock(lms_device_t * device,
-                                               float_type * clock_Hz);
-
-/**
- * Write value to VCTCXO trim DAC
+ * Write value to VCTCXO trim DAC. Used to adjust/calibrate reference clock
+ * generated by voltage controlled oscillator.
  *
  * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   val         Value to write to VCTCXO trim DAC
+ * @param   val         Value to write to VCTCXO trim DAC, range 0-255
  *
  * @return 0 on success, (-1) on failure
  */
@@ -943,50 +865,6 @@ API_EXPORT int CALL_CONV LMS_VCTCXOWrite(lms_device_t * dev, uint16_t val);
  */
 API_EXPORT int CALL_CONV LMS_VCTCXORead(lms_device_t * dev, uint16_t *val);
 
-/**
- * Get VCO value range.
- *
- * @param   dev     Device handle previously obtained by LMS_Open().
- * @param   vco_id  VCO identifier
- * @param   range   VCO range
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_GetVCORange(lms_device_t * dev, size_t vco_id,
-                                         lms_range_t* range);
-
-/**
- * Set VCO value range.
- *
- * @param   dev     Device handle previously obtained by LMS_Open().
- * @param   vco_id  VCO identifier
- * @param   range   VCO range
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_SetVCORange(lms_device_t * dev, size_t vco_id,
-                                         lms_range_t range);
-
-/**
- * @brief Sets callback for logging data write/read operations
- *
- * @param   dev     Device handle previously obtained by LMS_Open().
- * @param   func    callback function
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_SetDataLogCallback(lms_device_t *dev, void (*func)(bool tx, const unsigned char* data, const unsigned int length));
-
-/**
- * @brief Sets callback for logging info/warning/error messages
- *
- * @param   dev     Device handle previously obtained by LMS_Open().
- * @param   func    callback function
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_SetLogCallback(lms_device_t *dev, void (*func)(const char* cstr, const unsigned int type));
-
 /**
  * @defgroup LMS_CLOCK_ID   Clock definitions
  *
@@ -1000,7 +878,6 @@ API_EXPORT int CALL_CONV LMS_SetLogCallback(lms_device_t *dev, void (*func)(cons
 #define LMS_CLOCK_RXTSP 0x0004  /**<RXTSP reference clock*/
 #define LMS_CLOCK_TXTSP 0x0005  /**TXTSP reference clock*/
 
-
 /** @} (End LMS_CLOCK_ID) */
 
 /**
@@ -1028,91 +905,6 @@ API_EXPORT int CALL_CONV LMS_GetClockFreq(lms_device_t *dev, size_t clk_id,
 API_EXPORT int CALL_CONV LMS_SetClockFreq(lms_device_t *dev, size_t clk_id,
                                          float_type freq);
 
-/**
- * Load Si5351C configuration from file
- *
- * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   filename    file containing Si5351C configuration
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_LoadConfigSi5351C(lms_device_t *dev,
-                                                const char* filename);
-
-/**
- * Configure Si5351C frequencies
- *
- * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   clkin       PLLs input clock frequency.
- * @param   clks        output clock frequencies (must be 8).
- *                      0 - disabled, negative - inverted
- * @param   src         input clock source 0-XTAL, 1-CLKIN
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ConfigureSi5351C(lms_device_t *dev,
-                   float_type clkin, float_type *clks, uint32_t src);
-
-
-/**
- * Get Si5351C status bits
- *
- * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   status      Status bits. Pass Null to clear status bits
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_StatusSi5351C(lms_device_t *dev, uint32_t *status);
-
-
-typedef struct
-{
-    double fRef;
-    double fVCO;
-    //Reference counter latch
-    unsigned lockDetectPrec;   /**<3 cycles (0), or 5 cycles (1)*/
-    unsigned antiBacklash;     /**<2.9ns (0), 6ns (1)*/
-    unsigned referenceCounter; /**<range [0, 16383]*/
-    //N counter latch
-    unsigned cpGain;           /**<0 or 1*/
-    unsigned nCounter;         /**<range [0, 8191]*/
-    //Function latch
-    unsigned flCurrent1;
-    unsigned flCurrent2;
-    unsigned flTimerCounter;
-    unsigned flFastlock;
-    unsigned flMuxCtrl;
-    unsigned flPDPolarity;
-    unsigned flPD1;
-    unsigned flPD2;
-    unsigned flCounterReset;
-    unsigned flCPState;
-    //Initialization latch
-    unsigned ilCurrent1;
-    unsigned ilCurrent2;
-    unsigned ilTimerCounter;
-    unsigned ilFastlock;
-    unsigned ilMuxCtrl;
-    unsigned ilPDPolarity;
-    unsigned ilPD1;
-    unsigned ilPD2;
-    unsigned ilCounterReset;
-    unsigned ilCPState;
-    //Extra
-    uint32_t flags;
-
-}lms_adf4002_conf_t;
-
-/**
- * Get Si5351C status bits
- *
- * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   config      ADF4002 configuration structure ::lms_adf4002_conf_t
- *
- * @return 0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ConfigureADF4002(lms_device_t *dev, lms_adf4002_conf_t *config);
-
 /**
  * Synchronizes register values between API cache and chip
  *
@@ -1123,41 +915,44 @@ API_EXPORT int CALL_CONV LMS_ConfigureADF4002(lms_device_t *dev, lms_adf4002_con
  */
 API_EXPORT int CALL_CONV LMS_Synchronize(lms_device_t *dev, bool toChip);
 
-
 /**
- *
- * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   buffer      read values
- * @param   len         number of bytes to read
+ * @param       dev     Device handle previously obtained by LMS_Open().
+ * @param[in]   buffer  read values (8 GPIO values per byte, LSB first)
+ * @param       len     number of bytes to read
  *
  * @return 0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_GPIORead(lms_device_t *dev, uint8_t* buffer, size_t len);
 
 /**
- *
- * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   buffer      values to write
- * @param   len         number bytes to write
+ * @param       dev     Device handle previously obtained by LMS_Open().
+ * @param[out]  buffer  values to write (8 GPIO values per byte, LSB first)
+ * @param       len     number of bytes to write
  *
  * @return 0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_GPIOWrite(lms_device_t *dev, const uint8_t* buffer, size_t len);
 
 /**
- * Low-level data transfer using LMS64C protocol
- *
- * @param   dev         Device handle previously obtained by LMS_Open().
- * @param   cmd         command
- * @param   data        in/out data
- * @param   len         data length
+ * @param       dev     Device handle previously obtained by LMS_Open().
+ * @param[out]  buffer  GPIO direction configuration(8 GPIO per byte, LSB first; 0 input, 1 output)
+ * @param       len     number of bytes to read
  *
  * @return 0 on success, (-1) on failure
  */
-API_EXPORT int CALL_CONV LMS_TransferLMS64C(lms_device_t *dev, int cmd, uint8_t* data, size_t *len);
+API_EXPORT int CALL_CONV LMS_GPIODirRead(lms_device_t *dev, uint8_t* buffer, size_t len);
 
 /**
- *  Enables or disable caching of calibration values
+ * @param       dev     Device handle previously obtained by LMS_Open().
+ * @param[in]   buffer  GPIO direction configuration(8 GPIO per byte, LSB first; 0 input, 1 output)
+ * @param       len     number of bytes to write
+ *
+ * @return 0 on success, (-1) on failure
+ */
+API_EXPORT int CALL_CONV LMS_GPIODirWrite(lms_device_t *dev, const uint8_t* buffer, size_t len);
+
+/**
+ *  Enables or disable caching of calibration values.
  *
  * @param   dev         Device handle previously obtained by LMS_Open().
  * @param   enable      true to enable cache
@@ -1178,12 +973,10 @@ API_EXPORT int CALL_CONV LMS_EnableCalibCache(lms_device_t *dev, bool enable);
 API_EXPORT int CALL_CONV LMS_GetChipTemperature(lms_device_t *dev, size_t ind,
                                                 float_type *temp);
 
-
 /** @} (End FN_LOW_LVL) */
 
 /** @} (End FN_ADVANCED) */
 
-
 /**
  * @defgroup FN_STREAM    Sample Streaming functions
  * The functions in this section provides support for sending and receiving
@@ -1218,8 +1011,7 @@ typedef struct
 typedef struct
 {
     /**Stream handle. Should not be modified manually.
-     * Assigned by LMS_SetupStream().
-     */
+     * Assigned by LMS_SetupStream().*/
     size_t handle;
 
     //! Indicates whether stream is TX (true) or RX (false)
@@ -1234,8 +1026,7 @@ typedef struct
     /**Parameter for controlling configuration bias toward low latency or high
      * data throughput range [0,1.0].
      * 0 - lowest latency, usually results in lower throughput
-     * 1 - higher throughput, usually results in higher latency
-     */
+     * 1 - higher throughput, usually results in higher latency*/
     float throughputVsLatency;
 
     //! Data output format
@@ -1300,7 +1091,6 @@ API_EXPORT int CALL_CONV LMS_DestroyStream(lms_device_t *device, lms_stream_t *s
  */
 API_EXPORT int CALL_CONV LMS_StartStream(lms_stream_t *stream);
 
-
 /**
  * Stop stream
  *
@@ -1318,7 +1108,7 @@ API_EXPORT int CALL_CONV LMS_StopStream(lms_stream_t *conf);
  * @param samples       sample buffer.
  * @param sample_count  Number of samples to read
  * @param meta          Metadata. See the ::lms_stream_meta_t description.
- * @param timeout_ms    how long to wait for data before timing out(0=infinite).
+ * @param timeout_ms    how long to wait for data before timing out.
  *
  * @return number of samples received on success, (-1) on failure
  */
@@ -1342,7 +1132,7 @@ API_EXPORT int CALL_CONV LMS_GetStreamStatus(lms_stream_t *stream, lms_stream_st
  * @param samples       sample buffer.
  * @param sample_count  Number of samples to write
  * @param meta          Metadata. See the ::lms_stream_meta_t description.
- * @param timeout_ms    how long to wait for data before timing out(0=infinite).
+ * @param timeout_ms    how long to wait for data before timing out.
  *
  * @return number of samples send on success, (-1) on failure
  */
@@ -1356,22 +1146,21 @@ API_EXPORT int CALL_CONV LMS_SendStream(lms_stream_t *stream,
  * @param chCount       number of waveform channels
  * @param sample_count  number of samples in each channel
  * @param format        waveform data format
- * @return  0 on success
+ * @return              0 on success, (-1) on failure
  */
 API_EXPORT int CALL_CONV LMS_UploadWFM(lms_device_t *device, const void **samples,
-							uint8_t chCount, size_t sample_count, int format);
+                                uint8_t chCount, size_t sample_count, int format);
 
 /** @brief Enables/Disables transmitting of uploaded waveform
  * @param device    Device handle previously obtained by LMS_Open().
+ * @param chan      Channel index
  * @param active    Enable/Disable waveform playback
- * @return  0 on success
+ * @return          0 on success, (-1) on failure
  */
-API_EXPORT int CALL_CONV LMS_EnableTxWFM(lms_device_t *device, const bool active);
-
+API_EXPORT int CALL_CONV LMS_EnableTxWFM(lms_device_t *device, unsigned chan, bool active);
 
 /** @} (End FN_STREAM) */
 
-
 /**
  * @defgroup FN_VERSION   Version and update functions
  *
@@ -1380,68 +1169,22 @@ API_EXPORT int CALL_CONV LMS_EnableTxWFM(lms_device_t *device, const bool active
  * @{
  */
 
-/**Enumeration of device programming target*/
+/**Enumeration of programming mode*/
 typedef enum
 {
-    LMS_TARGET_RAM = 0,    /**<load firmware/bitstream to volatile storage*/
-    LMS_TARGET_FLASH = 1,  /**<load firmware/bitstream to non-volatile storage*/
-    LMS_TARGET_BOOT = 2    /**<reset and boot from flash*/
-}lms_target_t;
+    LMS_PROG_MD_RAM = 0,    /**<load firmware/bitstream to volatile storage*/
+    LMS_PROG_MD_FLASH = 1,  /**<load firmware/bitstream to non-volatile storage*/
+    LMS_PROG_MD_RST = 2    /**<reset and boot from flash*/
+}lms_prog_md_t;
 
-
-
-/**Device information structure*/
-typedef struct
+/**Enumeration of programmable board modules*/
+typedef enum
 {
-    //! The displayable name for the device
-    char deviceName[32];
-
-    /*! The displayable name for the expansion card
-     */
-    char expansionName[32];
-
-    //! The firmware version as a string
-    char firmwareVersion[16];
-
-    //! The hardware version as a string
-    char hardwareVersion[16];
-
-    //! The protocol version as a string
-    char protocolVersion[16];
-
-    //! A unique board serial number
-    uint32_t boardSerialNumber;
-
-    //! Gateware version as a string
-    char gatewareVersion[16];
-    //! Gateware revision as a string
-    char gatewareRevision[16];
-    //! Which board should use this gateware
-    char gatewareTargetBoard[32];
-}lms_dev_info_t;
-
-/**
- * Get device serial number and version information
- *
- * @note This function returns pointer to internal data structure that gets
- * deallocated when device is closed. Do not attempt to read from it after
- * closing the device. If you need to keep using device info returned by this
- * function after closing the device, make a copy before closing the device.
- *
- * @param device    Device handle previously obtained by LMS_Open().
- * @return          pointer to device info structure
- */
-API_EXPORT const lms_dev_info_t* CALL_CONV LMS_GetDeviceInfo(lms_device_t *device);
-
-/**
- * @brief Returns API library build type
-*/
-API_EXPORT const char* LMS_GetBuildTimestamp();
-
-/**
-* @brief Returns API library version
-*/
-API_EXPORT const char* LMS_GetLibraryVersion();
+    LMS_PROG_TRG_FX3 = 0,
+    LMS_PROG_TRG_FPGA,
+    LMS_PROG_TRG_MCU,
+    LMS_PROG_TRG_HPM7,
+}lms_prog_trg_t;
 
 /*!
  * Callback from programming processes
@@ -1453,110 +1196,57 @@ API_EXPORT const char* LMS_GetLibraryVersion();
 typedef bool (*lms_prog_callback_t)(int bsent, int btotal, const char* progressMsg);
 
 /**
- * Write binary FPGA image to device.
+ * Write binary firmware/bitsteam image to specified device component.
  *
  * @param device    Device handle previously obtained by LMS_Open().
- * @param data      Pointer to memory containing FPGA image
- * @param size      Size of FPGA image in bytes.
- * @param target    load to volatile or non-volatile storage
+ * @param data      Pointer to memory containing firmware/bitsteam image
+ * @param size      Size of firmware/bitsteam image in bytes.
+ * @params target   device component to program ::lms_prog_trg_t
+ * @param mode      programming mode ::lms_prog_md_t
  * @return          0 on success, (-1) on failure
  */
-API_EXPORT int CALL_CONV LMS_ProgramFPGA(lms_device_t *device, const char *data,
-                 size_t size, lms_target_t target,lms_prog_callback_t callback);
+API_EXPORT int CALL_CONV LMS_Program(lms_device_t *device, const char *data, size_t size,
+           lms_prog_trg_t target, lms_prog_md_t mode, lms_prog_callback_t callback);
 
 /**
- * Read FPGA image from the specified file and write it to device.
+ * @param device    Device handle previously obtained by LMS_Open().
+ * @param download  True to download missing images from the web.
+ * @return          0 on success, (-1) on failure
+ */
+API_EXPORT int CALL_CONV LMS_ProgramUpdate(lms_device_t *device,
+                const bool download, lms_prog_callback_t callback);
+
+/**Device information structure*/
+typedef struct
+{
+    char deviceName[32];            /**<The display name of the device*/
+    char expansionName[32];         /**<The display name of the expansion card*/
+    char firmwareVersion[16];       /**<The firmware version as a string*/
+    char hardwareVersion[16];       /**<The hardware version as a string*/
+    char protocolVersion[16];       /**<The protocol version as a string*/
+    uint32_t boardSerialNumber;     /**<A unique board serial number*/
+    char gatewareVersion[16];       /**<Gateware version as a string*/
+    char gatewareRevision[16];      /**<Gateware revision as a string*/
+    char gatewareTargetBoard[32];   /**<Which board should use this gateware*/
+}lms_dev_info_t;
+
+/**
+ * Get device serial number and version information
+ *
+ * @note This function returns pointer to internal data structure that gets
+ * deallocated when device is closed. Do not attempt to read from it after
+ * closing the device. If you need to keep using device info returned by this
+ * function after closing the device, make a copy before closing the device.
  *
  * @param device    Device handle previously obtained by LMS_Open().
- * @param file      File containing FPGA image
- * @param target    load to volatile or non-volatile storage
- * @return          0 on success, (-1) on failure
+ * @return          pointer to device info structure ::lms_dev_info_t
  */
-API_EXPORT int CALL_CONV LMS_ProgramFPGAFile(lms_device_t *device,
-           const char *file, lms_target_t target, lms_prog_callback_t callback);
+API_EXPORT const lms_dev_info_t* CALL_CONV LMS_GetDeviceInfo(lms_device_t *device);
 
 /**
- * Write firmware image to device.
- *
- * @param device    Device handle previously obtained by LMS_Open().
- * @param data      Pointer to memory containing firmware image
- * @param size      Size of firmware image in bytes.
- * @param target    load to volatile or non-volatile storage
- * @return          0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ProgramFirmware(lms_device_t *device, const char *data,
-                size_t size, lms_target_t target, lms_prog_callback_t callback);
-
-/**
- * Read firmware image from file and write it to device.
- *
- * @param device    Device handle previously obtained by LMS_Open().
- * @param file      File containing Firmware image
- * @param target    load to volatile or non-volatile storage
- * @return          0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ProgramFirmwareFile(lms_device_t *device,
-           const char *file, lms_target_t target, lms_prog_callback_t callback);
-
-
-/**
- *
- * @param device    Device handle previously obtained by LMS_Open().
- * @param data      Pointer to memory containing image
- * @param size      Size of image in bytes.
- * @param mode      programming mode
- * @return          0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ProgramHPM7(lms_device_t *device, const char *data,
-                size_t size, unsigned mode, lms_prog_callback_t callback);
-
-/**
- *
- * @param device    Device handle previously obtained by LMS_Open().
- * @param file      File containing image
- * @param mode      programming mode
- * @return          0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ProgramHPM7File(lms_device_t *device,
-                const char *file, unsigned mode, lms_prog_callback_t callback);
-
-/**
- * Program LMS7 internal MCU.
- *
- * @param device    Device handle previously obtained by LMS_Open().
- * @param file      Pointer to memory containing binary MCU firmware image data
- * @param target    load to volatile or non-volatile storage
- * @param size      Size of MCU firmware image in bytes.
- *
- * @return          0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ProgramLMSMCU(lms_device_t *device, const char *data,
-                size_t size, lms_target_t target, lms_prog_callback_t callback);
-/**
- * Boots LMS7 internal MCU from flash memory.
- *
- * @param device    Device handle previously obtained by LMS_Open().
- *
- * @return          0 on success, (-1) on failure
- */
-API_EXPORT int CALL_CONV LMS_ResetLMSMCU(lms_device_t *device);
-
-/** @} (End FN_VERSION) */
-
-/**
- * @defgroup FN_ERRORS    Error reporting
- *
- * LMS API functions return 0 or positive value on success and (-1) on failure.
- * The functions in this section provides ability to get more detailed
- * error descriptions.
- * @{
- */
-
-/**
- * Get the last error code. Error codes are as defined in errno.h
- * @return last error code
- */
-API_EXPORT int CALL_CONV LMS_GetLastError(void);
+* @brief Returns API library version
+*/
+API_EXPORT const char* LMS_GetLibraryVersion();
 
 /**
  * Get the error message detailing why the last error occurred.
@@ -1565,7 +1255,7 @@ API_EXPORT int CALL_CONV LMS_GetLastError(void);
  */
 API_EXPORT const char * CALL_CONV LMS_GetLastErrorMessage(void);
 
-/** @} (End FN_ERRORS) */
+/** @} (End FN_VERSION) */
 
 #ifdef __cplusplus
 } //extern "C"
