@@ -2,12 +2,14 @@
 #include "lms7002_gui_utilities.h"
 #include "numericSlider.h"
 #include "lms7002_dlgGFIR_Coefficients.h"
+#include "lms7suiteAppFrame.h"
 
 using namespace lime;
 using namespace LMS7002_WXGUI;
 
 indexValueMap hbi_ovr_txtsp_IndexValuePairs;
 indexValueMap tsgfcw_txtsp_IndexValuePairs;
+indexValueMap cmix_gain_txtsp_IndexValuePairs;
 
 lms7002_pnlTxTSP_view::lms7002_pnlTxTSP_view( wxWindow* parent )
 :
@@ -152,6 +154,12 @@ lms7002_pnlTxTSP_view::lms7002_pnlTxTSP_view( wxWindow* parent, wxWindowID id, c
     tsgfcw_txtsp_IndexValuePairs.push_back(indexValuePair(0, 1));
     tsgfcw_txtsp_IndexValuePairs.push_back(indexValuePair(1, 2));
 
+    cmix_gain_txtsp_IndexValuePairs.push_back(indexValuePair(0, 2));
+    cmix_gain_txtsp_IndexValuePairs.push_back(indexValuePair(1, 2));
+    cmix_gain_txtsp_IndexValuePairs.push_back(indexValuePair(2, 0));
+    cmix_gain_txtsp_IndexValuePairs.push_back(indexValuePair(3, 0));
+    cmix_gain_txtsp_IndexValuePairs.push_back(indexValuePair(4, 1));
+
     LMS7002_WXGUI::UpdateTooltips(wndId2Enum, true);
 }
 
@@ -159,6 +167,17 @@ void lms7002_pnlTxTSP_view::Initialize(lms_device_t* pControl)
 {
     lmsControl = pControl;
     assert(lmsControl != nullptr);
+    uint16_t value;
+    if (!LMS_IsOpen(lmsControl,0) || LMS_ReadParam(lmsControl,LMS7param(MASK),&value)!=0  || value != 0)
+         value = 1;
+    wxArrayString temp;
+    temp.clear();
+    temp.push_back("-6 dB");
+    temp.push_back(value ? "-3 dB":"-6 dB");
+    temp.push_back("0 dB");
+    temp.push_back(value ? "+3 dB":"+6 dB");
+    temp.push_back("+6 dB");
+    cmbCMIX_GAIN_TXTSP->Set(temp);
 }
 
 void lms7002_pnlTxTSP_view::ParameterChangeHandler(wxSpinEvent& event)
@@ -197,6 +216,12 @@ void lms7002_pnlTxTSP_view::ParameterChangeHandler( wxCommandEvent& event )
     {
         value = index2value(value, hbi_ovr_txtsp_IndexValuePairs);
     }
+    else if(event.GetEventObject() == cmbCMIX_GAIN_TXTSP)
+    {
+        LMS_WriteParam(lmsControl, LMS7_CMIX_GAIN_TXTSP_R3, value % 0x2);
+        value = index2value(value, cmix_gain_txtsp_IndexValuePairs);
+    }
+
     LMS_WriteParam(lmsControl,parameter,value);
 
     if(event.GetEventObject() == rgrMODE_TX)
@@ -255,7 +280,9 @@ void lms7002_pnlTxTSP_view::onbtnGFIR1Coef( wxCommandEvent& event )
     coefficients.resize(maxCoefCount, 0);
     uint16_t ch;
     LMS_ReadParam(lmsControl,LMS7param(MAC),&ch);
-    int status =  LMS_GetGFIRCoeff(lmsControl, LMS_CH_TX, ch-1, LMS_GFIR1, &coefficients[0]);
+    ch = (ch == 2) ? 1 : 0;
+    ch += 2*LMS7SuiteAppFrame::m_lmsSelection;
+    int status =  LMS_GetGFIRCoeff(lmsControl, LMS_CH_TX, ch, LMS_GFIR1, &coefficients[0]);
     if (status < 0)
     {
         wxMessageBox(_("Error reading GFIR coefficients: ") + wxString::From8BitData(LMS_GetLastErrorMessage()), _("ERROR"), wxICON_ERROR | wxOK);
@@ -267,11 +294,12 @@ void lms7002_pnlTxTSP_view::onbtnGFIR1Coef( wxCommandEvent& event )
     if (dlg->ShowModal() == wxID_OK)
     {
         coefficients = dlg->GetCoefficients();
-        status = LMS_SetGFIRCoeff(lmsControl, LMS_CH_TX, ch-1, LMS_GFIR1, &coefficients[0],coefficients.size());
+        status = LMS_SetGFIRCoeff(lmsControl, LMS_CH_TX, ch, LMS_GFIR1, &coefficients[0],coefficients.size());
         if (status != 0)
             wxMessageBox(wxString::From8BitData(LMS_GetLastErrorMessage()), _("ERROR"), wxICON_ERROR | wxOK);
     }
     dlg->Destroy();
+    UpdateGUI();
 }
 
 void lms7002_pnlTxTSP_view::onbtnGFIR2Coef( wxCommandEvent& event )
@@ -282,7 +310,9 @@ void lms7002_pnlTxTSP_view::onbtnGFIR2Coef( wxCommandEvent& event )
     coefficients.resize(maxCoefCount, 0);
     uint16_t ch;
     LMS_ReadParam(lmsControl,LMS7param(MAC),&ch);
-    int status =  LMS_GetGFIRCoeff(lmsControl, LMS_CH_TX, ch-1, LMS_GFIR2, &coefficients[0]);
+    ch = (ch == 2) ? 1 : 0;
+    ch += 2*LMS7SuiteAppFrame::m_lmsSelection;
+    int status =  LMS_GetGFIRCoeff(lmsControl, LMS_CH_TX, ch, LMS_GFIR2, &coefficients[0]);
     if (status < 0)
     {
         wxMessageBox(_("Error reading GFIR coefficients: ") + wxString::From8BitData(LMS_GetLastErrorMessage()), _("ERROR"), wxICON_ERROR | wxOK);
@@ -293,11 +323,12 @@ void lms7002_pnlTxTSP_view::onbtnGFIR2Coef( wxCommandEvent& event )
     if (dlg->ShowModal() == wxID_OK)
     {
         coefficients = dlg->GetCoefficients();
-        status = LMS_SetGFIRCoeff(lmsControl, LMS_CH_TX, ch-1, LMS_GFIR2, &coefficients[0],coefficients.size());
+        status = LMS_SetGFIRCoeff(lmsControl, LMS_CH_TX, ch, LMS_GFIR2, &coefficients[0],coefficients.size());
         if (status != 0)
             wxMessageBox(wxString::From8BitData(LMS_GetLastErrorMessage()), _("ERROR"), wxICON_ERROR | wxOK);
     }
     dlg->Destroy();
+    UpdateGUI();
 }
 
 void lms7002_pnlTxTSP_view::onbtnGFIR3Coef( wxCommandEvent& event )
@@ -308,7 +339,9 @@ void lms7002_pnlTxTSP_view::onbtnGFIR3Coef( wxCommandEvent& event )
     coefficients.resize(maxCoefCount, 0);
     uint16_t ch;
     LMS_ReadParam(lmsControl,LMS7param(MAC),&ch);
-    int status =  LMS_GetGFIRCoeff(lmsControl, LMS_CH_TX, ch-1, LMS_GFIR3, &coefficients[0]);
+    ch = (ch == 2) ? 1 : 0;
+    ch += 2*LMS7SuiteAppFrame::m_lmsSelection;
+    int status =  LMS_GetGFIRCoeff(lmsControl, LMS_CH_TX, ch, LMS_GFIR3, &coefficients[0]);
     if (status < 0)
     {
         wxMessageBox(_("Error reading GFIR coefficients: ") + wxString::From8BitData(LMS_GetLastErrorMessage()), _("ERROR"), wxICON_ERROR | wxOK);
@@ -319,11 +352,12 @@ void lms7002_pnlTxTSP_view::onbtnGFIR3Coef( wxCommandEvent& event )
     if (dlg->ShowModal() == wxID_OK)
     {
         coefficients = dlg->GetCoefficients();
-        status = LMS_SetGFIRCoeff(lmsControl, LMS_CH_TX, ch-1, LMS_GFIR3, &coefficients[0],coefficients.size());
+        status = LMS_SetGFIRCoeff(lmsControl, LMS_CH_TX, ch, LMS_GFIR3, &coefficients[0],coefficients.size());
         if (status != 0)
             wxMessageBox(wxString::From8BitData(LMS_GetLastErrorMessage()), _("ERROR"), wxICON_ERROR | wxOK);
     }
     dlg->Destroy();
+    UpdateGUI();
 }
 
 void lms7002_pnlTxTSP_view::OnbtnUploadNCOClick( wxCommandEvent& event )
@@ -332,6 +366,8 @@ void lms7002_pnlTxTSP_view::OnbtnUploadNCOClick( wxCommandEvent& event )
     assert(txtNCOinputs.size() == 16);
     uint16_t ch;
     LMS_ReadParam(lmsControl,LMS7param(MAC),&ch);
+    ch = (ch == 2) ? 1 : 0;
+    ch += 2*LMS7SuiteAppFrame::m_lmsSelection;
     if (rgrMODE_TX->GetSelection() == 0)
     {
         float_type nco_freq[16];
@@ -342,7 +378,7 @@ void lms7002_pnlTxTSP_view::OnbtnUploadNCOClick( wxCommandEvent& event )
         }
         long value;
         txtFCWPHOmodeAdditional->GetValue().ToLong(&value);
-        LMS_SetNCOFrequency(lmsControl,LMS_CH_TX,ch-1,nco_freq,value);
+        LMS_SetNCOFrequency(lmsControl,LMS_CH_TX,ch,nco_freq,value);
     }
     else //PHO mode
     {
@@ -355,36 +391,44 @@ void lms7002_pnlTxTSP_view::OnbtnUploadNCOClick( wxCommandEvent& event )
         txtFCWPHOmodeAdditional->GetValue().ToDouble(&freq_MHz);
         LMS_SetNCOPhase(lmsControl, LMS_CH_TX, ch-1, nco_phase, freq_MHz);
     }
+    UpdateGUI();// API changes nco selection
 }
 
 void lms7002_pnlTxTSP_view::UpdateNCOinputs()
 {
     assert(txtNCOinputs.size() == 16);
     uint16_t ch;
-    LMS_ReadParam(lmsControl,LMS7param(MAC), &ch);
+    LMS_ReadParam(lmsControl,LMS7param(MAC),&ch);
+    ch = (ch == 2) ? 1 : 0;
+    ch += 2*LMS7SuiteAppFrame::m_lmsSelection;
     if (rgrMODE_TX->GetSelection() == 0) //FCW mode
     {
-        float_type freq[16];
-        float_type pho;
-        LMS_GetNCOFrequency(lmsControl,LMS_CH_TX,ch-1,freq,&pho);
+        float_type freq[16] = { 0 };
+        float_type pho=0;
+        LMS_GetNCOFrequency(lmsControl, LMS_CH_TX, ch, freq, &pho);
         for (size_t i = 0; i < txtNCOinputs.size(); ++i)
         {
             txtNCOinputs[i]->SetValue(wxString::Format(_("%.6f"), freq[i]/1e6));
         }
-        txtFCWPHOmodeAdditional->SetValue(wxString::Format(_("%f"), pho));
-        lblFCWPHOmodeName->SetLabel(_("PHO"));
+        txtFCWPHOmodeAdditional->SetValue(wxString::Format(_("%3.3f"), pho));
+        lblFCWPHOmodeName->SetLabel(_("PHO (deg)"));
+        tableTitleCol1->SetLabel(_("FCW(MHz)"));
+        tableTitleCol2->SetLabel(_("PHO(deg)"));
     }
     else //PHO mode
     {
-        float_type phase[16];
-        float_type fcw;
-        LMS_GetNCOFrequency(lmsControl,LMS_CH_TX,ch-1,phase,&fcw);
+        float_type phase[16] = { 0 };
+        float_type fcw = 0;
+        if (LMS_GetNCOPhase(lmsControl, LMS_CH_TX, ch - 1, phase, &fcw) != 0)
+            return;
         for (size_t i = 0; i < txtNCOinputs.size(); ++i)
         {
-            txtNCOinputs[i]->SetValue(wxString::Format(_("%.6f"), (65536.0/360.0)*  phase[i]));
+            txtNCOinputs[i]->SetValue(wxString::Format(_("%.3f"), (65536.0/360.0)*  phase[i]));
         }
         txtFCWPHOmodeAdditional->SetValue(wxString::Format(_("%.6f"), fcw/1e6));
         lblFCWPHOmodeName->SetLabel(_("FCW(MHz)"));
+        tableTitleCol2->SetLabel(_("FCW(MHz)"));
+        tableTitleCol1->SetLabel(_("PHO(deg)"));
     }
 }
 
@@ -422,6 +466,16 @@ void lms7002_pnlTxTSP_view::UpdateGUI()
     dccorr = value;
     cmbDCCORRQ_TXTSP->SetValue(dccorr);
 
+    uint16_t g_cmix;
+    LMS_ReadParam(lmsControl,LMS7param(CMIX_GAIN_TXTSP),&g_cmix);
+    value = value2index(g_cmix, cmix_gain_txtsp_IndexValuePairs);
+    LMS_ReadParam(lmsControl,LMS7param(CMIX_GAIN_TXTSP_R3),&g_cmix);
+    if (g_cmix)
+        value |= 1;
+    else
+        value &= ~1;
+    cmbCMIX_GAIN_TXTSP->SetSelection(value);
+
     //check if B channel is enabled
     LMS_ReadParam(lmsControl,LMS7param(MAC),(uint16_t*)&value);
     if (value >= 2)
@@ -434,19 +488,47 @@ void lms7002_pnlTxTSP_view::UpdateGUI()
 
 void lms7002_pnlTxTSP_view::PHOinputChanged(wxCommandEvent& event)
 {
+    uint16_t ch;
+    LMS_ReadParam(lmsControl,LMS7param(MAC),&ch);
+    ch = (ch == 2) ? 1 : 0;
+    ch += 2*LMS7SuiteAppFrame::m_lmsSelection;
+    // Write values for NCO phase or frequency each time they change - to ease the tuning of these values in measurements
+    if (rgrMODE_TX->GetSelection() == 0)
+    {
+        double angle;
+        txtFCWPHOmodeAdditional->GetValue().ToDouble(&angle);
+        LMS_SetNCOFrequency(lmsControl,LMS_CH_TX,ch,nullptr,angle);
+    }
+    else //PHO mode
+    {
+        double freq;
+        txtFCWPHOmodeAdditional->GetValue().ToDouble(&freq);
+        LMS_SetNCOPhase(lmsControl, LMS_CH_TX, ch, nullptr, freq*1e6);
+    }
+
     assert(lblNCOangles.size() == 16);
     if (rgrMODE_TX->GetSelection() == 1)
-        for (int i = 0; i < 16; ++i)
-        {
-            long phoVal = 0;
-            txtNCOinputs[i]->GetValue().ToLong(&phoVal);
-            lblNCOangles[i]->SetLabel(wxString::Format("%3.3f", 2.0 * 180 * phoVal / (65536.0)));
+    {
+        double freq;
+        txtFCWPHOmodeAdditional->GetValue().ToDouble(&freq);
+        for (int i = 0; i < 16; ++i){
+            lblNCOangles[i]->SetLabel(wxString::Format("%3.3f", freq));
         }
+    }
     else
     {
-        long phoVal = 0;
-        txtFCWPHOmodeAdditional->GetValue().ToLong(&phoVal);
-        for (int i = 0; i < 16; ++i)
-            lblNCOangles[i]->SetLabel(wxString::Format("%3.3f", 2.0 * 180 * phoVal / (65536.0)));
+        double angle;
+        txtFCWPHOmodeAdditional->GetValue().ToDouble(&angle);
+        for (int i = 0; i < 16; ++i){
+            lblNCOangles[i]->SetLabel(wxString::Format("%3.3f", angle));
+        }
     }
 }
+
+void lms7002_pnlTxTSP_view::txtFCWPHOmodeAdditional_OnMouseWheel(wxMouseEvent& event){
+    double angle = 0;
+    txtFCWPHOmodeAdditional->GetValue().ToDouble(&angle);
+    int change = event.GetWheelRotation()/120;
+    angle += change*0.1;
+    txtFCWPHOmodeAdditional->SetValue(wxString::Format(_("%.1f"), angle > 0 ? angle : 0));
+}
