@@ -8,9 +8,13 @@
 #include <wx/wx.h>
 #endif //WX_PRECOMP
 
-
+#include "pnlUltimateEVB.h"
 #include "pnluLimeSDR.h"
 #include "pnlLimeSDR.h"
+#include "pnlBuffers.h"
+#include "lms7002m_novena_wxgui.h"
+#include "RFSpark_wxgui.h"
+ #include "pnlQSpark.h"
 #include <IConnection.h>
 #include <LMSBoards.h>
 #include <ADCUnits.h>
@@ -94,8 +98,9 @@ pnlBoardControls::pnlBoardControls(wxWindow* parent, wxWindowID id, const wxStri
 	cmbBoardSelection->SetSelection( 0 );
 	fgSizer248->Add( cmbBoardSelection, 0, wxALL, 5 );
 
-    for (int i = 0; i < LMS_DEV_COUNT; ++i)
-        cmbBoardSelection->AppendString(wxString::From8BitData(GetDeviceName((eLMS_DEV)i)));
+	for (int i = 0; i < LMS_DEV_COUNT; ++i) {
+            cmbBoardSelection->AppendString(wxString::From8BitData(GetDeviceName((eLMS_DEV)i)));
+	}
 
 	fgSizer247->Add( fgSizer248, 1, wxEXPAND, 5 );
 
@@ -153,61 +158,54 @@ pnlBoardControls::pnlBoardControls(wxWindow* parent, wxWindowID id, const wxStri
 
     fgSizer247->Add(pnlCustomControls, 1, wxEXPAND, 5);
 
-	wxFlexGridSizer* fgSizer249;
-	fgSizer249 = new wxFlexGridSizer( 0, 2, 5, 5 );
-	fgSizer249->AddGrowableCol( 0 );
-	fgSizer249->AddGrowableCol( 1 );
-	fgSizer249->SetFlexibleDirection( wxBOTH );
-	fgSizer249->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );
+    wxFlexGridSizer* fgSizer249 = new wxFlexGridSizer( 0, 2, 5, 5 );
+    fgSizer249->AddGrowableCol( 0 );
+    fgSizer249->AddGrowableCol( 1 );
+    fgSizer249->SetFlexibleDirection( wxBOTH );
+    fgSizer249->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );
 
-	wxStaticBoxSizer* sbSizer133;
-	sbSizer133 = new wxStaticBoxSizer( new wxStaticBox( this, wxID_ANY, wxT("Read") ), wxVERTICAL );
+    pnlReadControls = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, _("Read"));
+    wxStaticBoxSizer* sbSizer133 = new wxStaticBoxSizer( new wxStaticBox( pnlReadControls, wxID_ANY, wxT("Read") ), wxVERTICAL );
+    sizerAnalogRd = new wxFlexGridSizer( 0, 4, 2, 2 );
+    sizerAnalogRd->SetFlexibleDirection( wxBOTH );
+    sizerAnalogRd->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );
+    sizerAnalogRd->Add(new wxStaticText(pnlReadControls, wxID_ANY, _("Ch.")), 1, wxALL, 5);
+    sizerAnalogRd->Add(new wxStaticText(pnlReadControls, wxID_ANY, _("Name")), 1, wxALL, 5);
+    sizerAnalogRd->Add(new wxStaticText(pnlReadControls, wxID_ANY, _("Value")), 1, wxALL, 5);
+    sizerAnalogRd->Add(new wxStaticText(pnlReadControls, wxID_ANY, _("Units")), 1, wxALL, 5);
+    sbSizer133->Add( sizerAnalogRd, 1, wxEXPAND, 5 );
+    pnlReadControls->SetSizer(sbSizer133);
+    pnlReadControls->Fit();
+    pnlReadControls->Hide();
+    fgSizer249->Add( pnlReadControls, 1, wxEXPAND, 5 );
 
-	sizerAnalogRd = new wxFlexGridSizer( 0, 4, 2, 2 );
-	sizerAnalogRd->SetFlexibleDirection( wxBOTH );
-	sizerAnalogRd->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );
-    sizerAnalogRd->Add(new wxStaticText(this, wxID_ANY, _("Ch.")), 1, wxALL, 5);
-    sizerAnalogRd->Add(new wxStaticText(this, wxID_ANY, _("Name")), 1, wxALL, 5);
-    sizerAnalogRd->Add(new wxStaticText(this, wxID_ANY, _("Value")), 1, wxALL, 5);
-    sizerAnalogRd->Add(new wxStaticText(this, wxID_ANY, _("Units")), 1, wxALL, 5);
+    pnlWriteControls = new wxPanel(this, wxID_ANY, wxDefaultPosition, wxDefaultSize, 0, _("Read"));
+    wxStaticBoxSizer* sbSizer134 = new wxStaticBoxSizer( new wxStaticBox( pnlWriteControls, wxID_ANY, wxT("Write") ), wxVERTICAL );
+    sizerAnalogWr = new wxFlexGridSizer( 0, 4, 2, 2 );
+    sizerAnalogWr->SetFlexibleDirection( wxBOTH );
+    sizerAnalogWr->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );
+    sizerAnalogWr->Add(new wxStaticText(pnlWriteControls, wxID_ANY, _("Ch.")), 1, wxALL, 5);
+    sizerAnalogWr->Add(new wxStaticText(pnlWriteControls, wxID_ANY, _("Name")), 1, wxALL, 5);
+    sizerAnalogWr->Add(new wxStaticText(pnlWriteControls, wxID_ANY, _("Value")), 1, wxALL, 5);
+    sizerAnalogWr->Add(new wxStaticText(pnlWriteControls, wxID_ANY, _("Units")), 1, wxALL, 5);
+    sbSizer134->Add( sizerAnalogWr, 1, wxEXPAND, 5 );
+    pnlWriteControls->SetSizer(sbSizer134);
+    pnlWriteControls->Fit();
+    pnlWriteControls->Hide();
+    fgSizer249->Add( pnlWriteControls, 1, wxEXPAND, 5 );
 
-
-	sbSizer133->Add( sizerAnalogRd, 1, wxEXPAND, 5 );
-
-
-	fgSizer249->Add( sbSizer133, 1, wxEXPAND, 5 );
-
-	wxStaticBoxSizer* sbSizer134;
-	sbSizer134 = new wxStaticBoxSizer( new wxStaticBox( this, wxID_ANY, wxT("Write") ), wxVERTICAL );
-
-	sizerAnalogWr = new wxFlexGridSizer( 0, 4, 2, 2 );
-	sizerAnalogWr->SetFlexibleDirection( wxBOTH );
-	sizerAnalogWr->SetNonFlexibleGrowMode( wxFLEX_GROWMODE_SPECIFIED );
-
-
-	sbSizer134->Add( sizerAnalogWr, 1, wxEXPAND, 5 );
-    sizerAnalogWr->Add(new wxStaticText(this, wxID_ANY, _("Ch.")), 1, wxALL, 5);
-    sizerAnalogWr->Add(new wxStaticText(this, wxID_ANY, _("Name")), 1, wxALL, 5);
-    sizerAnalogWr->Add(new wxStaticText(this, wxID_ANY, _("Value")), 1, wxALL, 5);
-    sizerAnalogWr->Add(new wxStaticText(this, wxID_ANY, _("Units")), 1, wxALL, 5);
-
-
-	fgSizer249->Add( sbSizer134, 1, wxEXPAND, 5 );
-
-
-	fgSizer247->Add( fgSizer249, 1, wxEXPAND, 5 );
+    fgSizer247->Add( fgSizer249, 1, wxEXPAND, 5 );
 
     sizerAdditionalControls = new wxFlexGridSizer(0, 1, 0, 0);
     fgSizer247->Add(sizerAdditionalControls, 1, wxEXPAND, 5);
-
-	this->SetSizer( fgSizer247 );
-	this->Layout();
-	fgSizer247->Fit( this );
+    this->SetSizer( fgSizer247 );
+    this->Layout();
+    fgSizer247->Fit( this );
 
 	// Connect Events
     cmbBoardSelection->Connect(wxEVT_COMMAND_CHOICE_SELECTED, wxCommandEventHandler(pnlBoardControls::OnUserChangedBoardType), NULL, this);
-	btnReadAll->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( pnlBoardControls::OnReadAll ), NULL, this );
-	btnWriteAll->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( pnlBoardControls::OnWriteAll ), NULL, this );
+    btnReadAll->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( pnlBoardControls::OnReadAll ), NULL, this );
+    btnWriteAll->Connect( wxEVT_COMMAND_BUTTON_CLICKED, wxCommandEventHandler( pnlBoardControls::OnWriteAll ), NULL, this );
 
     SetupControls(GetDeviceName(LMS_DEV_UNKNOWN));
 }
@@ -268,7 +266,7 @@ void pnlBoardControls::OnWriteAll( wxCommandEvent& event )
 {
     if (!LMS_IsOpen(lmsControl,1))
     {
-        wxMessageBox(_("Board not connected"), _("Warning"));
+        wxMessageBox(_("Device not connected"), _("Warning"));
         return;
     }
 
@@ -318,7 +316,7 @@ void pnlBoardControls::UpdatePanel()
     {
         mADC_GUI_widgets[i]->channel->SetLabel(wxString::Format(_("%i"), mADCparameters[i].channel));
         mADC_GUI_widgets[i]->title->SetLabel(wxString(mADCparameters[i].name));
-        mADC_GUI_widgets[i]->value->SetLabel(wxString::Format(_("%i"), mADCparameters[i].value));
+        mADC_GUI_widgets[i]->value->SetLabel(wxString::Format(_("%1.0f"), mADCparameters[i].value));
         mADC_GUI_widgets[i]->units->SetLabelText(wxString::Format("%s", mADCparameters[i].units));
     }
 
@@ -335,24 +333,15 @@ std::vector<pnlBoardControls::ADC_DAC> pnlBoardControls::getBoardADCs(const stri
 {
     std::vector<ADC_DAC> paramList;
     if(boardID == GetDeviceName(LMS_DEV_LIMESDR)
+        || boardID == GetDeviceName(LMS_DEV_ULIMESDR)
         || boardID == GetDeviceName(LMS_DEV_LIMESDR_PCIE)
-        || boardID == GetDeviceName(LMS_DEV_QSPARK))
+        || boardID == GetDeviceName(LMS_DEV_LIMESDR_QPCIE)
+        || boardID == GetDeviceName(LMS_DEV_LIMESDR_USB_SP)
+        || boardID == GetDeviceName(LMS_DEV_LMS7002M_ULTIMATE_EVB))
     {
-        ADC_DAC dacVoltage;
-        dacVoltage.name = "DAC";
-        dacVoltage.channel = 0;
-        dacVoltage.powerOf10 = 0;
-        dacVoltage.units = adcUnits2string(RAW);
-        dacVoltage.value = 0;
-        paramList.push_back(dacVoltage);
 
-        ADC_DAC temperature;
-        temperature.name = "Temperature";
-        temperature.channel = 1;
-        temperature.powerOf10 = 0;
-        temperature.units = adcUnits2string(TEMPERATURE);
-        temperature.value = 0;
-        paramList.push_back(temperature);
+        paramList.push_back(ADC_DAC{"DAC", 0, 0, adcUnits2string(RAW)});
+        paramList.push_back(ADC_DAC{"Board Temperature", 0, 1, adcUnits2string(TEMPERATURE)});
     }
     return paramList;
 }
@@ -361,8 +350,11 @@ std::vector<pnlBoardControls::ADC_DAC> pnlBoardControls::getBoardDACs(const std:
 {
     std::vector<ADC_DAC> paramList;
     if(boardID == GetDeviceName(LMS_DEV_LIMESDR)
+        || boardID == GetDeviceName(LMS_DEV_ULIMESDR)
         || boardID == GetDeviceName(LMS_DEV_LIMESDR_PCIE)
-        || boardID == GetDeviceName(LMS_DEV_QSPARK))
+        || boardID == GetDeviceName(LMS_DEV_LIMESDR_QPCIE)
+        || boardID == GetDeviceName(LMS_DEV_LIMESDR_USB_SP)
+        || boardID == GetDeviceName(LMS_DEV_LMS7002M_ULTIMATE_EVB))
     {
         ADC_DAC dacVoltage;
         dacVoltage.name = "DAC";
@@ -412,13 +404,18 @@ void pnlBoardControls::SetupControls(const std::string &boardID)
 
     if (boardID != GetDeviceName(LMS_DEV_UNKNOWN))
     {
+        if (mADCparameters.size()!=0)
+            pnlReadControls->Show();
+        else
+            pnlReadControls->Hide();
+
         for (size_t i = 0; i < mADCparameters.size(); ++i)
         {
             ADC_GUI *gui = new ADC_GUI();
-            gui->channel = new wxStaticText(this, wxID_ANY, wxString::Format(_("%i"), mADCparameters[i].channel));
-            gui->title = new wxStaticText(this, wxID_ANY, wxString(mADCparameters[i].name));
-            gui->value = new wxStaticText(this, wxID_ANY, _(""));
-            gui->units = new wxStaticText(this, wxID_ANY, wxString::Format("%s", mADCparameters[i].units));
+            gui->channel = new wxStaticText(pnlReadControls, wxID_ANY, wxString::Format(_("%i"), mADCparameters[i].channel));
+            gui->title = new wxStaticText(pnlReadControls, wxID_ANY, wxString(mADCparameters[i].name));
+            gui->value = new wxStaticText(pnlReadControls, wxID_ANY, _(""));
+            gui->units = new wxStaticText(pnlReadControls, wxID_ANY, wxString::Format("%s", mADCparameters[i].units));
             mADC_GUI_widgets.push_back(gui);
             sizerAnalogRd->Add(gui->channel, 1, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
             sizerAnalogRd->Add(gui->title, 1, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
@@ -426,15 +423,20 @@ void pnlBoardControls::SetupControls(const std::string &boardID)
             sizerAnalogRd->Add(gui->units, 1, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
         }
 
+        if (mDACparameters.size()!=0)
+            pnlWriteControls->Show();
+        else
+            pnlWriteControls->Hide();
+
         for (size_t i = 0; i < mDACparameters.size(); ++i)
         {
             DAC_GUI *gui = new DAC_GUI();
-            gui->channel = new wxStaticText(this, wxID_ANY, wxString::Format(_("%i"), mDACparameters[i].channel));
-            gui->title = new wxStaticText(this, wxID_ANY, wxString(mDACparameters[i].name));
-            gui->value = new wxSpinCtrl(this, wxNewId(), _(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS | wxTE_PROCESS_ENTER, mDACparameters[i].minValue, mDACparameters[i].maxValue, mDACparameters[i].minValue);
+            gui->channel = new wxStaticText(pnlWriteControls, wxID_ANY, wxString::Format(_("%i"), mDACparameters[i].channel));
+            gui->title = new wxStaticText(pnlWriteControls, wxID_ANY, wxString(mDACparameters[i].name));
+            gui->value = new wxSpinCtrl(pnlWriteControls, wxNewId(), _(""), wxDefaultPosition, wxDefaultSize, wxSP_ARROW_KEYS | wxTE_PROCESS_ENTER, mDACparameters[i].minValue, mDACparameters[i].maxValue, mDACparameters[i].minValue);
             gui->value->Connect(wxEVT_COMMAND_SPINCTRL_UPDATED, wxSpinEventHandler(pnlBoardControls::OnSetDACvalues), NULL, this);
             gui->value->Connect(wxEVT_TEXT_ENTER, wxCommandEventHandler(pnlBoardControls::OnSetDACvaluesENTER), NULL, this);
-            gui->units = new wxStaticText(this, wxID_ANY, wxString::Format("%s", mDACparameters[i].units));
+            gui->units = new wxStaticText(pnlWriteControls, wxID_ANY, wxString::Format("%s", mDACparameters[i].units));
             mDAC_GUI_widgets.push_back(gui);
             sizerAnalogWr->Add(gui->channel, 1, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
             sizerAnalogWr->Add(gui->title, 1, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
@@ -452,14 +454,51 @@ void pnlBoardControls::SetupControls(const std::string &boardID)
         additionalControls = pnl;
         sizerAdditionalControls->Add(additionalControls);
     }
-    else if(boardID == GetDeviceName(LMS_DEV_LIMESDR) || boardID == GetDeviceName(LMS_DEV_LIMESDR_PCIE))
+    else if(boardID == GetDeviceName(LMS_DEV_LIMESDR)
+         || boardID == GetDeviceName(LMS_DEV_LIMESDR_PCIE))
     {
         pnlLimeSDR* pnl = new pnlLimeSDR(this, wxNewId());
         pnl->Initialize(lmsControl);
         additionalControls = pnl;
         sizerAdditionalControls->Add(additionalControls);
     }
-
+    else if(boardID == GetDeviceName(LMS_DEV_LMS7002M_ULTIMATE_EVB))
+    {
+        pnlUltimateEVB* pnl = new pnlUltimateEVB(this, wxNewId());
+        pnl->Initialize(lmsControl);
+        additionalControls = pnl;
+        sizerAdditionalControls->Add(additionalControls);
+    }
+    else if(boardID == GetDeviceName(LMS_DEV_RFSPARK)
+         || boardID == GetDeviceName(LMS_DEV_EVB7)
+         || boardID == GetDeviceName(LMS_DEV_EVB7V2))
+    {
+        pnlBuffers* pnl = new pnlBuffers(this, wxNewId());
+        pnl->Initialize(lmsControl);
+        additionalControls = pnl;
+        sizerAdditionalControls->Add(additionalControls);
+    }
+    else if (boardID == GetDeviceName(LMS_DEV_NOVENA))
+    {
+        LMS7002M_Novena_wxgui* pnl = new LMS7002M_Novena_wxgui(this, wxNewId());
+        pnl->Initialize(lmsControl);
+        additionalControls = pnl;
+        sizerAdditionalControls->Add(additionalControls);
+    }
+    else if (boardID == GetDeviceName(LMS_DEV_RFESPARK))
+    {
+        RFSpark_wxgui* pnl = new RFSpark_wxgui(this, wxNewId());
+        pnl->Initialize(lmsControl);
+        additionalControls = pnl;
+        sizerAdditionalControls->Add(additionalControls);
+    }
+    else if (boardID == GetDeviceName(LMS_DEV_LIMESDR_QPCIE))
+    {
+        pnlQSpark* pnl = new pnlQSpark(this, wxNewId());
+        pnl->Initialize(lmsControl);
+        additionalControls = pnl;
+        sizerAdditionalControls->Add(additionalControls);
+    }
     Layout();
     Fit();
 }
@@ -518,7 +557,8 @@ void pnlBoardControls::OnCustomRead(wxCommandEvent& event)
     }
 
     txtCustomUnitsRd->SetLabel(units);
-    txtCustomValueRd->SetLabel(wxString::Format(_("%f"), value));
+    txtCustomValueRd->SetLabel(wxString::Format(_("%1.1f"), value));
+
 }
 
 void pnlBoardControls::OnCustomWrite(wxCommandEvent& event)
