@@ -24,6 +24,7 @@
 #include <assert.h>
 #include <chrono>
 #include <thread>
+#include "Logger.h"
 
 #include "MCU_BD.h"
 const static uint16_t MCU_PARAMETER_ADDRESS = 0x002D; //register used to pass parameter values to MCU
@@ -37,7 +38,7 @@ using namespace lime;
 #include "MCU_BD.h"
 
 float_type LMS7002M::gVCO_frequency_table[3][2] = { { 3800e6, 5222e6 }, { 4961e6, 6754e6 }, {6306e6, 7714e6} };
-float_type LMS7002M::gCGEN_VCO_frequencies[2] = {1950e6, 2700e6};
+float_type LMS7002M::gCGEN_VCO_frequencies[2] = {1950e6, 2900e6};
 
 ///define for parameter enumeration if prefix might be needed
 extern std::vector<const LMS7Parameter*> LMS7parameterList;
@@ -55,22 +56,22 @@ void LMS7002M::Log(const char* text, LogType type)
     switch(type)
     {
     case LOG_INFO:
-        printf("%s\n", text);
+        lime::info(text);
         if(log_callback)
             log_callback(text, type);
         break;
     case LOG_WARNING:
-        printf("Warning: %s\n", text);
+        lime::warning(text);
         if(log_callback)
             log_callback(text, type);
         break;
     case LOG_ERROR:
-        printf("ERROR: %s\n", text);
+        lime::error(text);
         if(log_callback)
             log_callback(text, type);
         break;
     case LOG_DATA:
-        printf("DATA: %s\n", text);
+        lime::debug(text);
         if(log_callback)
             log_callback(text, type);
         break;
@@ -100,7 +101,7 @@ void LMS7002M::Log(LogType type, const char *format, va_list argList)
 
 /** @brief Sets connection which is used for data communication with chip
 */
-void LMS7002M::SetConnection(IConnection* port, const size_t devIndex, IConnection* samplesPort)
+void LMS7002M::SetConnection(IConnection* port, const size_t devIndex)
 {
     controlPort = port;
     mdevIndex = devIndex;
@@ -108,7 +109,6 @@ void LMS7002M::SetConnection(IConnection* port, const size_t devIndex, IConnecti
     if (controlPort != nullptr)
     {
         unsigned byte_array_size = 0;
-        addrLMS7002M = controlPort->GetDeviceInfo().addrsLMS7002M.at(devIndex);
         if (controlPort->IsOpen())
         {
             unsigned chipRev = this->Get_SPI_Reg_bits(LMS7_MASK, true);
@@ -117,12 +117,8 @@ void LMS7002M::SetConnection(IConnection* port, const size_t devIndex, IConnecti
             else
                 byte_array_size = 1024 * 8;
         }
-        mcuControl->Initialize(port, byte_array_size);
+        mcuControl->Initialize(port, mdevIndex, byte_array_size);
     }
-    if(samplesPort == nullptr)
-        dataPort = controlPort;
-    else
-        dataPort = samplesPort;
 }
 
 /** @brief Creates LMS7002M main control object.
@@ -133,8 +129,6 @@ LMS7002M::LMS7002M() :
     mValueCache(new CalibrationCache()),
     mRegistersMap(new LMS7002M_RegistersMap()),
     controlPort(nullptr),
-    dataPort(nullptr),
-    addrLMS7002M(-1),
     mdevIndex(0),
     mSelfCalDepth(0)
 {
@@ -197,10 +191,12 @@ LMS7002M::LMS7002M() :
     MemorySectionAddresses[RxGFIR3b][1] = 0x0567;
     MemorySectionAddresses[RxGFIR3c][0] = 0x0580;
     MemorySectionAddresses[RxGFIR3c][1] = 0x05A7;
+    MemorySectionAddresses[RSSI_DC_CALIBRATION][0] = 0x05C0;
+    MemorySectionAddresses[RSSI_DC_CALIBRATION][1] = 0x05CC;
 
     mRegistersMap->InitializeDefaultValues(LMS7parameterList);
     mcuControl = new MCU_BD();
-    mcuControl->Initialize(controlPort);
+    mcuControl->Initialize(nullptr);
 }
 
 LMS7002M::~LMS7002M()
@@ -248,8 +244,6 @@ int LMS7002M::EnableChannel(const bool isTx, const bool enable)
 
     //--- ADC/DAC ---
     this->Modify_SPI_Reg_bits(LMS7param(EN_DIR_AFE), 1);
-    this->Modify_SPI_Reg_bits(LMS7param(EN_G_AFE), enable?1:0);
-    this->Modify_SPI_Reg_bits(LMS7param(PD_AFE), enable?0:1);
     if (ch == ChA)
     {
         if (isTx) this->Modify_SPI_Reg_bits(LMS7param(PD_TX_AFE1), enable?0:1);
@@ -261,26 +255,43 @@ int LMS7002M::EnableChannel(const bool isTx, const bool enable)
         else      this->Modify_SPI_Reg_bits(LMS7param(PD_RX_AFE2), enable?0:1);
     }
 
+    int disabledChannels = (Get_SPI_Reg_bits(LMS7_PD_AFE.address,4,1)&0xF);//check if all channels are disabled
+    Modify_SPI_Reg_bits(LMS7param(EN_G_AFE),disabledChannels==0xF ? 0 : 1);
+    Modify_SPI_Reg_bits(LMS7param(PD_AFE), disabledChannels==0xF ? 1 : 0);
+
     //--- digital ---
     if (isTx)
     {
         this->Modify_SPI_Reg_bits(LMS7param(EN_TXTSP), enable?1:0);
+        this->Modify_SPI_Reg_bits(LMS7param(ISINC_BYP_TXTSP), enable?0:1);
         this->Modify_SPI_Reg_bits(LMS7param(GFIR3_BYP_TXTSP), 1);
         this->Modify_SPI_Reg_bits(LMS7param(GFIR2_BYP_TXTSP), 1);
         this->Modify_SPI_Reg_bits(LMS7param(GFIR1_BYP_TXTSP), 1);
+
+        if (!enable)
+        {
+            this->Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_TXTSP), 1);
+            this->Modify_SPI_Reg_bits(LMS7param(DC_BYP_TXTSP), 1);
+            this->Modify_SPI_Reg_bits(LMS7param(GC_BYP_TXTSP), 1);
+            this->Modify_SPI_Reg_bits(LMS7param(PH_BYP_TXTSP), 1);
+        }
     }
     else
     {
         this->Modify_SPI_Reg_bits(LMS7param(EN_RXTSP), enable?1:0);
+        this->Modify_SPI_Reg_bits(LMS7param(DC_BYP_RXTSP), enable?0:1);
+        this->Modify_SPI_Reg_bits(LMS7param(DCLOOP_STOP), enable?0:1);
         this->Modify_SPI_Reg_bits(LMS7param(AGC_MODE_RXTSP), 2); //bypass
-        this->Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_RXTSP), 1);
         this->Modify_SPI_Reg_bits(LMS7param(AGC_BYP_RXTSP), 1);
         this->Modify_SPI_Reg_bits(LMS7param(GFIR3_BYP_RXTSP), 1);
         this->Modify_SPI_Reg_bits(LMS7param(GFIR2_BYP_RXTSP), 1);
         this->Modify_SPI_Reg_bits(LMS7param(GFIR1_BYP_RXTSP), 1);
-        this->Modify_SPI_Reg_bits(LMS7param(DC_BYP_RXTSP), 1);
-        this->Modify_SPI_Reg_bits(LMS7param(GC_BYP_RXTSP), 1);
-        this->Modify_SPI_Reg_bits(LMS7param(PH_BYP_RXTSP), 1);
+        if (!enable)
+        {
+            this->Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_RXTSP), 1);
+            this->Modify_SPI_Reg_bits(LMS7param(GC_BYP_RXTSP), 1);
+            this->Modify_SPI_Reg_bits(LMS7param(PH_BYP_RXTSP), 1);
+        }
     }
 
     //--- baseband ---
@@ -319,7 +330,7 @@ int LMS7002M::EnableChannel(const bool isTx, const bool enable)
     {
         this->SetActiveChannel(ChSXT);
         this->Modify_SPI_Reg_bits(LMS7param(EN_DIR_SXRSXT), 1);
-        this->Modify_SPI_Reg_bits(LMS7param(EN_G), enable?1:0);
+        this->Modify_SPI_Reg_bits(LMS7param(EN_G), (disabledChannels&3) == 3?0:1);
         if (ch == ChB) //enable LO to channel B
         {
             this->SetActiveChannel(ChA);
@@ -330,7 +341,7 @@ int LMS7002M::EnableChannel(const bool isTx, const bool enable)
     {
         this->SetActiveChannel(ChSXR);
         this->Modify_SPI_Reg_bits(LMS7param(EN_DIR_SXRSXT), 1);
-        this->Modify_SPI_Reg_bits(LMS7param(EN_G), enable?1:0);
+        this->Modify_SPI_Reg_bits(LMS7param(EN_G), (disabledChannels&0xC)==0xC?0:1);
         if (ch == ChB) //enable LO to channel B
         {
             this->SetActiveChannel(ChA);
@@ -690,9 +701,10 @@ int LMS7002M::SaveConfig(const char* filename)
     fout << "[lms7002_registers_b]" << endl;
     addrToRead.clear(); //add only B channel addresses
     for (uint8_t i = 0; i < MEMORY_SECTIONS_COUNT; ++i)
-        for (uint16_t addr = MemorySectionAddresses[i][0]; addr <= MemorySectionAddresses[i][1]; ++addr)
-            if (addr >= 0x0100)
-                addrToRead.push_back(addr);
+        if (i != RSSI_DC_CALIBRATION)
+            for (uint16_t addr = MemorySectionAddresses[i][0]; addr <= MemorySectionAddresses[i][1]; ++addr)
+                if (addr >= 0x0100)
+                    addrToRead.push_back(addr);
 
     this->SetActiveChannel(ChB);
     for (uint16_t i = 0; i < addrToRead.size(); ++i)
@@ -968,11 +980,11 @@ int LMS7002M::SetPathRFE(PathRFE path)
 
 LMS7002M::PathRFE LMS7002M::GetPathRFE(void)
 {
-    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_LB1_RFE)) != 0) return PATH_RFE_LB1;
-    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_LB2_RFE)) != 0) return PATH_RFE_LB2;
-    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_L_RFE)) != 0) return PATH_RFE_LNAL;
-    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_W_RFE)) != 0) return PATH_RFE_LNAW;
-    if (this->Get_SPI_Reg_bits(LMS7param(PD_LNA_RFE)) != 0) return PATH_RFE_NONE;
+    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_LB1_RFE)) == 0) return PATH_RFE_LB1;
+    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_LB2_RFE)) == 0) return PATH_RFE_LB2;
+    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_L_RFE)) == 0) return PATH_RFE_LNAL;
+    if (this->Get_SPI_Reg_bits(LMS7param(EN_INSHSW_W_RFE)) == 0) return PATH_RFE_LNAW;
+    if (this->Get_SPI_Reg_bits(LMS7param(PD_LNA_RFE)) == 0) return PATH_RFE_NONE;
     return PATH_RFE_LNAH;
 }
 
@@ -1028,9 +1040,9 @@ float_type LMS7002M::GetReferenceClk_SX(bool tx)
 */
 float_type LMS7002M::GetFrequencyCGEN()
 {
-    float_type dMul = (GetReferenceClk_SX(Rx)/2.0)/(Get_SPI_Reg_bits(LMS7param(DIV_OUTCH_CGEN))+1); //DIV_OUTCH_CGEN
-    uint16_t gINT = Get_SPI_Reg_bits(0x0088, 13, 0); //read whole register to reduce SPI transfers
-    uint32_t gFRAC = ((gINT & 0xF) * 65536) | Get_SPI_Reg_bits(0x0087, 15, 0);
+    float_type dMul = (GetReferenceClk_SX(Rx)/2.0)/(Get_SPI_Reg_bits(LMS7param(DIV_OUTCH_CGEN), true)+1); //DIV_OUTCH_CGEN
+    uint16_t gINT = Get_SPI_Reg_bits(0x0088, 13, 0, true); //read whole register to reduce SPI transfers
+    uint32_t gFRAC = ((gINT & 0xF) * 65536) | Get_SPI_Reg_bits(0x0087, 15, 0, true);
     return dMul * (((gINT>>4) + 1 + gFRAC/1048576.0));
 }
 
@@ -1175,7 +1187,7 @@ bool LMS7002M::GetSXLocked(bool tx)
     @return 0-success, other-failure
 */
 int LMS7002M::TuneVCO(VCO_Module module) // 0-cgen, 1-SXR, 2-SXT
-{   
+{
     auto settlingTime = chrono::microseconds(50); //can be lower
     struct CSWInteval
     {
@@ -1431,6 +1443,7 @@ int LMS7002M::SetFrequencySX(bool tx, float_type freq_Hz, SX_details* output)
 
     Channel ch = this->GetActiveChannel();
     this->SetActiveChannel(tx?ChSXT:ChSXR);
+    Modify_SPI_Reg_bits(LMS7param(EN_INTONLY_SDM), 0);
     Modify_SPI_Reg_bits(LMS7param(INT_SDM), integerPart); //INT_SDM
     Modify_SPI_Reg_bits(0x011D, 15, 0, fractionalPart & 0xFFFF); //FRAC_SDM[15:0]
     Modify_SPI_Reg_bits(0x011E, 3, 0, (fractionalPart >> 16)); //FRAC_SDM[19:16]
@@ -1522,6 +1535,78 @@ int LMS7002M::SetFrequencySX(bool tx, float_type freq_Hz, SX_details* output)
     return 0;
 }
 
+/** @brief Sets SX frequency with Reference clock spur cancelation
+    @param Tx Rx/Tx module selection
+    @param freq_Hz desired frequency in Hz
+    @return 0-success, other-cannot deliver requested frequency
+*/
+int LMS7002M::SetFrequencySXWithSpurCancelation(bool tx, float_type freq_Hz, float_type BW)
+{
+    const float BWOffset = 2e6;
+    BW += BWOffset; //offset to avoid ref clock on BW edge
+    bool needCancelation = false;
+    float_type refClk = GetReferenceClk_SX(false);
+    int low = (freq_Hz-BW/2)/refClk;
+    int high = (freq_Hz+BW/2)/refClk;
+    if(low != high)
+        needCancelation = true;
+
+    int status;
+    float newFreq;
+    if(needCancelation)
+    {
+        newFreq = (int)(freq_Hz/refClk+0.5)*refClk;
+        TuneRxFilter(BW-BWOffset+2*abs(freq_Hz-newFreq));
+        status = SetFrequencySX(tx, newFreq);
+    }
+    else
+        status = SetFrequencySX(tx, freq_Hz);
+    if(status != 0)
+        return status;
+    const int ch = Get_SPI_Reg_bits(LMS7param(MAC));
+    for(int i=0; i<2; ++i)
+    {
+        Modify_SPI_Reg_bits(LMS7param(MAC), i+1);
+        SetNCOFrequency(LMS7002M::Rx, 15, 0);
+    }
+    if(needCancelation)
+    {
+        Modify_SPI_Reg_bits(LMS7param(MAC), ch);
+        Modify_SPI_Reg_bits(LMS7param(EN_INTONLY_SDM), 1);
+
+        /*uint16_t gINT = Get_SPI_Reg_bits(0x011E, 13, 0);	// read whole register to reduce SPI transfers
+        uint32_t gFRAC = ((gINT&0xF) * 65536) | Get_SPI_Reg_bits(0x011D, 15, 0);
+        bool upconvert = gFRAC > (1 << 19);
+        gINT = gINT >> 4;
+        if(upconvert)
+        {
+            gINT+=;
+            Modify_SPI_Reg_bits(LMS7param(INT_SDM), gINT);
+        }
+        Modify_SPI_Reg_bits(0x011D, 15, 0, 0);
+        Modify_SPI_Reg_bits(0x011E, 3, 0, 0);*/
+        //const float_type refClk_Hz = GetReferenceClk_SX(tx);
+        //float actualFreq = (float_type)refClk_Hz / (1 << (Get_SPI_Reg_bits(LMS7param(DIV_LOCH)) + 1));
+        //actualFreq *= (gINT + 4) * (Get_SPI_Reg_bits(LMS7param(EN_DIV2_DIVPROG)) + 1);
+        float actualFreq = newFreq;
+        float userFreq = freq_Hz;
+        bool upconvert = actualFreq > userFreq;
+        for(int i=0; i<2; ++i)
+        {
+            Modify_SPI_Reg_bits(LMS7param(MAC), i+1);
+            Modify_SPI_Reg_bits(LMS7param(CMIX_SC_RXTSP), !upconvert);
+            Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_RXTSP), 0);
+            Modify_SPI_Reg_bits(LMS7param(SEL_RX), 15);
+            Modify_SPI_Reg_bits(LMS7param(CMIX_GAIN_RXTSP), 1);
+            SetNCOFrequency(LMS7002M::Rx, 14, 0);
+            SetNCOFrequency(LMS7002M::Rx, 15, abs(actualFreq-userFreq));
+        }
+    }
+
+    Modify_SPI_Reg_bits(LMS7param(MAC), ch);
+    return 0;
+}
+
 /**	@brief Returns currently set SXR/SXT frequency
 	@return SX frequency Hz
 */
@@ -1719,7 +1804,17 @@ int LMS7002M::GetGFIRCoefficients(bool tx, uint8_t GFIR_index, int16_t *coef, ui
 */
 int LMS7002M::SPI_write(uint16_t address, uint16_t data)
 {
-    return this->SPI_write_batch(&address, &data, 1);
+    if(address == 0x0640 || address == 0x0641)
+    {
+        MCU_BD* mcu = GetMCUControls();
+        SPI_write(0x002D, address);
+        SPI_write(0x020C, data);
+        mcu->RunProcedure(7);
+        mcu->WaitForMCU(50);
+        return SPI_read(0x040B);
+    }
+    else
+        return this->SPI_write_batch(&address, &data, 1);
 }
 
 /** @brief Reads whole register value from given address
@@ -1742,7 +1837,18 @@ uint16_t LMS7002M::SPI_read(uint16_t address, bool fromChip, int *status)
     if(controlPort)
     {
         uint16_t data = 0;
-        int st = this->SPI_read_batch(&address, &data, 1);
+        int st;
+        if(address == 0x0640 || address == 0x0641)
+        {
+            MCU_BD* mcu = GetMCUControls();
+            SPI_write(0x002D, address);
+            mcu->RunProcedure(8);
+            mcu->WaitForMCU(50);
+            uint16_t rdVal = SPI_read(0x040B, true, status);
+            return rdVal;
+        }
+        else
+            st = this->SPI_read_batch(&address, &data, 1);
         if (status != nullptr) *status = st;
         return data;
     }
@@ -1777,8 +1883,7 @@ int LMS7002M::SPI_write_batch(const uint16_t* spiAddr, const uint16_t* spiData,
     }
 
     checkConnection();
-
-    return controlPort->TransactSPI(addrLMS7002M, data.data(), nullptr, cnt);
+    return controlPort->WriteLMS7002MSPI(data.data(), cnt,mdevIndex);
 }
 
 /** @brief Batches multiple register reads into least amount of transactions
@@ -1798,7 +1903,8 @@ int LMS7002M::SPI_read_batch(const uint16_t* spiAddr, uint16_t* spiData, uint16_
         dataWr[i] = (uint32_t(spiAddr[i]) << 16);
     }
 
-    int status = controlPort->TransactSPI(addrLMS7002M, dataWr.data(), dataRd.data(), cnt);
+
+    int status = controlPort->ReadLMS7002MSPI(dataWr.data(), dataRd.data(), cnt,mdevIndex);
     if (status != 0) return status;
 
     int mac = mRegistersMap->GetValue(0, LMS7param(MAC).address) & 0x0003;
@@ -1891,9 +1997,10 @@ int LMS7002M::RegistersTest(const char* fileName)
             ss << moduleNames[i] << "  [" << chex << ":";
             sprintf(chex, "0x%04X", endAddr);
             ss << chex << "]";
-            if (startAddr >= 0x0100)
+            if (startAddr >= 0x0100) {
                 ss << " Ch." << (cc == 1 ? "A" : "B");
-                ss << endl;
+	    }
+            ss << endl;
             for (uint8_t p = 0; p < patternsCount; ++p)
                 moduleTestsSuccess &= RegistersTestInterval(startAddr, endAddr, patterns[p], ss) == 0;
         }
@@ -2041,7 +2148,8 @@ bool LMS7002M::IsSynced()
     std::vector<uint32_t> dataRd(addrToRead.size());
     for(size_t i = 0; i < addrToRead.size(); ++i)
         dataWr[i] = (uint32_t(addrToRead[i]) << 16);
-    status = controlPort->TransactSPI(addrLMS7002M, dataWr.data(), dataRd.data(), dataWr.size());
+    status = controlPort->ReadLMS7002MSPI(dataWr.data(),  dataRd.data(), dataWr.size(),mdevIndex);
+
     for(size_t i=0; i<addrToRead.size(); ++i)
         dataReceived[i] = dataRd[i] & 0xFFFF;
     if (status != 0)
@@ -2079,7 +2187,7 @@ bool LMS7002M::IsSynced()
     dataRd.resize(addrToRead.size());
     for(size_t i = 0; i < addrToRead.size(); ++i)
         dataWr[i] = (uint32_t(addrToRead[i]) << 16);
-    status = controlPort->TransactSPI(addrLMS7002M, dataWr.data(), dataRd.data(), dataWr.size());
+    status = controlPort->ReadLMS7002MSPI(dataWr.data(),  dataRd.data(), dataWr.size(),mdevIndex);
     for(size_t i=0; i<addrToRead.size(); ++i)
         dataReceived[i] = dataRd[i] & 0xFFFF;
     if (status != 0)
@@ -2362,7 +2470,7 @@ bool LMS7002M::GetRxDCRemoval(void)
 int LMS7002M::SetTxDCOffset(const float_type I, const float_type Q)
 {
     const bool bypass = I == 0.0 and Q == 0.0;
-    this->Modify_SPI_Reg_bits(LMS7param(DC_BYP_RXTSP), bypass?1:0);
+    this->Modify_SPI_Reg_bits(LMS7param(DC_BYP_TXTSP), bypass?1:0);
     this->Modify_SPI_Reg_bits(LMS7param(DCCORRI_TXTSP), std::lrint(I*128));
     this->Modify_SPI_Reg_bits(LMS7param(DCCORRQ_TXTSP), std::lrint(Q*128));
 	return 0;
@@ -2450,55 +2558,23 @@ void LMS7002M::EnableCalibrationByMCU(bool enabled)
 
 float_type LMS7002M::GetTemperature()
 {
-    auto ch = Get_SPI_Reg_bits(LMS7param(MAC));
-    auto regMap = BackupRegisterMap();
-    Modify_SPI_Reg_bits(LMS7param(MAC), 1);
+    Modify_SPI_Reg_bits(LMS7_RSSI_PD, 0);
+    Modify_SPI_Reg_bits(LMS7_RSSI_RSSIMODE, 0);
+    Modify_SPI_Reg_bits(LMS7_DAC_CLKDIV, 32);
+    uint16_t biasMux = Get_SPI_Reg_bits(LMS7_MUX_BIAS_OUT);
+    Modify_SPI_Reg_bits(LMS7_MUX_BIAS_OUT, 2);
 
-    //RFE
-    Modify_SPI_Reg_bits(LMS7param(EN_G_RFE), 0);
-    //RBB
-    Modify_SPI_Reg_bits(LMS7param(EN_G_RBB), 0);
-    //AFE
-    Modify_SPI_Reg_bits(LMS7param(PD_RX_AFE1), 0);
-    Modify_SPI_Reg_bits(LMS7param(MUX_AFE_1), 2);
-    Modify_SPI_Reg_bits(LMS7param(EN_G_AFE), 1);
-    //BIAS
-    Modify_SPI_Reg_bits(LMS7param(MUX_BIAS_OUT), 2);
-    //RxTSP
-    SetDefaults(RxTSP);
-    Modify_SPI_Reg_bits(LMS7param(DC_BYP_RXTSP), 1);
-    Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_RXTSP), 1);
-    Modify_SPI_Reg_bits(LMS7param(GFIR3_BYP_RXTSP), 1);
-    Modify_SPI_Reg_bits(LMS7param(GFIR2_BYP_RXTSP), 1);
-    Modify_SPI_Reg_bits(LMS7param(GFIR1_BYP_RXTSP), 1);
-    Modify_SPI_Reg_bits(LMS7param(GCORRI_RXTSP), 0);
-    Modify_SPI_Reg_bits(LMS7param(AGC_AVG_RXTSP), 7);
-    Modify_SPI_Reg_bits(LMS7param(AGC_MODE_RXTSP), 1);
-
-    //READ ADCQ value
-    Modify_SPI_Reg_bits(LMS7param(CAPSEL), 1);
-    Modify_SPI_Reg_bits(LMS7param(CAPTURE), 0);
-    int negativeCount = 0;
-    int signMeasureCount = 19;
-    int sign = -1;
-    for(int i = 0; i < signMeasureCount; ++i)
-    {
-        Modify_SPI_Reg_bits(LMS7param(CAPTURE), 1);
-        Modify_SPI_Reg_bits(LMS7param(CAPTURE), 0);
-        int16_t adcq = SPI_read(0x040F, true);
-        if(adcq & 0x200)
-            ++negativeCount;
-    }
-    if(negativeCount > signMeasureCount / 2)
-        sign = 1;
-
-    Modify_SPI_Reg_bits(LMS7param(CAPSEL), 0);
-    Modify_SPI_Reg_bits(LMS7param(CAPTURE), 0);
-    Modify_SPI_Reg_bits(LMS7param(CAPTURE), 1);
-    uint32_t rssi = (Get_SPI_Reg_bits(0x040F, 15, 0, true) << 2) | Get_SPI_Reg_bits(0x040E, 1, 0, true);
-    double temperature = (rssi / 16.0) * 0.443892 * sign + 40.5;
-    RestoreRegisterMap(regMap);
-    Modify_SPI_Reg_bits(LMS7param(MAC), ch);
+    this_thread::sleep_for(chrono::microseconds(250));
+    const uint16_t reg606 = SPI_read(0x0606, true);
+    float Vtemp = (reg606 >> 8) & 0xFF;
+    Vtemp *= 3.515625;
+    float Vptat = reg606 & 0xFF;
+    Vptat *= 3.515625;
+    float Vdiff = Vptat-Vtemp;
+    Vdiff /= 3.9;
+    float temperature = 40.5+Vdiff;
+    Modify_SPI_Reg_bits(LMS7_MUX_BIAS_OUT, biasMux);
+    printf("Vtemp 0x%04X, Vptat 0x%04X, Vdiff = %.2f, temp= %.3f\n", (reg606 >> 8) & 0xFF, reg606 & 0xFF, Vdiff, temperature);
     return temperature;
 }
 
@@ -2530,3 +2606,46 @@ int LMS7002M::CopyChannelRegisters(const Channel src, const Channel dest, const
     this->UpdateExternalBandSelect();
     return 0;
 }
+
+int LMS7002M::CalibrateAnalogRSSI_DC_Offset()
+{
+    Modify_SPI_Reg_bits(LMS7param(PD_RSSI_RFE), 0);
+    Modify_SPI_Reg_bits(LMS7param(PD_TIA_RFE), 0);
+    Modify_SPI_Reg_bits(LMS7param(RSSIDC_RSEL), 26);
+    int value = -63;
+    uint8_t wrValue = abs(value);
+    if(value < 0)
+        wrValue |= 0x40;
+    Modify_SPI_Reg_bits(LMS7param(RSSIDC_DCO1), wrValue, true);
+    uint8_t cmp = Get_SPI_Reg_bits(LMS7param(RSSIDC_CMPSTATUS), true);
+    uint8_t cmpPrev = cmp;
+    vector<int8_t> edges;
+    for(value = -63; value < 64; ++value)
+    {
+        wrValue = abs(value);
+        if(value < 0)
+            wrValue |= 0x40;
+        Modify_SPI_Reg_bits(LMS7param(RSSIDC_DCO1), wrValue, true);
+        this_thread::sleep_for(chrono::microseconds(5));
+        cmp = Get_SPI_Reg_bits(LMS7param(RSSIDC_CMPSTATUS), true);
+        if(cmp != cmpPrev)
+        {
+            edges.push_back(value);
+            cmpPrev = cmp;
+        }
+        if(edges.size() > 1)
+            break;
+    }
+    if(edges.size() != 2)
+    {
+        printf("Not found\n");
+        return ReportError(EINVAL, "Failed to find value");
+    }
+    int8_t found = (edges[0]+edges[1])/2;
+    wrValue = abs(found);
+    if(found < 0)
+        wrValue |= 0x40;
+    Modify_SPI_Reg_bits(LMS7param(RSSIDC_DCO1), wrValue, true);
+    printf("Found %i\n", found);
+    return 0;
+}
