@@ -24,7 +24,7 @@ struct LMS7Parameter
     const char* tooltip;
 };
 
-int LMS7ParameterCompare(LMS7Parameter a, LMS7Parameter b);
+int LMS7ParameterCompare(struct LMS7Parameter a, struct LMS7Parameter b);
 
 static const struct LMS7Parameter LMS7_LRST_TX_B = { 0x0020, 15, 15, 1, "LRST_TX_B", "Resets all the logic registers to the default state for Tx MIMO channel B" };
 static const struct LMS7Parameter LMS7_MRST_TX_B = { 0x0020, 14, 14, 1, "MRST_TX_B", "Resets all the configuration memory to the default state for Tx MIMO channel B" };
@@ -414,8 +414,8 @@ static const struct LMS7Parameter LMS7_ICT_LPFH_F_TBB = { 0x0107, 14, 10, 12, "I
 static const struct LMS7Parameter LMS7_ICT_LPFLAD_F_TBB = { 0x0107, 9, 5, 12, "ICT_LPFLAD_F_TBB", "This controls the operational amplfiers' output stages bias referene current of the low band ladder filter of the transmisster's baseband" };
 static const struct LMS7Parameter LMS7_ICT_LPFLAD_PT_TBB = { 0x0107, 4, 0, 12, "ICT_LPFLAD_PT_TBB", "This controls the operational amplifers' input stages bias reference current of the low band ladder filter of the transmitter's baseband" };
 static const struct LMS7Parameter LMS7_CG_IAMP_TBB = { 0x0108, 15, 10, 37, "CG_IAMP_TBB", "This controls the frontend gain of the TBB. For a given gain value, this control value varies with the set TX mode. After resistance calibration, the following table gives the nominal values for each frequency setting. However, this table is to be updated and corrected after calibration" };
-static const struct LMS7Parameter LMS7_ICT_IAMP_FRP_TBB = { 0x0108, 9, 5, 1, "ICT_IAMP_FRP_TBB", "This controls the reference bias current of the IAMP main bias current sources" };
-static const struct LMS7Parameter LMS7_ICT_IAMP_GG_FRP_TBB = { 0x0108, 4, 0, 6, "ICT_IAMP_GG_FRP_TBB", "This controls the reference bias current of the IAMP's cascode transistors gate voltages that set the IAMP's input voltage level. The IAMP's input is connected to the DAC output" };
+static const struct LMS7Parameter LMS7_ICT_IAMP_FRP_TBB = { 0x0108, 9, 5, 12, "ICT_IAMP_FRP_TBB", "This controls the reference bias current of the IAMP main bias current sources" };
+static const struct LMS7Parameter LMS7_ICT_IAMP_GG_FRP_TBB = { 0x0108, 4, 0, 12, "ICT_IAMP_GG_FRP_TBB", "This controls the reference bias current of the IAMP's cascode transistors gate voltages that set the IAMP's input voltage level. The IAMP's input is connected to the DAC output" };
 static const struct LMS7Parameter LMS7_RCAL_LPFH_TBB = { 0x0109, 15, 8, 97, "RCAL_LPFH_TBB", "This controls the value of the equivalent resistance of the resistor banks of the biquad filter stage (of the high band section) of the transmitter baseband(TBB)" };
 static const struct LMS7Parameter LMS7_RCAL_LPFLAD_TBB = { 0x0109, 7, 0, 193, "RCAL_LPFLAD_TBB", "This controls the value of the equivalent resistance of the resistor banks of the low pass filter ladder (of the low band section) of the transmitter baseband(TBB)" };
 static const struct LMS7Parameter LMS7_TSTIN_TBB = { 0x010A, 15, 14, 0, "TSTIN_TBB", "This control selects where the input test signal (vinp/n_aux_bbq/i) is routed to as well as disabling the route." };
@@ -442,10 +442,10 @@ static const struct LMS7Parameter LMS7_EN_NEXTRX_RFE = { 0x010D, 0, 0, 0, "EN_NE
 static const struct LMS7Parameter LMS7_DCOFFI_RFE = { 0x010E, 13, 7, 64, "DCOFFI_RFE", "Controls DC offset at the output of the TIA by injecting current to the input of the TIA (I side)" };
 static const struct LMS7Parameter LMS7_DCOFFQ_RFE = { 0x010E, 6, 0, 64, "DCOFFQ_RFE", "Controls DC offset at the output of the TIA by injecting current to the input of the TIA (Q side)" };
 static const struct LMS7Parameter LMS7_ICT_LOOPB_RFE = { 0x010F, 14, 10, 12, "ICT_LOOPB_RFE", "Controls the reference current of the RXFE loopback amplifier" };
-static const struct LMS7Parameter LMS7_ICT_TIAMAIN_RFE = { 0x010F, 9, 5, 2, "ICT_TIAMAIN_RFE", "Controls the reference current of the RXFE TIA first stage" };
-static const struct LMS7Parameter LMS7_ICT_TIAOUT_RFE = { 0x010F, 4, 0, 2, "ICT_TIAOUT_RFE", "Controls the reference current of the RXFE TIA 2nd stage" };
+static const struct LMS7Parameter LMS7_ICT_TIAMAIN_RFE = { 0x010F, 9, 5, 6, "ICT_TIAMAIN_RFE", "Controls the reference current of the RXFE TIA first stage" };
+static const struct LMS7Parameter LMS7_ICT_TIAOUT_RFE = { 0x010F, 4, 0, 6, "ICT_TIAOUT_RFE", "Controls the reference current of the RXFE TIA 2nd stage" };
 static const struct LMS7Parameter LMS7_ICT_LNACMO_RFE = { 0x0110, 14, 10, 2, "ICT_LNACMO_RFE", "Controls the current generating LNA output common mode voltage" };
-static const struct LMS7Parameter LMS7_ICT_LNA_RFE = { 0x0110, 9, 5, 31, "ICT_LNA_RFE", "Controls the current of the LNA core" };
+static const struct LMS7Parameter LMS7_ICT_LNA_RFE = { 0x0110, 9, 5, 12, "ICT_LNA_RFE", "Controls the current of the LNA core" };
 static const struct LMS7Parameter LMS7_ICT_LODC_RFE = { 0x0110, 4, 0, 20, "ICT_LODC_RFE", "Controls the DC of the mixer LO signal at the gate of the mixer switches" };
 static const struct LMS7Parameter LMS7_CAP_RXMXO_RFE = { 0x0111, 9, 5, 4, "CAP_RXMXO_RFE", "Control the decoupling cap at the output of the RX Mixer" };
 static const struct LMS7Parameter LMS7_CGSIN_LNA_RFE = { 0x0111, 4, 0, 3, "CGSIN_LNA_RFE", "Controls the cap parallel with the LNA input input NMOS CGS to control the Q of the maching circuit and provides trade off between gain/NF and IIP. The higher the frequency, the lower CGSIN_LNA_RFE should be. Also, the higher CGSIN, the lower the Q, The lower the gain, the higher the NF, and the higher the IIP3" };
@@ -471,8 +471,8 @@ static const struct LMS7Parameter LMS7_INPUT_CTL_PGA_RBB = { 0x0118, 15, 13, 0,
 static const struct LMS7Parameter LMS7_ICT_LPF_IN_RBB = { 0x0118, 9, 5, 12, "ICT_LPF_IN_RBB", "Controls the reference bias current of the input stage of the operational amplifier used in RBB_LPF blocks (Low or High). " };
 static const struct LMS7Parameter LMS7_ICT_LPF_OUT_RBB = { 0x0118, 4, 0, 12, "ICT_LPF_OUT_RBB", "The reference bias current of the output stage of the operational amplifier used in RBB_LPF blocks (low or High)" };
 static const struct LMS7Parameter LMS7_OSW_PGA_RBB = { 0x0119, 15, 15, 0, "OSW_PGA_RBB", "There are two instances of the PGA circuit in the design. The output of the RBB_LPF blocks are connected the input of these PGA blocks (common). The output of one of them is connected to two pads pgaoutn and pgaoutp and the output of the other PGA is connected directly to the ADC input" };
-static const struct LMS7Parameter LMS7_ICT_PGA_OUT_RBB = { 0x0119, 14, 10, 6, "ICT_PGA_OUT_RBB", "Controls the output stage reference bias current of the operational amplifier used in the PGA circuit" };
-static const struct LMS7Parameter LMS7_ICT_PGA_IN_RBB = { 0x0119, 9, 5, 6, "ICT_PGA_IN_RBB", "Controls the input stage reference bias current of the operational amplifier used in the PGA circuit" };
+static const struct LMS7Parameter LMS7_ICT_PGA_OUT_RBB = { 0x0119, 14, 10, 20, "ICT_PGA_OUT_RBB", "Controls the output stage reference bias current of the operational amplifier used in the PGA circuit" };
+static const struct LMS7Parameter LMS7_ICT_PGA_IN_RBB = { 0x0119, 9, 5, 20, "ICT_PGA_IN_RBB", "Controls the input stage reference bias current of the operational amplifier used in the PGA circuit" };
 static const struct LMS7Parameter LMS7_G_PGA_RBB = { 0x0119, 4, 0, 11, "G_PGA_RBB", "This is the gain of the PGA" };
 static const struct LMS7Parameter LMS7_RCC_CTL_PGA_RBB = { 0x011A, 13, 9, 23, "RCC_CTL_PGA_RBB", "Controls the stability passive compensation of the PGA_RBB operational amplifier" };
 static const struct LMS7Parameter LMS7_C_CTL_PGA_RBB = { 0x011A, 6, 0, 2, "C_CTL_PGA_RBB", "Control the value of the feedback capacitor of the PGA that is used to help against the parasitic cap at the virtual node for stability" };
@@ -559,9 +559,9 @@ static const struct LMS7Parameter LMS7_CMIX_GAIN_TXTSP = { 0x0208, 15, 14, 0, "C
 static const struct LMS7Parameter LMS7_CMIX_SC_TXTSP = { 0x0208, 13, 13, 0, "CMIX_SC_TXTSP", "Spectrum control of CMIX" };
 static const struct LMS7Parameter LMS7_CMIX_BYP_TXTSP = { 0x0208, 8, 8, 0, "CMIX_BYP_TXTSP", "CMIX bypass" };
 static const struct LMS7Parameter LMS7_ISINC_BYP_TXTSP = { 0x0208, 7, 7, 0, "ISINC_BYP_TXTSP", "ISINC bypass" };
-static const struct LMS7Parameter LMS7_GFIR3_BYP_TXTSP = { 0x0208, 6, 6, 0, "GFIR3_BYP_TXTSP", "GFIR3 bypass" };
-static const struct LMS7Parameter LMS7_GFIR2_BYP_TXTSP = { 0x0208, 5, 5, 0, "GFIR2_BYP_TXTSP", "GFIR2 bypass" };
-static const struct LMS7Parameter LMS7_GFIR1_BYP_TXTSP = { 0x0208, 4, 4, 0, "GFIR1_BYP_TXTSP", "GFIR1 bypass" };
+static const struct LMS7Parameter LMS7_GFIR3_BYP_TXTSP = { 0x0208, 6, 6, 1, "GFIR3_BYP_TXTSP", "GFIR3 bypass" };
+static const struct LMS7Parameter LMS7_GFIR2_BYP_TXTSP = { 0x0208, 5, 5, 1, "GFIR2_BYP_TXTSP", "GFIR2 bypass" };
+static const struct LMS7Parameter LMS7_GFIR1_BYP_TXTSP = { 0x0208, 4, 4, 1, "GFIR1_BYP_TXTSP", "GFIR1 bypass" };
 static const struct LMS7Parameter LMS7_DC_BYP_TXTSP = { 0x0208, 3, 3, 0, "DC_BYP_TXTSP", "DC corrector bypass" };
 static const struct LMS7Parameter LMS7_GC_BYP_TXTSP = { 0x0208, 1, 1, 0, "GC_BYP_TXTSP", "Gain corrector bypass" };
 static const struct LMS7Parameter LMS7_PH_BYP_TXTSP = { 0x0208, 0, 0, 0, "PH_BYP_TXTSP", "Phase corrector bypass" };
@@ -575,7 +575,7 @@ static const struct LMS7Parameter LMS7_MODE_TX = { 0x0240, 0, 0, 0, "MODE_TX", "
 static const struct LMS7Parameter LMS7_PHO_TX = { 0x0241, 15, 0, 0, "PHO_TX", "" };
 static const struct LMS7Parameter LMS7_CAPTURE = { 0x0400, 15, 15, 0, "CAPTURE", "" };
 static const struct LMS7Parameter LMS7_CAPSEL = { 0x0400, 14, 13, 0, "CAPSEL", "" };
-static const struct LMS7Parameter LMS7_CAPSEL_ADC = { 0x0400, 12, 12, 0, "CAPSEL_ADC", "" };
+static const struct LMS7Parameter LMS7_CAPSEL_ADC = { 0x0400, 12, 12, 0, "CAPSEL_ADC", "Selects ADC value source to be captured" };
 static const struct LMS7Parameter LMS7_TSGFC_RXTSP = { 0x0400, 9, 9, 0, "TSGFC_RXTSP", "TSG full scale control" };
 static const struct LMS7Parameter LMS7_TSGFCW_RXTSP = { 0x0400, 8, 7, 1, "TSGFCW_RXTSP", "Set frequency of TSG's NCO" };
 static const struct LMS7Parameter LMS7_TSGDCLDQ_RXTSP = { 0x0400, 6, 6, 0, "TSGDCLDQ_RXTSP", "" };
@@ -590,7 +590,6 @@ static const struct LMS7Parameter LMS7_GCORRI_RXTSP = { 0x0402, 10, 0, 2047, "GC
 static const struct LMS7Parameter LMS7_HBD_OVR_RXTSP = { 0x0403, 14, 12, 0, "HBD_OVR_RXTSP", "HBD interpolation ratio. Interpolation ratio is 2HBD_OVR+1" };
 static const struct LMS7Parameter LMS7_IQCORR_RXTSP = { 0x0403, 11, 0, 0, "IQCORR_RXTSP", "Phase corrector value (tan(Alpha/2)). Integer, 2's complement" };
 static const struct LMS7Parameter LMS7_HBD_DLY = { 0x0404, 15, 13, 0, "HBD_DLY", "" };
-static const struct LMS7Parameter LMS7_DCLOOP_BYP = { 0x0404, 8, 8, 0, "DCLOOP_BYP", "" };
 static const struct LMS7Parameter LMS7_DCCORR_AVG_RXTSP = { 0x0404, 2, 0, 0, "DCCORR_AVG_RXTSP", "Number of samples to average for Automatic DC corrector. Number of samples to average is 2DCCORR_AVG + 12" };
 static const struct LMS7Parameter LMS7_GFIR1_L_RXTSP = { 0x0405, 10, 8, 0, "GFIR1_L_RXTSP", "Parameter l of GFIR1 (l = roundUp(CoeffN/5)-1). Unsigned integer" };
 static const struct LMS7Parameter LMS7_GFIR1_N_RXTSP = { 0x0405, 7, 0, 0, "GFIR1_N_RXTSP", "Clock division ratio of GFIR1 is GFIR1_N + 1. Unsigned integer" };
@@ -601,16 +600,16 @@ static const struct LMS7Parameter LMS7_GFIR3_N_RXTSP = { 0x0407, 7, 0, 0, "GFIR3
 static const struct LMS7Parameter LMS7_AGC_K_RXTSP = { 0x0408, 17, 0, 0, "AGC_K_RXTSP", "AGC loop gain" };
 static const struct LMS7Parameter LMS7_AGC_ADESIRED_RXTSP = { 0x0409, 15, 4, 0, "AGC_ADESIRED_RXTSP", "AGC_ADESIRED" };
 static const struct LMS7Parameter LMS7_RSSI_MODE = { 0x040A, 15, 14, 0, "RSSI_MODE", "" };
-static const struct LMS7Parameter LMS7_AGC_MODE_RXTSP = { 0x040A, 13, 12, 0, "AGC_MODE_RXTSP", "" };
+static const struct LMS7Parameter LMS7_AGC_MODE_RXTSP = { 0x040A, 13, 12, 1, "AGC_MODE_RXTSP", "" };
 static const struct LMS7Parameter LMS7_AGC_AVG_RXTSP = { 0x040A, 2, 0, 0, "AGC_AVG_RXTSP", "AGC Averaging window size" };
 static const struct LMS7Parameter LMS7_DC_REG_RXTSP = { 0x040B, 15, 0, 0, "DC_REG_RXTSP", "" };
 static const struct LMS7Parameter LMS7_CMIX_GAIN_RXTSP = { 0x040C, 15, 14, 0, "CMIX_GAIN_RXTSP", "Gain of CMIX output" };
 static const struct LMS7Parameter LMS7_CMIX_SC_RXTSP = { 0x040C, 13, 13, 0, "CMIX_SC_RXTSP", "Spectrum control of CMIX" };
 static const struct LMS7Parameter LMS7_CMIX_BYP_RXTSP = { 0x040C, 7, 7, 0, "CMIX_BYP_RXTSP", "CMIX bypass" };
 static const struct LMS7Parameter LMS7_AGC_BYP_RXTSP = { 0x040C, 6, 6, 0, "AGC_BYP_RXTSP", "AGC bypass" };
-static const struct LMS7Parameter LMS7_GFIR3_BYP_RXTSP = { 0x040C, 5, 5, 0, "GFIR3_BYP_RXTSP", "GFIR3 bypass" };
-static const struct LMS7Parameter LMS7_GFIR2_BYP_RXTSP = { 0x040C, 4, 4, 0, "GFIR2_BYP_RXTSP", "GFIR2 bypass" };
-static const struct LMS7Parameter LMS7_GFIR1_BYP_RXTSP = { 0x040C, 3, 3, 0, "GFIR1_BYP_RXTSP", "GFIR1 bypass" };
+static const struct LMS7Parameter LMS7_GFIR3_BYP_RXTSP = { 0x040C, 5, 5, 1, "GFIR3_BYP_RXTSP", "GFIR3 bypass" };
+static const struct LMS7Parameter LMS7_GFIR2_BYP_RXTSP = { 0x040C, 4, 4, 1, "GFIR2_BYP_RXTSP", "GFIR2 bypass" };
+static const struct LMS7Parameter LMS7_GFIR1_BYP_RXTSP = { 0x040C, 3, 3, 1, "GFIR1_BYP_RXTSP", "GFIR1 bypass" };
 static const struct LMS7Parameter LMS7_DC_BYP_RXTSP = { 0x040C, 2, 2, 0, "DC_BYP_RXTSP", "DC corrector bypass" };
 static const struct LMS7Parameter LMS7_GC_BYP_RXTSP = { 0x040C, 1, 1, 0, "GC_BYP_RXTSP", "Gain corrector bypass" };
 static const struct LMS7Parameter LMS7_PH_BYP_RXTSP = { 0x040C, 0, 0, 0, "PH_BYP_RXTSP", "Phase corrector bypass" };
@@ -619,6 +618,140 @@ static const struct LMS7Parameter LMS7_DTHBIT_RX = { 0x0440, 8, 5, 1, "DTHBIT_RX
 static const struct LMS7Parameter LMS7_SEL_RX = { 0x0440, 4, 1, 0, "SEL_RX", "" };
 static const struct LMS7Parameter LMS7_MODE_RX = { 0x0440, 0, 0, 0, "MODE_RX", "" };
 static const struct LMS7Parameter LMS7_PHO_RX = { 0x0441, 15, 0, 0, "PHO_RX", "" };
+static const struct LMS7Parameter LMS7_TRX_GAIN_SRC = { 0x0081, 15, 15, 0, "TRX_GAIN_SRC", "Alternative TRX gain source select" };
+
+static const struct LMS7Parameter LMS7_LML2_TRXIQPULSE = { 0x0022, 15, 15, 0, "LML2_TRXIQPULSE", "TRXIQPULSE mode selection for LML Port 2" };
+static const struct LMS7Parameter LMS7_LML2_SISODDR = { 0x0022, 14, 14, 0, "LML2_SISODDR", "SISODDR mode selection for LML Port 2" };
+static const struct LMS7Parameter LMS7_LML1_TRXIQPULSE = { 0x0022, 13, 13, 0, "LML1_TRXIQPULSE", "TRXIQPULSE mode selection for LML Port 1" };
+static const struct LMS7Parameter LMS7_LML1_SISODDR = { 0x0022, 12, 12, 0, "LML1_SISODDR", "SISODDR mode selection for LML Port 1" };
+static const struct LMS7Parameter LMS7_MCLK2_DLY = { 0x002B, 13, 12, 0, "MCLK2_DLY", "MCLK2 clock internal delay." };
+static const struct LMS7Parameter LMS7_MCLK1_DLY = { 0x002B, 11, 10, 0, "MCLK1_DLY", "MCLK1 clock internal delay." };
+static const struct LMS7Parameter LMS7_MCLK2_INV = { 0x002B, 9, 9, 0, "MCLK2_INV", "MCLK2 clock inversion." };
+static const struct LMS7Parameter LMS7_MCLK1_INV = { 0x002B, 8, 8, 0, "MCLK1_INV", "MCLK1 clock inversion." };
+
+static const struct LMS7Parameter LMS7_CMIX_GAIN_TXTSP_R3 = { 0x0208, 12, 12, 0, "CMIX_GAIN_TXTSP_R3", "Gain of CMIX output, most significant part" };
+static const struct LMS7Parameter LMS7_CMIX_GAIN_RXTSP_R3 = { 0x040C, 12, 12, 0, "CMIX_GAIN_RXTSP_R3", "Gain of CMIX output, most significant part" };
+static const struct LMS7Parameter LMS7_R5_LPF_BYP_TBB = { 0x010B, 0, 0, 0, "R5_LPF_BYP_TBB", "Bypasses LPFS5_TBB low pass real-pole filter capacitor banks" };
+static const struct LMS7Parameter LMS7_CG_IAMP_TBB_R3 = { 0x0125, 15, 10, 37, "CG_IAMP_TBB_R3", "[Alternative control] This controls the reference bias current of the IAMP's cascode transistors gate voltages that set the IAMP's input voltage level. The IAMP's input is connected to the DAC output"};
+static const struct LMS7Parameter LMS7_LOSS_LIN_TXPAD_R3 = { 0x0125, 9, 5, 0, "LOSS_LIN_TXPAD_R3", "[Alternative control] Controls the gain of the linearizing part of of the TXPAD"};
+static const struct LMS7Parameter LMS7_LOSS_MAIN_TXPAD_R3 = { 0x0125, 4, 0, 0, "LOSS_MAIN_TXPAD_R3", "[Alternative control] Controls the gain  output power of the TXPAD"};
+static const struct LMS7Parameter LMS7_C_CTL_PGA_RBB_R3 = { 0x0126, 12, 11, 2, "C_CTL_PGA_RBB_R3", "[Alternative control] Control the value of the feedback capacitor of the PGA that is used to help against the parasitic cap at the virtual node for stability"};
+static const struct LMS7Parameter LMS7_G_PGA_RBB_R3 = { 0x0126, 10, 6, 11, "G_PGA_RBB_R3", "[Alternative control] Gain of the PGA"};
+static const struct LMS7Parameter LMS7_G_LNA_RFE_R3 = { 0x0126, 5, 2, 15, "G_LNA_RFE_R3", "[Alternative control] Controls the gain of the LNA"};
+static const struct LMS7Parameter LMS7_G_TIA_RFE_R3 = { 0x0126, 1, 0, 3, "G_TIA_RFE_R3", "[Alternative control] Controls the Gain of the TIA"};
+static const struct LMS7Parameter LMS7_RZ_CTRL = { 0x0122, 15, 14, 0, "RZ_CTRL", "Controls the PLL LPF zero resistor values"};
+static const struct LMS7Parameter LMS7_CMPLO_CTRL_SX = { 0x0122, 13, 13, 0, "CMPLO_CTRL_SX", "Controls the SXR/SXT PLL VCO comparator low treshold value"};
+static const struct LMS7Parameter LMS7_CMPLO_CTRL_CGEN_R3 = { 0x008B, 14, 14, 0, "CMPLO_CTRL_CGEN", "Controls the CGEN PLL VCO comparator low treshold value"};
+static const struct LMS7Parameter LMS7_ISINK_SPIBUFF = { 0x00A6, 15, 13, 0, "ISINK_SPI_BUFF", "Controls the SPIBUF LDO output resistive load"};
+
+static const struct LMS7Parameter LMS7_DCMODE = { 0x05C0, 15, 15, 0, "DCMODE", "DC calibration mode" };
+static const struct LMS7Parameter LMS7_PD_DCDAC_RXB = { 0x05C0, 7, 7, 1, "PD_DCDAC_RXB", ""};
+static const struct LMS7Parameter LMS7_PD_DCDAC_RXA = { 0x05C0, 6, 6, 1, "PD_DCDAC_RXA", ""};
+static const struct LMS7Parameter LMS7_PD_DCDAC_TXB = { 0x05C0, 5, 5, 1, "PD_DCDAC_TXB", ""};
+static const struct LMS7Parameter LMS7_PD_DCDAC_TXA = { 0x05C0, 4, 4, 1, "PD_DCDAC_TXA", ""};
+static const struct LMS7Parameter LMS7_PD_DCCMP_RXB = { 0x05C0, 3, 3, 1, "PD_DCCMP_RXB", ""};
+static const struct LMS7Parameter LMS7_PD_DCCMP_RXA = { 0x05C0, 2, 2, 1, "PD_DCCMP_RXA", ""};
+static const struct LMS7Parameter LMS7_PD_DCCMP_TXB = { 0x05C0, 1, 1, 1, "PD_DCCMP_TXB", ""};
+static const struct LMS7Parameter LMS7_PD_DCCMP_TXA = { 0x05C0, 0, 0, 1, "PD_DCCMP_TXA", ""};
+
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_RXBQ = { 0x05C1, 15, 15, 0, "DCCAL_CALSTATUS_RXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_RXBI = { 0x05C1, 14, 14, 0, "DCCAL_CALSTATUS_RXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_RXAQ = { 0x05C1, 13, 13, 0, "DCCAL_CALSTATUS_RXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_RXAI = { 0x05C1, 12, 12, 0, "DCCAL_CALSTATUS_RXAI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_TXBQ = { 0x05C1, 11, 11, 0, "DCCAL_CALSTATUS_TXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_TXBI = { 0x05C1, 10, 10, 0, "DCCAL_CALSTATUS_TXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_TXAQ = { 0x05C1, 9, 9, 0, "DCCAL_CALSTATUS_TXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CALSTATUS_TXAI = { 0x05C1, 8, 8, 0, "DCCAL_CALSTATUS_TXAI", ""};
+
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_RXBQ = { 0x05C1, 7, 7, 0, "DCCAL_CMPSTATUS_RXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_RXBI = { 0x05C1, 6, 6, 0, "DCCAL_CMPSTATUS_RXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_RXAQ = { 0x05C1, 5, 5, 0, "DCCAL_CMPSTATUS_RXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_RXAI = { 0x05C1, 4, 4, 0, "DCCAL_CMPSTATUS_RXAI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_TXBQ = { 0x05C1, 3, 3, 0, "DCCAL_CMPSTATUS_TXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_TXBI = { 0x05C1, 2, 2, 0, "DCCAL_CMPSTATUS_TXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_TXAQ = { 0x05C1, 1, 1, 0, "DCCAL_CMPSTATUS_TXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPSTATUS_TXAI = { 0x05C1, 0, 0, 0, "DCCAL_CMPSTATUS_TXAI", ""};
+
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_RXBQ = { 0x05C2, 15, 15, 0, "DCCAL_CMPCFG_RXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_RXBI = { 0x05C2, 14, 14, 0, "DCCAL_CMPCFG_RXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_RXAQ = { 0x05C2, 13, 13, 0, "DCCAL_CMPCFG_RXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_RXAI = { 0x05C2, 12, 12, 0, "DCCAL_CMPCFG_RXAI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_TXBQ = { 0x05C2, 11, 11, 0, "DCCAL_CMPCFG_TXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_TXBI = { 0x05C2, 10, 10, 0, "DCCAL_CMPCFG_TXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_TXAQ = { 0x05C2, 9, 9, 0, "DCCAL_CMPCFG_TXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_CMPCFG_TXAI = { 0x05C2, 8, 8, 0, "DCCAL_CMPCFG_TXAI", ""};
+
+static const struct LMS7Parameter LMS7_DCCAL_START_RXBQ = { 0x05C2, 7, 7, 0, "DCCAL_START_RXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_START_RXBI = { 0x05C2, 6, 6, 0, "DCCAL_START_RXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_START_RXAQ = { 0x05C2, 5, 5, 0, "DCCAL_START_RXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_START_RXAI = { 0x05C2, 4, 4, 0, "DCCAL_START_RXAI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_START_TXBQ = { 0x05C2, 3, 3, 0, "DCCAL_START_TXBQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_START_TXBI = { 0x05C2, 2, 2, 0, "DCCAL_START_TXBI", ""};
+static const struct LMS7Parameter LMS7_DCCAL_START_TXAQ = { 0x05C2, 1, 1, 0, "DCCAL_START_TXAQ", ""};
+static const struct LMS7Parameter LMS7_DCCAL_START_TXAI = { 0x05C2, 0, 0, 0, "DCCAL_START_TXAI", ""};
+
+static const struct LMS7Parameter LMS7_DCWR_TXAI = { 0x05C3, 15, 15, 0, "DCWR_TXAI", ""};
+static const struct LMS7Parameter LMS7_DCRD_TXAI = { 0x05C3, 14, 14, 0, "DCRD_TXAI", ""};
+static const struct LMS7Parameter LMS7_DC_TXAI = { 0x05C3, 10, 0, 0, "DC_TXAI", ""};
+static const struct LMS7Parameter LMS7_DCWR_TXAQ = { 0x05C4, 15, 15, 0, "DCWR_TXAQ", ""};
+static const struct LMS7Parameter LMS7_DCRD_TXAQ = { 0x05C4, 14, 14, 0, "DCRD_TXAQ", ""};
+static const struct LMS7Parameter LMS7_DC_TXAQ = { 0x05C4, 10, 0, 0, "DC_TXAQ", ""};
+static const struct LMS7Parameter LMS7_DCWR_TXBI = { 0x05C5, 15, 15, 0, "DCWR_TXBI", ""};
+static const struct LMS7Parameter LMS7_DCRD_TXBI = { 0x05C5, 14, 14, 0, "DCRD_TXBI", ""};
+static const struct LMS7Parameter LMS7_DC_TXBI = { 0x05C5, 10, 0, 0, "DC_TXBI", ""};
+static const struct LMS7Parameter LMS7_DCWR_TXBQ = { 0x05C6, 15, 15, 0, "DCWR_TXBQ", ""};
+static const struct LMS7Parameter LMS7_DCRD_TXBQ = { 0x05C6, 14, 14, 0, "DCRD_TXBQ", ""};
+static const struct LMS7Parameter LMS7_DC_TXBQ = { 0x05C6, 10, 0, 0, "DC_TXBQ", ""};
+static const struct LMS7Parameter LMS7_DCWR_RXAI = { 0x05C7, 15, 15, 0, "DCWR_RXAI", ""};
+static const struct LMS7Parameter LMS7_DCRD_RXAI = { 0x05C7, 14, 14, 0, "DCRD_RXAI", ""};
+static const struct LMS7Parameter LMS7_DC_RXAI = { 0x05C7, 6, 0, 0, "DC_RXAI", ""};
+static const struct LMS7Parameter LMS7_DCWR_RXAQ = { 0x05C8, 15, 15, 0, "DCWR_RXAQ", ""};
+static const struct LMS7Parameter LMS7_DCRD_RXAQ = { 0x05C8, 14, 14, 0, "DCRD_RXAQ", ""};
+static const struct LMS7Parameter LMS7_DC_RXAQ = { 0x05C8, 6, 0, 0, "DC_RXAQ", ""};
+static const struct LMS7Parameter LMS7_DCWR_RXBI = { 0x05C9, 15, 15, 0, "DCWR_RXBI", ""};
+static const struct LMS7Parameter LMS7_DCRD_RXBI = { 0x05C9, 14, 14, 0, "DCRD_RXBI", ""};
+static const struct LMS7Parameter LMS7_DC_RXBI = { 0x05C9, 6, 0, 0, "DC_RXBI", ""};
+static const struct LMS7Parameter LMS7_DCWR_RXBQ = { 0x05CA, 15, 15, 0, "DCWR_RXBQ", ""};
+static const struct LMS7Parameter LMS7_DCRD_RXBQ = { 0x05CA, 14, 14, 0, "DCRD_RXBQ", ""};
+static const struct LMS7Parameter LMS7_DC_RXBQ = { 0x05CA, 6, 0, 0, "DC_RXBQ", ""};
+static const struct LMS7Parameter LMS7_DC_RXCDIV = { 0x05CB, 15, 8, 32, "DC_RXCDIV", "Clock division ratio for Rx DC calibration loop"};
+static const struct LMS7Parameter LMS7_DC_TXCDIV = { 0x05CB, 7, 0, 32, "DC_TXCDIV", "Clock division ratio for Tx DC calibration loop"};
+static const struct LMS7Parameter LMS7_HYSCMP_RXB = { 0x05CC, 11, 9, 0, "HYSCMP_RXB", "Comparator hysteresis control, RXB channel"};
+static const struct LMS7Parameter LMS7_HYSCMP_RXA = { 0x05CC, 8, 6, 0, "HYSCMP_RXA", "Comparator hysteresis control, RXA channel"};
+static const struct LMS7Parameter LMS7_HYSCMP_TXB = { 0x05CC, 5, 3, 0, "HYSCMP_TXB", "Comparator hysteresis control, TXB channel"};
+static const struct LMS7Parameter LMS7_HYSCMP_TXA = { 0x05CC, 2, 0, 0, "HYSCMP_TXA", "Comparator hysteresis control, TXA channel"};
+static const struct LMS7Parameter LMS7_DAC_CLKDIV = { 0x0600, 15, 8, 15, "DAC_CLKDIV", "Clock division ratio for measurement loop" };
+static const struct LMS7Parameter LMS7_RSSI_RSSIMODE = { 0x0600, 1, 1, 0, "RSSI_RSSIMODE", ""};
+static const struct LMS7Parameter LMS7_RSSI_PD = { 0x0600, 0, 0, 0, "RSSI_PD", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPSTATUS_TEMPREF = { 0x0601, 5, 5, 0, "INTADC_CMPSTATUS_TEMPREF", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPSTATUS_TEMPVPTAT = { 0x0601, 4, 4, 0, "INTADC_CMPSTATUS_TEMPVPTAT", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPSTATUS_RSSI2 = { 0x0601, 3, 3, 0, "INTADC_CMPSTATUS_RSSI2", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPSTATUS_RSSI1 = { 0x0601, 2, 2, 0, "INTADC_CMPSTATUS_RSSI1", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPSTATUS_PDET2 = { 0x0601, 1, 1, 0, "INTADC_CMPSTATUS_PDET2", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPSTATUS_PDET1 = { 0x0601, 0, 0, 0, "INTADC_CMPSTATUS_PDET1", ""};
+static const struct LMS7Parameter LMS7_RSSI_BIAS = { 0x0602, 13, 9, 1<<4, "RSSI_BIAS", "Controls the reference bias current of the test ADC"};
+static const struct LMS7Parameter LMS7_RSSI_HYSCMP = { 0x0602, 8, 6, 0, "RSSI_HYSCMP", "Comparator hysteresis control"};
+static const struct LMS7Parameter LMS7_INTADC_CMPCFG_TEMPREF = { 0x0602, 5, 5, 0, "INTADC_CMPCFG_TEMPREF", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPCFG_TEMPVPTAT = { 0x0602, 4, 4, 0, "INTADC_CMPCFG_TEMPVPTAT", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPCFG_RSSI2 = { 0x0602, 3, 3, 0, "INTADC_CMPCFG_RSSI2", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPCFG_RSSI1 = { 0x0602, 2, 2, 0, "INTADC_CMPCFG_RSSI1", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPCFG_PDET2 = { 0x0602, 1, 1, 0, "INTADC_CMPCFG_PDET2", ""};
+static const struct LMS7Parameter LMS7_INTADC_CMPCFG_PDET1 = { 0x0602, 0, 0, 0, "INTADC_CMPCFG_PDET1", ""};
+static const struct LMS7Parameter LMS7_RSSI_DAC_VAL = { 0x0603, 7, 0, 0, "RSSI_DAC_VAL", "Stores the value to be written to the DAC, when MODE = 1"};
+static const struct LMS7Parameter LMS7_RSSI_PDET2_VAL = { 0x0604, 15, 8, 0, "RSSI_PDET2_VAL", "Stores the value of Power Detector 2"};
+static const struct LMS7Parameter LMS7_RSSI_PDET1_VAL = { 0x0604, 7, 0, 0, "RSSI_PDET1_VAL", "Stores the value of Power Detector 1"};
+static const struct LMS7Parameter LMS7_RSSI_RSSI2_VAL = { 0x0605, 15, 8, 0, "RSSI_RSSI2_VAL", "Stores the value of RSSI 2"};
+static const struct LMS7Parameter LMS7_RSSI_RSSI1_VAL = { 0x0605, 7, 0, 0, "RSSI_RSSI1_VAL", "Stores the value of RSSI 1"};
+static const struct LMS7Parameter LMS7_RSSI_TREF_VAL = { 0x0606, 15, 8, 0, "RSSI_TREF_CAL", "Stores the temperature reference value"};
+static const struct LMS7Parameter LMS7_RSSI_TVPTAT_VAL = { 0x0606, 7, 0, 0, "RSSI_TVPTAT_VAL", "Stores the voltage proportional to absolute temperature value"};
+static const struct LMS7Parameter LMS7_RSSIDC_CMPSTATUS = { 0x0640, 15, 15, 0, "RSSIDC_CMPSTATUS", ""};
+static const struct LMS7Parameter LMS7_RSSIDC_RSEL = { 0x0640, 8, 4, 10, "RSSIDC_RSEL", "Reference voltage for the RSSI output comparator"};
+static const struct LMS7Parameter LMS7_RSSIDC_HYSCMP = { 0x0640, 3, 1, 0, "RSSIDC_HYSCMP", ""};
+static const struct LMS7Parameter LMS7_RSSIDC_PD = { 0x0640, 0, 0, 0, "RSSIDC_PD", "Power down modules"};
+static const struct LMS7Parameter LMS7_RSSIDC_DCO2 = { 0x0641, 13, 7, 32, "RSSIDC_DCO2", "Value of RSSI offset DAC2"};
+static const struct LMS7Parameter LMS7_RSSIDC_DCO1 = { 0x0641, 6, 0, 32, "RSSIDC_DCO1", "Value of RSSI offset DAC1"};
+static const struct LMS7Parameter LMS7_DCLOOP_STOP = { 0x040C, 8, 8, 0, "DCLOOP_STOP", "Stops RxDC tracking loop" };
+
 //}
 
 #ifdef __cplusplus
