@@ -34,7 +34,7 @@ static int convertStatus(const int &status, const LMS64CProtocol::GenericPacket
     {
     case STATUS_COMPLETED_CMD: return 0;
     case STATUS_UNKNOWN_CMD:
-        return ReportError(EPROTONOSUPPORT, "unknown lms64c protocol command");
+        return ReportError(EPROTONOSUPPORT, "Command not supported");
     default: break;
     }
     return ReportError(EPROTO, status2string(pkt.status));
@@ -145,10 +145,11 @@ int LMS64CProtocol::SetReferenceClockRate(const double rate)
 /***********************************************************************
  * LMS7002M SPI access
  **********************************************************************/
-int LMS64CProtocol::WriteLMS7002MSPI(const uint32_t *writeData, const size_t size)
+int LMS64CProtocol::WriteLMS7002MSPI(const uint32_t *writeData, size_t size, unsigned periphID)
 {
     GenericPacket pkt;
     pkt.cmd = CMD_LMS7002_WR;
+    pkt.periphID = periphID;
     for (size_t i = 0; i < size; ++i)
     {
         uint16_t addr = (writeData[i] >> 16) & 0x7fff;
@@ -164,10 +165,11 @@ int LMS64CProtocol::WriteLMS7002MSPI(const uint32_t *writeData, const size_t siz
     return convertStatus(status, pkt);
 }
 
-int LMS64CProtocol::ReadLMS7002MSPI(const uint32_t *writeData, uint32_t *readData, const size_t size)
+int LMS64CProtocol::ReadLMS7002MSPI(const uint32_t *writeData, uint32_t *readData, size_t size, unsigned periphID)
 {
     GenericPacket pkt;
     pkt.cmd = CMD_LMS7002_RD;
+    pkt.periphID = periphID;
     for (size_t i = 0; i < size; ++i)
     {
         uint16_t addr = (writeData[i] >> 16) & 0x7fff;
@@ -491,9 +493,9 @@ int LMS64CProtocol::TransferPacket(GenericPacket& pkt)
         }
         ParsePacket(pkt, inBuffer, inDataPos, protocol);
     }
-    delete outBuffer;
+    delete[] outBuffer;
     delete[] inBuffer;
-    return status;
+    return convertStatus(status, pkt);
 }
 
 /** @brief Takes generic packet and converts to specific protocol buffer
@@ -514,6 +516,7 @@ unsigned char* LMS64CProtocol::PreparePacket(const GenericPacket& pkt, int& leng
         int maxDataLength = packet.maxDataLength;
         packet.cmd = pkt.cmd;
         packet.status = pkt.status;
+        packet.periphID = pkt.periphID;
         int byteBlockRatio = 1; //block ratio - how many bytes in one block
         switch( packet.cmd )
         {
@@ -561,6 +564,7 @@ unsigned char* LMS64CProtocol::PreparePacket(const GenericPacket& pkt, int& leng
             int pktPos = j*packet.pktLength;
             buffer[pktPos] = packet.cmd;
             buffer[pktPos+1] = packet.status;
+            buffer[pktPos + 3] = packet.periphID;
             if(blockCount > (maxDataLength/byteBlockRatio))
             {
                 buffer[pktPos+2] = maxDataLength/byteBlockRatio;
@@ -568,7 +572,7 @@ unsigned char* LMS64CProtocol::PreparePacket(const GenericPacket& pkt, int& leng
             }
             else
                 buffer[pktPos+2] = blockCount;
-            memcpy(&buffer[pktPos+3], packet.reserved, sizeof(packet.reserved));
+            memcpy(&buffer[pktPos+4], packet.reserved, sizeof(packet.reserved));
             int bytesToPack = (maxDataLength/byteBlockRatio)*byteBlockRatio;
             for (int k = 0; k<bytesToPack && srcPos < pkt.outBuffer.size(); ++srcPos, ++k)
                 buffer[pktPos + 8 + k] = pkt.outBuffer[srcPos];
@@ -788,19 +792,25 @@ int LMS64CProtocol::CustomParameterRead(const uint8_t *ids, double *values, cons
         pkt.outBuffer.push_back(ids[i]);
 
     int status = this->TransferPacket(pkt);
-    if (status != 0) return convertStatus(status, pkt);
+    if (status != 0) return status;
 
     assert(pkt.inBuffer.size() >= 4 * count);
 
     for (size_t i = 0; i < count; ++i)
     {
-        int unitsIndex = (pkt.inBuffer[i * 4 + 1] & 0xF0) >> 4;
+        int unitsIndex = pkt.inBuffer[i * 4 + 1];
         if(units)
-            units[i] = adcUnits2string(unitsIndex);
-        values[i] = pkt.inBuffer[i * 4 + 2] << 8 | pkt.inBuffer[i * 4 + 3];
-        int powerOf10 = pkt.inBuffer[i * 4 + 1] & 0x0F;
-        values[i] *= pow(10, powerOf10);
-        if(unitsIndex == TEMPERATURE)
+        {
+
+            const char adc_units_prefix[] = {
+                ' ', 'k', 'M', 'G', 'T', 'P', 'E', 'Z',
+                'y', 'z', 'a', 'f', 'p', 'n', 'u', 'm'};
+            units[i] = adc_units_prefix[unitsIndex&0x0F];
+            units[i] += adcUnits2string((unitsIndex & 0xF0)>>4);
+        }
+        values[i] = (int16_t)(pkt.inBuffer[i * 4 + 2] << 8 | pkt.inBuffer[i * 4 + 3]);
+
+        if((unitsIndex & 0xF0)>>4 == TEMPERATURE)
             values[i] /= 10;
     }
     return 0;
@@ -823,8 +833,7 @@ int LMS64CProtocol::CustomParameterWrite(const uint8_t *ids, const double *value
         pkt.outBuffer.push_back(value >> 8);
         pkt.outBuffer.push_back(value & 0xFF);
     }
-    int status = this->TransferPacket(pkt);
-    return convertStatus(status, pkt);
+    return TransferPacket(pkt);
 }
 
 int LMS64CProtocol::GPIOWrite(const uint8_t *buffer, const size_t bufLength)
@@ -833,8 +842,7 @@ int LMS64CProtocol::GPIOWrite(const uint8_t *buffer, const size_t bufLength)
     pkt.cmd = CMD_GPIO_WR;
     for (size_t i=0; i<bufLength; ++i)
         pkt.outBuffer.push_back(buffer[i]);
-    int status = TransferPacket(pkt);
-    return convertStatus(status, pkt);
+    return TransferPacket(pkt);
 }
 
 int LMS64CProtocol::GPIORead(uint8_t *buffer, const size_t bufLength)
@@ -842,20 +850,38 @@ int LMS64CProtocol::GPIORead(uint8_t *buffer, const size_t bufLength)
     LMS64CProtocol::GenericPacket pkt;
     pkt.cmd = CMD_GPIO_RD;
     int status = TransferPacket(pkt);
+    if(status != 0)
+        return status;
+
+    for (size_t i=0; i<bufLength; ++i)
+        buffer[i] = pkt.inBuffer[i];
+    return status;
+}
+
+int LMS64CProtocol::GPIODirWrite(const uint8_t *buffer, const size_t bufLength)
+{
+    LMS64CProtocol::GenericPacket pkt;
+    pkt.cmd = CMD_GPIO_DIR_WR;
+    for (size_t i=0; i<bufLength; ++i)
+        pkt.outBuffer.push_back(buffer[i]);
+    return TransferPacket(pkt);
+}
+
+int LMS64CProtocol::GPIODirRead(uint8_t *buffer, const size_t bufLength)
+{
+    LMS64CProtocol::GenericPacket pkt;
+    pkt.cmd = CMD_GPIO_DIR_RD;
+    int status = TransferPacket(pkt);
     if(status != 0)
         return convertStatus(status, pkt);
 
     for (size_t i=0; i<bufLength; ++i)
         buffer[i] = pkt.inBuffer[i];
-    return convertStatus(status, pkt);
+    return status;
 }
 
 int LMS64CProtocol::ProgramMCU(const uint8_t *buffer, const size_t length, const MCU_PROG_MODE mode, ProgrammingCallback callback)
 {
-    LMSinfo lmsInfo = this->GetInfo();
-    if(lmsInfo.device == LMS_DEV_LIMESDR)
-        return IConnection::ProgramMCU(buffer, length, mode, callback);
-
 #ifndef NDEBUG
     auto timeStart = std::chrono::high_resolution_clock::now();
 #endif
@@ -911,3 +937,18 @@ int LMS64CProtocol::ProgramMCU(const uint8_t *buffer, const size_t length, const
 #endif
     return success ? 0 : -1;
 }
+
+/**	@brief Reads chip version information form LMS7 chip.
+*/
+int LMS64CProtocol::GetChipVersion()
+{
+    LMS64CProtocol::GenericPacket ctrPkt;
+    ctrPkt.cmd = CMD_LMS7002_RD;
+    ctrPkt.outBuffer.push_back(0x00); //reset bulk endpoints
+    ctrPkt.outBuffer.push_back(0x2F); //reset bulk endpoints
+    if(TransferPacket(ctrPkt) != 0)
+        this->chipVersion = 0;
+    else
+        this->chipVersion=(ctrPkt.inBuffer[2]<<8)|ctrPkt.inBuffer[3];
+    return this->chipVersion;
+}
